<?php
/**
 *
 *
 * @package qa_heatmap
 */

new QAHM_Admin_Common();

// 管理画面全体で動作するクラス
class QAHM_Admin_Common extends QAHM_Base {

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		// アドミンバー
		//add_action( 'admin_bar_menu', array( $this, 'customize_admin_bar_menu' ), 9999 );

		// RSSフィードのキャッシュを3時間に変更
		add_filter( 'wp_feed_cache_transient_lifetime', array( $this, 'set_feed_cache_time' ) );

		// 計測フラグ催促
		add_action( 'admin_notices', array( $this, 'remind_rec_flag' ) );
	}

	/**
	 * アドミンバーにQAHMアイコンを追加
	 */
	public function customize_admin_bar_menu( $wp_admin_bar ) {
		if ( ! is_user_logged_in() || is_admin() || ! $this->is_qahm_page() ) {
			return;
		}

		$rec          = '';
		$rec_flag     = false;
		$is_raw_save_all = $this->wrap_get_option( 'is_raw_save_all' );

		// 計測フラグ判定
		$id   = -1;
		$type = '';
		if ( is_home() || is_front_page() ) {
			$type = 'home';
			$id   = 1;
		} elseif ( is_page() ) {
			$type = 'page_id';
			$id   = get_the_id();
		} elseif ( is_single() ) {
			$type = 'p';
			$id   = get_the_id();
		} elseif ( is_category() ) {
			$type = 'cat';
			$id   = get_query_var( 'cat' );
		} elseif ( is_tag() ) {
			$type = 'tag';
			$id   = get_query_var( 'tag_id' );
		} elseif ( is_tax() ) {
			$type = 'tax';
			$id   = get_queried_object_id();
		}

		if( $id !== -1 && $type ) {
			$title =
				'<span class="ab-item">'
					. '<span class="ab-icon">'
						. '<img src="' . plugin_dir_url( __FILE__ ) . 'img/quokka-icon.png">'
					. '</span>'
					. '<span class="ab-label">' . QAHM_PLUGIN_NAME . $rec . '</span>'
				. '</span>';

			$wp_admin_bar->add_menu(
				array(
					'id'    => QAHM_NAME,
					'meta'  => array(),
					'title' => $title,
				)
			);
		}
	}

	/**
	 * RSSフィードのキャッシュを3時間に変更
	 */
	public function set_feed_cache_time() {
		return 60 * 60 * 3;
	}

	/**
	 * 計測フラグ催促
	 */
	public function remind_rec_flag() {
		global $qahm_recterm;
		$rts      = $qahm_recterm->get_all_record();
		$rec_flag = $this->wrap_get_option( 'is_raw_save_all' );
		if ( $rts && ! $rec_flag ) {
			foreach ( $rts as $rt ) {
				if ( $rt['rec_flag'] ) {
					$rec_flag = true;
					break;
				}
			}
		}

		if ( ! $rec_flag ) {
			echo '<div class="message error"><p>';
			printf(
				__( 'The measurement flag of QA Heatmap Analytics is not set. For the setting<a href="%s"> click here </a>', 'qa-heatmap-analytics' ),
				'admin.php?page=' . QAHM_Admin_Page_Heatmap::SLUG
			);
			echo '</p></div>';
		}
	}
} // end of class
