<?php
/**
 * 
 *
 * @package qa_heatmap
 */

$qahm_admin_page_config = new QAHM_Admin_Page_Config();

class QAHM_Admin_Page_Config extends QAHM_Admin_Page_Base {

	// スラッグ
	const SLUG = QAHM_NAME . '-config';

	// nonce
	const NONCE_ACTION = self::SLUG . '-nonce-action';
	const NONCE_NAME   = self::SLUG . '-nonce-name';

	private static $error_msg = array();

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		parent::__construct();

		 // コールバック
		add_action( 'init', array( $this, 'init_wp_filesystem' ) );
		add_action( 'admin_init', array( $this, 'save_config' ) );
	}
	
	/**
	 * 初期化
	 */
	public function enqueue_scripts( $hook_suffix ) {
		if( $this->hook_suffix !== $hook_suffix ||
			! $this->is_enqueue_jquery()
		) {
			return;
		}

		global $qahm_time;
		$js_dir       = $this->get_js_dir_url();
		$data_dir     = $this->get_data_dir_url();
		$license_plan = $this->wrap_get_option( 'license_plan' );

		// enqueue_style
		$this->common_enqueue_style();
		wp_enqueue_script( QAHM_NAME . '-admin-page-config', $js_dir . 'admin-page-config.js', array( QAHM_NAME . '-admin-common' ), QAHM_PLUGIN_VERSION );
		if ( QAHM_License::PLAN['personal'] <= $license_plan ) {
			wp_enqueue_script( QAHM_NAME . '-powerup-config', $data_dir . 'js/powerup-config.js', array( QAHM_NAME . '-admin-common' ), $qahm_time->now_unixtime() );
		}

		// enqueue script
		$this->common_enqueue_script();
		
		// inline script
		$scripts = $this->get_common_inline_script();
		$scripts['access_role']     = $this->wrap_get_option( 'access_role' );
		$scripts['is_raw_save_all'] = $this->wrap_get_option( 'is_raw_save_all' );
		$scripts['data_save_month'] = $this->wrap_get_option( 'data_save_month' );
		wp_add_inline_script( QAHM_NAME . '-common', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', obj );', 'before' );

		// localize
		$localize = $this->get_common_localize_script();
		$localize['data_save_month_title']     = __( 'Data Storage Period (month(s))', 'qa-heatmap-analytics' );
		$localize['data_save_month_desc']      = __( '* If your database capacity is less than 1GB, it is safer to store for about 2 months. If you don\'t know the database capacity, to have it set for 2 months or less would be suggested.', 'qa-heatmap-analytics' );
		$localize['all_page_measure_title']    = __( 'Measure all pages', 'qa-heatmap-analytics' );
		$localize['all_page_measure_desc']     = __( 'Ignore the individual measurement settings on Heatmap management screen and measure all pages. Measurement starts automatically including when a new page is added (published).', 'qa-heatmap-analytics' );
		$localize['access_role_title']         = __( 'Access Privileges', 'qa-heatmap-analytics' );
		$localize['access_role_desc']          = __( 'Set the privileges to access the heatmap view.', 'qa-heatmap-analytics' );
		$localize['access_role_option_admin']  = __( 'Administrator only', 'qa-heatmap-analytics' );
		$localize['access_role_option_editor'] = __( 'Editor or above', 'qa-heatmap-analytics' );
		wp_localize_script( QAHM_NAME . '-common', QAHM_NAME . 'l10n', $localize );
	}

	/**
	 * ページの表示
	 */
	public function create_html() {
		// データを取得
		$license_plan = $this->wrap_get_option( 'license_plan' );
		?>

		<div id="<?php echo esc_attr( basename( __FILE__, '.php' ) ); ?>" class="qahm-admin-page">
			<div class="wrap">
				<form action="" method="post">
					<?php // nonceの設定 ?>
					<?php wp_nonce_field( self::NONCE_ACTION, self::NONCE_NAME ); ?>

					<h1>QA <?php _e( 'Settings', 'qa-heatmap-analytics' ); ?></h1>
					<?php $this->view_announce_html(); ?>

					<?php
						// メッセージ表示
					if ( ! empty( self::$error_msg ) ) {
						foreach ( self::$error_msg as $msg ) {
							echo '<div class="error notice is-dismissible"><p><strong>' . __( 'Error', 'qa-heatmap-analytics' ) . ' : ' . esc_html( $msg ) . '</strong></p></div>';
						}
					} elseif ( isset( $_POST[ self::NONCE_NAME ] ) ) {
						echo '<div class="updated notice is-dismissible"><p><strong>' . __( 'Saved settings.', 'qa-heatmap-analytics' ) . '</strong></p></div>';
					}

					// プランによって変更
					$disabled = '';
					if ( QAHM_License::PLAN['personal'] <= $license_plan ) {
						echo '<table id="config-table" class="form-table">' .
							'<tbody>' .
							'</tbody>' .
						'</table>';
					} else {
						$img = $this->get_img_dir_url() . 'config-pro.png';
						echo '<img src="' . $img . '" style="display: inline-block; margin: 16px 0;">';
						$disabled = ' disabled';
					}
					?>


					<!--
							<tr>
								<th scope="row"><label for="data_save_pv">データ保存上限</label></th>
								<td><input type="number" id="data_save_pv" name="data_save_pv" min="10000" max="10000000" step="1" value="<?php //echo esc_attr( $data_save_pv ); ?>">
								<p class="description">説明文</p></td>
							</tr>
					-->

					<p class="submit">
						<input type="submit" name="submit" id="submit" value="<?php _e( 'Save Changes', 'qa-heatmap-analytics' ); ?>" class="button button-primary"<?php echo $disabled; ?>>
					</p>

				</form>
			</div>
		</div>
		<?php
	}

	/**
	 * 設定画面の項目をデータベースに保存する
	 */
	public function save_config() {
		$post = array();
		// nonceで設定したcredentialのチェック
		// 設定画面
		if ( isset( $_POST[ self::NONCE_NAME ] ) ) {
			if ( check_admin_referer( self::NONCE_ACTION, self::NONCE_NAME ) ) {
				$license_plan    = $this->wrap_get_option( 'license_plan' );

				if ( QAHM_License::PLAN['personal'] <= $license_plan ) {
					$data_save_month = filter_input( INPUT_POST, 'data_save_month' );
					//$data_save_pv    = filter_input( INPUT_POST, 'data_save_pv' );
	
					// データ保存＆エラーメッセージを登録
					if ( $data_save_month < 1 ) {
						self::$error_msg['data_save_month'] = __( 'Set the data storage period to 1 or more.', 'qa-heatmap-analytics' );
					} elseif ( ! is_numeric( $data_save_month ) ) {
						self::$error_msg['data_save_month'] = __( 'Enter only a numerical value for the data strage period.', 'qa-heatmap-analytics' );
					} else {
						$this->wrap_update_option( 'data_save_month', $data_save_month );
					}
	
					/*
					if ( $data_save_pv < 1 ) {
						self::$error_msg['data_save_pv'] = 'データ保存上限は１日以上にしてください。';
					} elseif ( ! is_numeric( $data_save_pv ) ) {
						self::$error_msg['data_save_pv'] = 'データ保存上限は数値のみ入力してください。';
					} else {
						$this->wrap_update_option( 'data_save_pv', $data_save_pv );
					}
					*/

					$is_raw_save_all = filter_input( INPUT_POST, 'is_raw_save_all' );
					$access_role     = filter_input( INPUT_POST, 'access_role' );
					
					$this->wrap_update_option( 'is_raw_save_all', $is_raw_save_all );

					if ( $access_role !== null ) {
						$this->wrap_update_option( 'access_role', $access_role );
					}
				}
			}
		}
	}
} // end of class
