<?php
/**
 *
 *
 * @package qa_heatmap
 */

$qahm_admin_page_heatmap = new QAHM_Admin_Page_Heatmap();

class QAHM_Admin_Page_Heatmap extends QAHM_Admin_Page_Base {

	// スラッグ
	const SLUG = QAHM_NAME . '-heatmap';

	// nonce
	const NONCE_ACTION  = self::SLUG . '-nonce-action';
	const NONCE_NAME    = self::SLUG . '-nonce-name';
	const NONCE_REFRESH = 'refresh';

	private static $error_msg = array();

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		parent::__construct();

		$this->regist_ajax_func( 'ajax_refresh_version' );
		$this->regist_ajax_func( 'ajax_create_heatmap_list' );
		$this->regist_ajax_func( 'ajax_set_achievements' );
		$this->regist_ajax_func( 'ajax_change_is_raw_save_all' );
		$this->regist_ajax_func( 'ajax_get_data_num' );

		add_action( 'init', array( $this, 'init_wp_filesystem' ) );
		add_action( 'admin_init', array( $this, 'save_config' ) );
	}

	/**
	 * 初期化
	 */
	public function enqueue_scripts( $hook_suffix ) {
		if( $this->hook_suffix !== $hook_suffix ||
			! $this->is_enqueue_jquery()
		) {
			return;
		}

		$css_dir_url = $this->get_css_dir_url();
		$js_dir_url  = $this->get_js_dir_url();

		// enqueue_style
		$this->common_enqueue_style();
		wp_enqueue_style( QAHM_NAME . '-admin-page-heatmap', $css_dir_url . 'admin-page-heatmap.css', array( QAHM_NAME . '-reset' ), QAHM_PLUGIN_VERSION );	
	
		// enqueue script
		$this->common_enqueue_script();
		wp_enqueue_script( QAHM_NAME . '-cap-create', $js_dir_url . 'cap-create.js', array( QAHM_NAME . '-admin-common' ), QAHM_PLUGIN_VERSION );
		wp_enqueue_script( QAHM_NAME . '-admin-page-heatmap', $js_dir_url . 'admin-page-heatmap.js', array( QAHM_NAME . '-admin-common' ), QAHM_PLUGIN_VERSION );
	
		// inline script
		$scripts = $this->get_common_inline_script();
		$scripts['heatmap_measure_max']      = $this->wrap_get_option( 'heatmap_measure_max' );
		$scripts['is_raw_save_all']          = $this->wrap_get_option( 'is_raw_save_all' );
		$scripts['achievements']             = $this->wrap_get_option( 'achievements' );
		$scripts['nonce_refresh']            = wp_create_nonce( self::NONCE_REFRESH );
		$scripts['is_first_heatmap_setting'] = $this->wrap_get_option( 'is_first_heatmap_setting' );
		wp_add_inline_script( QAHM_NAME . '-common', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', obj );', 'before' );

		// localize
		$localize = $this->get_common_localize_script();
		$localize['home']                             = __( 'Home', 'qa-heatmap-analytics' );
		$localize['post']                             = __( 'Post', 'qa-heatmap-analytics' );
		$localize['page']                             = __( 'Page', 'qa-heatmap-analytics' );
		$localize['cat']                              = __( 'Category', 'qa-heatmap-analytics' );
		$localize['tag']                              = __( 'Tag', 'qa-heatmap-analytics' );
		$localize['tax']                              = __( 'Custom taxonomy', 'qa-heatmap-analytics' );
		$localize['ver']                              = __( 'ver.', 'qa-heatmap-analytics' );
		$localize['limit_measure_title']              = __( 'About measurement', 'qa-heatmap-analytics' );
		$localize['limit_measure_text']               = __( 'With your current license, only %d pages can be measured. <br>', 'qa-heatmap-analytics' );
		$localize['limit_free_text']                  = __( 'You can get the license for free. <br> For more information, <a href="https://quarka.org/friends-progoram/" target="_blank" rel="noopener"> click here </a>.', 'qa-heatmap-analytics' );
		$localize['limit_friend_text']                = __( 'To see the details of the license, <a href="https://quarka.org/plan/" target="_blank" rel="noopener"> click here </a>.', 'qa-heatmap-analytics' );
		$localize['switch']                           = __( 'switched', 'qa-heatmap-analytics' );
		$localize['version_switch']                   = __( 'Switch the Page Version', 'qa-heatmap-analytics' );
		$localize['version_switch_error_text']        = __( 'Failed to switch the version.<br>Please try again aftertime.', 'qa-heatmap-analytics' );
		$localize['version_switch_start_text_1']      = __( 'Save the current page version and start a new measurement.', 'qa-heatmap-analytics' );
		$localize['version_switch_start_text_2']      = __( 'Use this after you edited the page or changed the CSS.', 'qa-heatmap-analytics' );
		$localize['achievements_data_num_1_title']    = __( 'Now on taking the first step!', 'qa-heatmap-analytics' );
		$localize['achievements_data_num_1_text']     = __( 'Congratulations! <br> Data has been collected for the first time.', 'qa-heatmap-analytics' );
		$localize['achievements_data_num_1000_title'] = __( 'Improved analytical skills!', 'qa-heatmap-analytics' );
		$localize['achievements_data_num_1000_text']  = __( 'In total, %d data have been collected in QA!', 'qa-heatmap-analytics' );
		$localize['measure_comment_text']             = __( 'The maximum number of mesurable pages has been reached. <br> The more you introduce QA Heatmap Analytics to your friends, the more pages you get to measure. Moreover, the number of mesurable pages for your introduced friends will also be raised!', 'qa-heatmap-analytics' );
		wp_localize_script( QAHM_NAME . '-common', QAHM_NAME . 'l10n', $localize );
	}


	/**
	 * ページの表示
	 */
	public function create_html() {
		if( ! $this->is_enqueue_jquery() ) {
			$this->view_not_enqueue_jquery_html();
			return;
		}
		
		if( $this->is_maintenance() ) {
			$this->view_maintenance_html();
			return;
		}

		// データを取得
		$is_raw_save_all   = $this->wrap_get_option( 'is_raw_save_all' );
		$heatmap_sort_rec  = $this->wrap_get_option( 'heatmap_sort_rec' );
		$heatmap_sort_view = $this->wrap_get_option( 'heatmap_sort_view' );
		?>

		<div id="<?php echo esc_attr( basename( __FILE__, '.php' ) ); ?>" class="qahm-admin-page">
			<div class="wrap">
				<h1>QA <?php _e( 'Heatmap Manager', 'qa-heatmap-analytics' ); ?></h1>
				
				<?php
					// メッセージ表示
					if ( ! empty( self::$error_msg ) ) {
						foreach ( self::$error_msg as $msg ) {
							echo '<div class="error notice is-dismissible"><p><strong>' . __( 'Error', 'qa-heatmap-analytics' ) . ' : ' . esc_html( $msg ) . '</strong></p></div>';
						}
					} elseif ( isset( $_POST[ self::NONCE_NAME ] ) ) {
						echo '<div class="updated notice is-dismissible"><p><strong>' . __( 'Saved settings.', 'qa-heatmap-analytics' ) . '</strong></p></div>';
					}
					$this->view_rss_feed();
				?>


				<div id="qahm-measure-comment-container" class="qahm-announce-container" style="display: none;">
					<div id="qahm-measure-comment-icon" class="qahm-announce-icon"><svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 130 126"><defs><style>.cls-1{fill:#00709e;}.cls-2{fill:none;}</style></defs><path class="cls-1" d="M312.85,477.41h-1.46a2.28,2.28,0,1,1,0-4.55h.66c20.14.07,29.47-4.91,33.75-9.16a17.25,17.25,0,0,0,5.09-12.9,21.7,21.7,0,0,0-8-16.54,2.28,2.28,0,0,1-.94-1.79c-.44-20.23-9.06-33-9.15-33.14a2.3,2.3,0,0,1-.19-2.24c0-.08,3.52-7.67,4-12.64.55-6,.52-16.44-5.89-18.84l-.11-.05c-2.09-.92-8.47,5.79-11.9,9.4-5.25,5.52-8.63,8.9-11.75,8.9a2.23,2.23,0,0,1-1-.22,18.27,18.27,0,0,0-14,0,2.23,2.23,0,0,1-1,.22c-3.12,0-6.5-3.38-11.75-8.9-3.43-3.6-9.81-10.32-11.9-9.4l-.11.05c-6.41,2.4-6.44,12.83-5.89,18.84.47,5,3.95,12.56,4,12.64a2.29,2.29,0,0,1-.19,2.25c-.09.12-8.71,12.9-9.15,33.13a2.29,2.29,0,0,1-1,1.81,21,21,0,0,0-7.95,16.52,17.26,17.26,0,0,0,5.1,12.9c4.21,4.2,13.36,9.16,33,9.16h1.44a2.28,2.28,0,0,1,0,4.55H286c-17.4.07-29.87-3.41-37-10.47a21.9,21.9,0,0,1-6.44-16.13,25.62,25.62,0,0,1,9-19.5c.62-17.83,7-29.86,9.22-33.45-1.07-2.48-3.46-8.44-3.88-13-1.16-12.56,1.95-20.9,8.76-23.48,5-2.13,10.8,4,16.95,10.44,2.62,2.76,6.5,6.84,8.17,7.43a22.95,22.95,0,0,1,16.53,0c1.65-.59,5.53-4.67,8.16-7.43,6.15-6.46,12-12.57,17-10.44,6.8,2.58,9.91,10.92,8.75,23.48-.42,4.55-2.81,10.51-3.87,13,2.17,3.59,8.59,15.62,9.22,33.47a26.11,26.11,0,0,1,8.95,19.48A21.9,21.9,0,0,1,349,466.94c-7,6.95-19.16,10.47-36.15,10.47Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M270.59,398.56c-1.22-.44-3.72-7.75-4.85-12.43-1.28-5.61-1.88-12.75,2.52-14.88a3.23,3.23,0,0,1,1.4-.34c2,0,3.48,2.14,4.95,4.21.35.49.7,1,1.07,1.47a29.44,29.44,0,0,0,5.75,5.19c1.88,1.38,2.44,1.94,2,2.57-.14.21-.44.33-1.38.7a23.45,23.45,0,0,0-7.12,4c-2.86,2.45-3.51,6.37-3.83,8.25-.13.83-.18,1.06-.32,1.2a.48.48,0,0,1-.23.1Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M327.41,398.56a.4.4,0,0,1-.23-.1c-.14-.14-.19-.37-.32-1.2-.32-1.89-1-5.81-3.83-8.25a23.41,23.41,0,0,0-7.1-4c-1-.39-1.27-.5-1.4-.7-.4-.63.16-1.2,2-2.57a29.52,29.52,0,0,0,5.76-5.2c.36-.47.71-1,1.06-1.45,1.47-2.08,3-4.22,4.95-4.22a3.16,3.16,0,0,1,1.39.34c4.42,2.13,3.81,9.3,2.52,14.94-1.59,6.37-3.8,12-4.84,12.37Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M278.77,412.57s-1.53,4.08,0,5.92a7.45,7.45,0,0,0,5.24,2,8.57,8.57,0,1,1-5.25-7.93Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M325.93,412.57s-1.53,4.08,0,5.92a7.47,7.47,0,0,0,5.25,2,8.59,8.59,0,1,1-5.26-7.93Z" transform="translate(-233 -357.5)"/><path class="cls-1" d="M299,437.87c-5.84,0-10.58,3-10.58,6.63S293.15,455,299,455s10.58-6.85,10.58-10.51S304.83,437.87,299,437.87Z" transform="translate(-233 -357.5)"/><rect id="_スライス_" data-name="&lt;スライス&gt;" class="cls-2" width="130" height="126"/></svg></div>
					<div id="qahm-measure-comment-text" class="qahm-announce-text"><?php _e( 'By default, the heatmap measuring page is set to Home. <br>Select the checkbox on the column of the page you want to measure.', 'qa-heatmap-analytics' ); ?></div>
					<div id="qahm-measure-comment-button" class="qahm-announce-button" style="display: none;">
						<a href="https://quarka.org/friends-progoram/" target="_blank" rel="noopener">
							<?php _e( 'Introduce QA Heatmap to your friend <br>and get more measureable pages.', 'qa-heatmap-analytics' ); ?>
						</a>
					</div>
				</div>
				<?php $this->view_announce_html(); ?>

				<h2><?php _e( 'Data and Measurement Settings of Your Heatmap', 'qa-heatmap-analytics' ); ?></h2>
				<p>
					<?php
						_e( 'Select a checkbox on the column of the page which you want to measure heatmap data and save the changes.* <br>*You can view the heatmap data from the day after the measurement is enabled.', 'qa-heatmap-analytics' );
					?>
				</p>

				<form action="" method="post" id="qahm-data-sort-form" onsubmit="qahm.createList(); return false;">
					<div class="tablenav top">
						<div class="alignleft">
							<input type="text" id="qahm-data-sort-title" placeholder="<?php _e( 'Title to search', 'qa-heatmap-analytics' ); ?>" class="regular-text">
						</div>

						<select id="qahm-data-sort-rec">
							<option value="all" 
								<?php
								if ( $heatmap_sort_rec === 'all' ) {
									echo ' selected';
								}
								?>
							>
								<?php _e( 'All pages', 'qa-heatmap-analytics' ); ?>
							</option>

							<option value="true" 
								<?php
								if ( $heatmap_sort_rec === 'true' ) {
									echo ' selected';
								}
								?>
							>
								<?php _e( 'Pages being measured', 'qa-heatmap-analytics' ); ?>
							</option>
							<option value="false" 
								<?php
								if ( $heatmap_sort_rec === 'false' ) {
									echo ' selected';
								}
								?>
							>
								<?php _e( 'Pages NOT being measured', 'qa-heatmap-analytics' ); ?>
							</option>
						</select>

						<select id="qahm-data-sort-view">
							<option value="10"
								<?php
								if ( $heatmap_sort_view === '10' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php printf( esc_html__( 'Show %d', 'qa-heatmap-analytics' ), 10 ); ?>
							</option>

							<option value="25"
								<?php
								if ( $heatmap_sort_view === '25' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php printf( esc_html__( 'Show %d', 'qa-heatmap-analytics' ), 25 ); ?>
							</option>

							<option value="50"
								<?php
								if ( $heatmap_sort_view === '50' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php printf( esc_html__( 'Show %d', 'qa-heatmap-analytics' ), 50 ); ?>
							</option>

							<option value="100"
								<?php
								if ( $heatmap_sort_view === '100' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php printf( esc_html__( 'Show %d', 'qa-heatmap-analytics' ), 100 ); ?>
							</option>
							
							<option value="all"
								<?php
								if ( $heatmap_sort_view === 'all' ) {
									echo esc_attr( ' selected' );
								}
								?>
							>
								<?php _e( 'Show ALL', 'qa-heatmap-analytics' ); ?>
							</option>
						</select>

						<input type="submit" id="qahm-data-sort-button" class="button" value="<?php esc_attr_e( 'Get Search', 'qa-heatmap-analytics' ); ?>">
						<div class="alignright">
							<p>
								<label for="qahm-all-rec-flag">
									<input type="checkbox" id="qahm-all-rec-flag"<?php if( $is_raw_save_all ){ echo ' checked'; } ?>>
									<?php _e( 'Measure all of the pages automatically, including newly published pages/posts.', 'qa-heatmap-analytics' ); ?>
								<label>
							</p>
						</div>
						<br class="clear">
					</div>
				</form>

				<form action="" method="post" id="dataForm" name="dataForm">
					<?php // ②：nonceの設定 ?>
					<?php wp_nonce_field( self::NONCE_ACTION, self::NONCE_NAME ); ?>
					
					<table id="qahm-data-list" class="widefat striped">
						<thead>
							<tr>
								<th scope="col">
									<?php esc_html_e( 'Page Title', 'qa-heatmap-analytics' ); ?>
								</th>
								<th scope="col">
									<?php esc_html_e( 'QA Heatmap Analysis (Amount of Data)', 'qa-heatmap-analytics' ); ?>
									<span class="qahm-tooltip" data-qahm-tooltip="<?php echo(
										esc_attr__( 'Select the icon of each device to see the heatmap.', 'qa-heatmap-analytics' )
										); ?>">
										<i class="far fa-question-circle"></i>
									</span>
								</th>
								<th scope="col">
									<?php
										esc_html_e( 'Page Version', 'qa-heatmap-analytics' );
										echo ' : ';
										esc_html_e( 'Span', 'qa-heatmap-analytics' );
									?>
								</th>
								<th scope="col" id="test">
									<?php esc_html_e( 'Switch Version', 'qa-heatmap-analytics' ); ?>
									<span class="qahm-tooltip" data-qahm-tooltip="<?php echo(
										esc_attr__( 'Save the current page version and start a new measurement. ', 'qa-heatmap-analytics' ) .
										esc_attr__( 'Use this after you edited the page or changed the CSS.', 'qa-heatmap-analytics' )
										); ?>">
										<i class="far fa-question-circle"></i>
									</span>
								</th>
								<th scope="col">
									<?php esc_html_e( 'Type', 'qa-heatmap-analytics' ); ?>
								</th>
								<th scope="col"<?php
									if( $is_raw_save_all ){
										echo ' style="' . esc_attr( 'background-color: #dc3545; color: #fff;' ) . '"';
									}
									?>>
									<?php 
									if ( $is_raw_save_all ) {
										esc_html_e( 'Measuring All Pages.', 'qa-heatmap-analytics' );
									} else {
										esc_html_e( 'Measuring', 'qa-heatmap-analytics' );
									}
									?>
								</th>
							</tr>
						</thead>
						<tbody>
						</tbody>
					</table>

					<p class="submit">
						<input type="submit" name="submit" id="submit" value="<?php esc_attr_e( 'Save Changes', 'qa-heatmap-analytics' ); ?>" onclick="qahm.postParam();" class="button button-primary">
					</p>
				</form>
			</div>
		</div>
		<?php
	}

	/**
	 * QAHMで使いやすい形のリストを作成
	 */
	public function create_heatmap_list() {
		$list = array();

		// ホーム
		$list[] = array(
			'page_id'       => null,
			'version_index' => null,
			'url'           => esc_url( get_home_url() ),
			'title'         => esc_html__( 'Home', 'qa-heatmap-analytics' ),
			'id'            => 1,
			'type'          => 'home',
			'measure'       => 0,
			'change'        => false,
		);

		// 投稿、固定、カスタム投稿
		$posts = get_posts(
			array(
				'posts_per_page' => -1,
				'post_type'      => 'any',
			)
		);

		foreach ( $posts as $post ) {
			switch ( $post->post_type ) {
				case 'post':
					$type = 'p';
					break;
				case 'page':
					$type = 'page_id';
					break;
				// default: $type = 'カスタム投稿'; break;
				default:
					$type = 'p';
					break;
			}
			$list[] = array(
				'page_id'       => null,
				'version_index' => null,
				'url'           => esc_url( get_permalink( $post->ID ) ),
				'title'         => esc_html( $post->post_title ),
				'id'            => $post->ID,
				'type'          => $type,
				//'slug'          => $post->post_name,
				'measure'       => 0,
				'change'        => false,
			);
		}

		// カテゴリ、タグ、カスタムタクソノミー
		$terms = get_terms( array( 'get' => 'all' ) );
		foreach ( $terms as $term ) {
			switch ( $term->taxonomy ) {
				case 'category':
					$type = 'cat';
					break;
				case 'post_tag':
					$type = 'tag';
					break;
				default:
					$type = 'tax';
					break;
			}
			$list[] = array(
				'page_id'       => null,
				'version_index' => null,
				'url'           => esc_url( get_term_link( $term->term_id ) ),
				'title'         => esc_html( $term->name ),
				'id'            => $term->term_id,
				'type'          => $type,
				'measure'       => 0,
				'change'        => false,
			);
		}

		return $list;
	}

	/**
	 * テーブルの存在確認
	 * 他でも使うようならqahm baseに持っていっても良い
	 */
	private function exists_database_table( $table_name ) {
		global $wpdb;
		$query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $table_name ) );
		return $wpdb->get_var( $query ) === $table_name ? true : false;
	}

	/**
	 * qaテーブルからバージョン情報を作成
	 * cronでcase分けをするためcreate_heatmap_listとは関数を分けている
	 */
	public function add_version_info( $list ) {
		global $qahm_db;
		global $qahm_time;
		global $wp_filesystem;
		global $qahm_log;

		// テーブルの存在確認。cronによるdbテーブル作成処理がまだ実行されていない時用
		if ( ! $this->exists_database_table( $qahm_db->prefix . 'qa_pages' ) ||
			 ! $this->exists_database_table( $qahm_db->prefix . 'qa_page_version_hist' ) ||
			 ! $this->exists_database_table( $qahm_db->prefix . 'qa_pv_log' ) ) {
			return $list;
		}

		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		if ( ! $wp_filesystem->exists( $cache_dir_path ) ) {
			$wp_filesystem->mkdir( $cache_dir_path );
		}
		
		$list_temp_path     = $cache_dir_path . 'heatmap_list_temp.php';
		$list_idx_temp_path = $cache_dir_path . 'heatmap_list_idx_temp.php';
		$list_idx_def       = $this->wrap_get_contents( $list_idx_temp_path );
		if ( $list_idx_def ) {
			$list_idx_def = (int)$list_idx_def; 
		} else {
			$list_idx_def = 0;
		}

		for ( $list_idx = $list_idx_def, $list_max = count( $list ); $list_idx < $list_max; $list_idx++ ) {
			// 進行状況を定期的に保存
			if ( $list_idx % 10 === 0 && $list_idx !== $list_idx_def ) {
				$this->wrap_put_contents( $list_temp_path, serialize( $list ) );
				$this->wrap_put_contents( $list_idx_temp_path, $list_idx );
			}
			
			// スラッシュあり、なしの両方を検索
			$url1 = $list[$list_idx]['url'];
			if( substr( $url1, -1 ) === '/' ) {
				$url2 = rtrim( $url1, '/' );
			} else {
				$url2 = $url1 . '/';
			}
            $url1_hash = hash( 'fnv164', $url1 );
            $url2_hash = hash( 'fnv164', $url2 );

			$table_name   = $qahm_db->prefix . 'qa_pages';
			$query        = 'SELECT page_id,url FROM ' . $table_name . ' WHERE url_hash IN(%s,%s)';
			$qa_pages_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $url1_hash, $url2_hash ), ARRAY_A );
			if( ! $qa_pages_ary ) {
				continue;
			}
			$qa_pages = $qa_pages_ary[0];
			$list[$list_idx]['page_id'] = (int) $qa_pages['page_id'];

			$table_name               = $qahm_db->prefix . 'view_page_version_hist';
			$query                    = 'SELECT page_id,version_id,device_id,version_no,insert_datetime FROM ' . $table_name . ' WHERE page_id = %d';
			$qa_page_version_hist_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $qa_pages['page_id'] ), ARRAY_A );
			if ( ! $qa_page_version_hist_ary  ) {
				continue;
			}

			// version_noが大きい順にソート
			$ver_sort_ary = array();
			foreach ( $qa_page_version_hist_ary as $key => $value ) {
				$ver_sort_ary[$key] = (int) $value['version_no'];
			}
			array_multisort( $ver_sort_ary, SORT_DESC, $qa_page_version_hist_ary );


			// 最大のバージョンナンバーを求める（ソートされているのでインデックス0は最大値）
			$ver_no_max = (int) $qa_page_version_hist_ary[0]['version_no'];

			$ver_no_now          = null;
			$ins_date_unixtime   = null;
			$ins_date            = null;
			$period_to           = null;
			$period_from         = null;

			foreach ( $qa_page_version_hist_ary as $qa_page_version_hist_idx => $qa_page_version_hist ) {
				// 初回検索時
				if ( $qa_page_version_hist_idx === 0 ) {
					$ver_no_now                       = (int) $qa_page_version_hist['version_no'];
					$ins_date_unixtime                = $qahm_time->str_to_unixtime( $qa_page_version_hist['insert_datetime'] );
					$ins_date                         = $qahm_time->unixtime_to_str( $ins_date_unixtime, 'Y-m-d' );
					$period_to                        = $qahm_time->today_str();
					$period_from                      = $ins_date;
					$list[$list_idx]['version_index'] = $ver_no_now;
				}

				// version_infoの初期化
				if ( empty( $list[$list_idx]['version_info'][$qa_page_version_hist['version_no']] ) ) {
					$list[$list_idx]['version_info'][$qa_page_version_hist['version_no']] = array();
					$version_info = &$list[$list_idx]['version_info'][$qa_page_version_hist['version_no']];

					$version_info['data_all'] = 0;
					foreach ( QAHM_DEVICES as $qahm_dev ) {
						$version_info['data_' . $qahm_dev['name']] = 0;
						$version_info['version_id_' . $qahm_dev['name']] = null;
					}
				} else {
					$version_info = &$list[$list_idx]['version_info'][$qa_page_version_hist['version_no']];
				}

				// version_infoの設定
				$data_num = 0;
				$ins_date_unixtime = $qahm_time->str_to_unixtime( $qa_page_version_hist['insert_datetime'] );
				$ins_date          = $qahm_time->unixtime_to_str( $ins_date_unixtime, 'Y-m-d' );
				if ( $ver_no_now > (int) $qa_page_version_hist['version_no']) {
					$ver_no_now  = (int) $qa_page_version_hist['version_no'];
					$period_to   = $period_from;
					$period_from = $ins_date;
				}

				// 最新バージョンの検索時のみ、データ数を見る（view_pvを検索する）
				if ( $ver_no_max === $ver_no_now ) {
					$table_name    = $qahm_db->prefix . 'view_pv';
					$query         = 'SELECT access_time,is_raw_p,is_raw_e FROM ' . $table_name . ' WHERE version_id = %d';
					//$table_name    = $qahm_db->prefix . 'qa_pv_log';
					//$query         = 'SELECT access_time,raw_p,raw_e FROM ' . $table_name . ' WHERE version_id = %d';
					$qa_pv_log_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $qa_page_version_hist['version_id'] ), ARRAY_A );

					if ( $qa_pv_log_ary ) {
						$fetch_now_utime = $qahm_time->now_unixtime();
						$older_utime     = $fetch_now_utime;
						foreach ( $qa_pv_log_ary as $qa_pv_log ) {
							if ( $qa_pv_log['is_raw_p'] || $qa_pv_log['is_raw_e'] ) {
							//if ( $qa_pv_log['raw_p'] || $qa_pv_log['raw_e'] ) {
								$data_num++;
								$access_utime = $qahm_time->str_to_unixtime( $qa_pv_log['access_time'] );
								if ( $access_utime < $older_utime ) {
									$older_utime = $access_utime;
								}
							}
						}
						if ( $older_utime !== $fetch_now_utime ) {
						$period_from = $qahm_time->unixtime_to_str( $older_utime, 'Y-m-d' );
						}
					}
				}

				if ( empty( $version_info['period_from'] ) ) {
					$version_info['period_from'] = $period_from;
					$version_info['period_to']   = $period_to;
				}

				$dev_name = $this->device_id_to_device_name( $qa_page_version_hist['device_id'] );
				$version_info[ 'version_id_' . $dev_name ] = (int) $qa_page_version_hist['version_id'];
				if ( $data_num > 0 ) {
					$version_info[ 'data_' . $dev_name ] = $data_num;
					$version_info[ 'data_all' ] += $data_num;
				}
			}
		}

		return $list;
	}

	/**
	 * 計測中の情報をリストに付与
	 */
	public function add_refresh_info( $heatmap_list ) {
		global $wp_filesystem;

		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		if ( ! $wp_filesystem->exists( $cache_dir_path ) ) {
			$wp_filesystem->mkdir( $cache_dir_path );
		}

		$refresh_path = $cache_dir_path . 'heatmap_list_refresh.php';
		if ( $wp_filesystem->exists( $refresh_path ) ) {
			$refresh_list = unserialize( $this->wrap_get_contents( $refresh_path ) );

			foreach ( $refresh_list as $refresh ) {
				foreach( $heatmap_list as &$heatmap ) {
					if( $heatmap['page_id'] === $refresh['page_id'] ) {
						$heatmap['version_refresh'] = $refresh['version_no'];
						break;
					}
				}
			}

		}

		return $heatmap_list;
	}

	/**
	 * 計測フラグをリストに付与
	 */
	public function add_rec_flag( $list ) {
		global $qahm_recterm;
		$rt_recs = $qahm_recterm->get_all_record();
		if ( is_array( $list ) ) {
			for ( $i = 0; $i < count( $list ); $i++ ) {
				if ( $rt_recs ) {
					foreach ( $rt_recs as $rt_rec ) {
						if ( $list[ $i ]['type'] === $rt_rec['type'] && $list[ $i ]['id'] === (int) $rt_rec['ids'] ) {
							$list[ $i ]['measure'] = (int) $rt_rec['rec_flag'];
							break;
						}
						//usleep( 100 );
					}
				}
			}
		}

		return $list;
	}

	/**
	 * 設定画面の項目をデータベースに保存する
	 */
	public function save_config() {
		$post = array();
		// nonceで設定したcredentialのチェック

		// ヒートマップ管理画面
		if ( isset( $_POST[ self::NONCE_NAME ] ) && $_POST[ self::NONCE_NAME ] ) {
			if ( check_admin_referer( self::NONCE_ACTION, self::NONCE_NAME ) ) {
				//$this->wrap_update_option( 'is_first_heatmap_setting', false );
				// falseだとkeyがdbにない場合、値が何も入らないので確実に空欄が1入るようにしている
				$this->wrap_update_option( 'is_first_heatmap_setting', '' );

				/*
				// ライセンス認証による制限 あとで今の仕様に変更
				if ( ! $this->wrap_get_option( 'license_auth' ) ) {
					$post['list_rec_num'] = filter_input( INPUT_POST, 'list_rec_num' );
					if ( $post['list_rec_num'] > QAHM_License::FREE_MEASURE_PAGE ) {
						self::$error_msg['list_rec_num'] = ( QAHM_License::FREE_MEASURE_PAGE + 1 ) . 'ページ以上の計測を行う場合はライセンス認証を行ってください。';
						return;
					}
				}
				*/

				global $wpdb;
				global $qahm_time;
				global $qahm_recterm;
				$db_rts   = $qahm_recterm->get_all_record();
				$bulk_rts = array();

				$post['list_num'] = filter_input( INPUT_POST, 'list_num' );
				if ( $post['list_num'] > 0 ) {
					for ( $i = 0; $i < $post['list_num']; $i++ ) {
						$post[ 'type_' . $i ]     = filter_input( INPUT_POST, 'type_' . $i );
						$post[ 'ids_' . $i ]      = filter_input( INPUT_POST, 'ids_' . $i );
						$post[ 'rec_days_' . $i ] = filter_input( INPUT_POST, 'rec_days_' . $i );
						$post[ 'rec_flag_' . $i ] = filter_input( INPUT_POST, 'rec_flag_' . $i );
						if ( ! $post[ 'type_' . $i ] || ! $post[ 'ids_' . $i ] ) {
							continue;
						}

						$idx_rt = -1;

						if ( $db_rts ) {
							$db_rts_cnt = count( $db_rts );
							for ( $j = 0; $j < $db_rts_cnt; $j++ ) {
								if ( $post[ 'type_' . $i ] == $db_rts[ $j ]['type'] &&
									$post[ 'ids_' . $i ] == $db_rts[ $j ]['ids'] ) {
									$idx_rt = $j;
									break;
								}
								usleep( 100 );
							}
						}

						// recterm内に見つからなかった場合、recrefreshも無いということなのでrecrefreshも作成する
						if ( $idx_rt === -1 ) {
							$bulk_rts[] = array(
								'type'        => $post[ 'type_' . $i ],
								'ids'         => $post[ 'ids_' . $i ],
								'rec_flag'    => $post[ 'rec_flag_' . $i ],
								'update_date' => $post[ 'rec_flag_' . $i ] ? date_i18n( 'Y-m-d' ) : null,
							);

						} else {
							// rec_flagを前回オフからオンに切り替えた時はupdate_dateを更新
							if ( $post[ 'rec_flag_' . $i ] && ! $db_rts[ $idx_rt ]['rec_flag'] ) {
								$update_date = date_i18n( 'Y-m-d' );
							} else {
								$update_date = $db_rts[ $idx_rt ]['update_date'];
							}

							$bulk_rts[] = array(
								'type'        => $db_rts[ $idx_rt ]['type'],
								'ids'         => $db_rts[ $idx_rt ]['ids'],
								'rec_days'    => $post[ 'rec_days_' . $i ] ? $post[ 'rec_days_' . $i ] : $db_rts[ $idx_rt ]['rec_days'],
								'rec_flag'    => $post[ 'rec_flag_' . $i ],
								'file_size'   => $db_rts[ $idx_rt ]['file_size'],
								'insert_date' => $db_rts[ $idx_rt ]['insert_date'],
								'update_date' => $update_date,
								'start_date'  => $db_rts[ $idx_rt ]['start_date'],
								'stop_date'   => $db_rts[ $idx_rt ]['stop_date'],
							);
						}

						usleep( 100 );
					}

					// バルクインサート
					if ( isset( $bulk_rts ) ) {
						$qahm_recterm->bulk_insert( $bulk_rts );
					}
				}
			}
		}
	}

	/**
	 * ヒートマップリストを取得
	 */
	public function ajax_create_heatmap_list() {
		global $wp_filesystem;

		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		$this->wrap_mkdir( $cache_dir_path );
		
		// リストファイル読み込み
		$list_path      = $cache_dir_path . 'heatmap_list.php';
		$list_temp_path = $cache_dir_path . 'heatmap_list_temp.php';

		if ( $wp_filesystem->exists( $list_path ) ) {
			$list = unserialize( $this->wrap_get_contents( $list_path ) );
		} else {
			// 一時的なリスト作成
			if ( $wp_filesystem->exists( $list_temp_path ) ) {
				$list = unserialize( $this->wrap_get_contents( $list_temp_path ) );
			} else {
				$list = $this->create_heatmap_list();
				$this->wrap_put_contents( $list_temp_path, serialize( $list ) );
			}

			// 一時的なリストにバージョン情報を追加してリストを完成させる
			$list = $this->add_version_info( $list );
		
			// リスト配列の中身をファイルに保存
			$this->wrap_put_contents( $list_path, serialize( $list ) );
		}

		$list = $this->add_refresh_info( $list );
		$list = $this->add_rec_flag( $list );

		echo wp_json_encode( $list );
		die();
	}

	/**
	 * バージョン切り替え
	 */
	public function ajax_refresh_version() {
		$nonce = filter_input( INPUT_POST, 'nonce' );
		if ( ! wp_verify_nonce( $nonce, self::NONCE_REFRESH ) ) {
			http_response_code( 400 );
			die( 'wp_verify_nonce error' ); 
		}

		global $wp_filesystem;
		global $wpdb;
		global $qahm_time;
		$today_str  = $qahm_time->today_str();
		$now_str    = $qahm_time->now_str();

		$page_id    = (int) filter_input( INPUT_POST, 'page_id' );
		$wp_qa_type = filter_input( INPUT_POST, 'wp_qa_type' );
		$wp_qa_id   = (int) filter_input( INPUT_POST, 'wp_qa_id' );
		$url        = filter_input( INPUT_POST, 'url' );
		$title      = filter_input( INPUT_POST, 'title' );

		if ( empty( $page_id ) ) {
			// qa_pages insert
			$table_name = $wpdb->prefix . 'qa_pages';
			
			$wpdb->insert( 
				$table_name, 
				array( 
					'tracking_id' => $this->get_tracking_id(),
					'wp_qa_type'  => $wp_qa_type, 
					'wp_qa_id'    => $wp_qa_id,
					'url'         => $url,
					'url_hash'    => hash( 'fnv164', $url ),
					'title'       => $title,
					'update_date' => $today_str
				), 
				array( 
					'%s',
					'%s',
					'%d',
					'%s',
					'%s',
					'%s',
					'%s'
				) 
			);
			$page_id = $wpdb->insert_id;
		}
		
		$ver_hist_bulk_ary         = array();
		$ver_hist_place_holder_ary = array();

		// 最新バージョンを調べる。存在しなければバージョン1作成
		// 存在していれば全デバイスそのバージョンを作成＆新規バージョン追加
		$table_name           = $wpdb->prefix . 'qa_page_version_hist';
		$query                = 'SELECT version_id,device_id,version_no FROM ' . $table_name . ' WHERE page_id=%d and version_no=(select max(version_no) FROM ' . $table_name . ' WHERE page_id=%d)';
		$qa_page_version_hist = $wpdb->get_results( $wpdb->prepare( $query, $page_id, $page_id ) );

		$dev_id_ary = array();
		$cur_ver_no = 0;

		if ( $qa_page_version_hist ) {
			foreach ( $qa_page_version_hist as $hist ) {
				$dev_id_ary[] = $hist->device_id;
				$cur_ver_no   = $hist->version_no;
			}
		}

		// 今のバージョンは全デバイス分データを作成
		foreach ( QAHM_DEVICES as $qahm_dev ) {
			$dev_name  = $this->device_id_to_device_name( $qahm_dev['id'] );
			
			// デバイスによるユーザーエージェント指定
			$response = $this->wrap_remote_get( $url, $dev_name );
			if ( is_wp_error( $response ) ) {
				http_response_code( 400 );
				die( 'wp_remote_get is wp error' ); 
			} elseif( $response['response']['code'] !== 200 ) {
				http_response_code( 400 );
				die( 'wp_remote_get status code is other than 200' ); 
			} else {
				$base_html = $response['body'];
			}

			if ( $qa_page_version_hist ) {
				$find = false;
				if ( $dev_id_ary !== array() ) {
					foreach ( $dev_id_ary as $dev_id ) {
						if ( $qahm_dev['id'] === (int) $dev_id ) {
							$find = true;
							break;
						}
					}
				}

				// 現行バージョンで作られていないデバイスのversion histを作成
				if ( ! $find ) {
					$ver_hist_bulk_ary[]         = $page_id;
					$ver_hist_bulk_ary[]         = $qahm_dev['id'];
					$ver_hist_bulk_ary[]         = $cur_ver_no;
					$ver_hist_bulk_ary[]         = $base_html;
					$ver_hist_bulk_ary[]         = $today_str;
					$ver_hist_bulk_ary[]         = $now_str;
					$ver_hist_place_holder_ary[] = '(%d, %d, %d, %s, %s, %s)';
				}
			}

			// バージョン追加
			$ver_hist_bulk_ary[] = $page_id;
			$ver_hist_bulk_ary[] = $qahm_dev['id'];
			$ver_hist_bulk_ary[] = $cur_ver_no + 1;
			$ver_hist_bulk_ary[] = $base_html;
			$ver_hist_bulk_ary[] = $today_str;
			$ver_hist_bulk_ary[] = $now_str;
			$ver_hist_place_holder_ary[] = '(%d, %d, %d, %s, %s, %s)';
		}

		// バルクインサート
		$query = 'INSERT INTO ' . $table_name . ' ' .
				'(page_id, device_id, version_no, base_html, update_date, insert_datetime) ' .
				'VALUES ' . join( ',', $ver_hist_place_holder_ary ) . ' ' .
				'ON DUPLICATE KEY UPDATE ' .
				'page_id = VALUES(page_id), ' .
				'device_id = VALUES(device_id), ' .
				'version_no = VALUES(version_no), ' .
				'base_html = VALUES(base_html), ' .
				'update_date = VALUES(update_date), ' .
				'insert_datetime = VALUES(insert_datetime)';
		$data['result']  = $wpdb->query( $wpdb->prepare( $query, $ver_hist_bulk_ary ) );
		$data['version'] = $cur_ver_no + 1;

		$data['version_switch_success_text'] = sprintf( __( 'Switched to version %d. <br> The data will be available tomorrow or later.', 'qa-heatmap-analytics' ), $data['version'] );

		
		// その日のバージョンアップファイルを作成
		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		if ( ! $wp_filesystem->exists( $cache_dir_path ) ) {
			$wp_filesystem->mkdir( $cache_dir_path );
		}

		$refresh_path = $cache_dir_path . 'heatmap_list_refresh.php';
		$find  = false;
		$index = 0;
		if ( $wp_filesystem->exists( $refresh_path ) ) {
			$refresh_list = unserialize( $this->wrap_get_contents( $refresh_path ) );

			foreach ( $refresh_list as &$refresh ) {
				if( $refresh['page_id'] === $page_id ) {
					$refresh['version_no'] = $data['version'];
					$find                = true;
					break;
				}
				$index++;
			}

		} else {
			$refresh_list = array();
		}

		if ( ! $find ) {
			$refresh_list[ $index ] = array(
				'page_id'    => $page_id,
				'version_no' => $data['version']
			);
		}

		$this->wrap_put_contents( $refresh_path, serialize( $refresh_list ) );

		echo wp_json_encode( $data );
		die();
	}

	/**
	 * 実績の設定
	 */
	public function ajax_set_achievements() {
		$achievements = json_decode( filter_input( INPUT_POST, 'achievements' ), true );
		$this->wrap_update_option( 'achievements', $achievements );
	}

	/**
	 * 全ページ計測チェックの変更
	 */
	public function ajax_change_is_raw_save_all() {
		$is_raw_save_all = $this->wrap_get_option( 'is_raw_save_all' );
		if ( $is_raw_save_all ) {
			$is_raw_save_all = '';
		} else {
			$is_raw_save_all = 'on';
		}
		
		$this->wrap_update_option( 'is_raw_save_all', $is_raw_save_all );
	}

	/**
	 * バージョンIDから全データ数を返す
	 */
	public function ajax_get_data_num() {
		try {
			global $qahm_db;

			// $ver_id_aryは全端末のversion_idが入っている配列
			$ver_id_ary = json_decode( filter_input( INPUT_POST, 'version_id_ary' ), true );
			
			$table_name   = $qahm_db->prefix . 'view_pv';
			$query        = 'SELECT is_raw_p,is_raw_e FROM ' . $table_name . ' WHERE version_id = %d';
			//$table_name   = $qahm_db->prefix . 'qa_pv_log';
			//$query        = 'SELECT raw_p,raw_e FROM ' . $table_name . ' WHERE version_id = %d';
			$data_num_ary = array();
			
			foreach ( $ver_id_ary as $ver_idx => $ver_id ) {
				if ( ! $ver_id ) {
					continue;
				}

				$data_num_ary[$ver_idx] = 0;
				$qa_pv_log_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $ver_id ), ARRAY_A );
				if ( ! $qa_pv_log_ary ) {
					continue;
				}

				foreach ( $qa_pv_log_ary as $qa_pv_log ) {
					if( $qa_pv_log['is_raw_p'] || $qa_pv_log['is_raw_e'] ) {
					//if( $qa_pv_log['raw_p'] || $qa_pv_log['raw_e'] ) {
						$data_num_ary[$ver_idx]++;
					}
				}
			}

			echo wp_json_encode( $data_num_ary );
		} catch ( Exception $ex ) {
			echo '';
		}
		die();
	}

	/**
	 * テスト用 一応残しているが消してもいい
	 */
	/*
	public function test_add_version_info( $list ) {
		global $qahm_db;
		global $qahm_time;
		global $wp_filesystem;
		global $qahm_log;

		// テーブルの存在確認。cronによるdbテーブル作成処理がまだ実行されていない時用
		if ( ! $this->exists_database_table( $qahm_db->prefix . 'qa_pages' ) ||
			 ! $this->exists_database_table( $qahm_db->prefix . 'qa_page_version_hist' ) ||
			 ! $this->exists_database_table( $qahm_db->prefix . 'qa_pv_log' ) ) {
			return $list;
		}

		$cache_dir_path = $this->get_data_dir_path() . 'cache/';
		if ( ! $wp_filesystem->exists( $cache_dir_path ) ) {
			$wp_filesystem->mkdir( $cache_dir_path );
		}
		
		$list_temp_path     = $cache_dir_path . 'heatmap_list_temp.php';
		$list_idx_temp_path = $cache_dir_path . 'heatmap_list_idx_temp.php';
		$list_idx_def       = $this->wrap_get_contents( $list_idx_temp_path );
		if ( $list_idx_def ) {
			$list_idx_def = (int)$list_idx_def; 
		} else {
			$list_idx_def = 0;
		}

		for ( $list_idx = $list_idx_def, $list_max = count( $list ); $list_idx < $list_max; $list_idx++ ) {
			if ( $list[$list_idx]['id'] == 11358 ) {
			} else {
				continue;
			}
			// 進行状況を定期的に保存
			if ( $list_idx % 10 === 0 && $list_idx !== $list_idx_def ) {
				//$this->wrap_put_contents( $list_temp_path, serialize( $list ) );
				//$this->wrap_put_contents( $list_idx_temp_path, $list_idx );
			}
			
			// スラッシュあり、なしの両方を検索
			$url1 = $list[$list_idx]['url'];
			if( substr( $url1, -1 ) === '/' ) {
				$url2 = rtrim( $url1, '/' );
			} else {
				$url2 = $url1 . '/';
			}
            $url1_hash = hash( 'fnv164', $url1 );
            $url2_hash = hash( 'fnv164', $url2 );

			echo $list_idx . ':qa_pages_ary get_results start<br>';
			$start_time = microtime( true );
			$table_name   = $qahm_db->prefix . 'qa_pages';
			$query        = 'SELECT page_id,url FROM ' . $table_name . ' WHERE url_hash IN(%s,%s)';
			$qa_pages_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $url1_hash, $url2_hash ), ARRAY_A );
			if( ! $qa_pages_ary ) {
				echo $list_idx . ':qa_pages_aryが存在しません<br><br>';
				continue;
			}
			$qa_pages = $qa_pages_ary[0];
			$list[$list_idx]['page_id'] = (int) $qa_pages['page_id'];
			$end_time = microtime( true );
			if ( $end_time - $start_time < 0.0001 ) {
				$sec = '0.0001未満';
			} else {
				$sec = round( $end_time - $start_time, 6 );
			}
			echo $list_idx . ':qa_pages_ary get_results end<br>処理時間：' . $sec . ' 秒<br><br>';

			echo $list_idx . ':view_page_version_hist get_results start<br>';
			$start_time = microtime( true );
			$table_name               = $qahm_db->prefix . 'view_page_version_hist';
			$query                    = 'SELECT page_id,version_id,device_id,version_no,insert_datetime FROM ' . $table_name . ' WHERE page_id = %d';
			$qa_page_version_hist_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $qa_pages['page_id'] ), ARRAY_A );
			if ( ! $qa_page_version_hist_ary  ) {
				echo $list_idx . ':view_page_version_histが存在しません<br><br>';
				continue;
			}
			$end_time = microtime( true );
			if ( $end_time - $start_time < 0.0001 ) {
				$sec = '0.0001未満';
			} else {
				$sec = round( $end_time - $start_time, 6 );
			}
			echo $list_idx . ':view_page_version_hist get_results end<br>処理時間：' . $sec . ' 秒<br><br>';
			

			// version_noが大きい順にソート
			echo $list_idx . ':array_multisort start<br>';
			$start_time = microtime( true );
			$ver_sort_ary = array();
			foreach ( $qa_page_version_hist_ary as $key => $value ) {
				$ver_sort_ary[$key] = (int) $value['version_no'];
			}
			array_multisort( $ver_sort_ary, SORT_DESC, $qa_page_version_hist_ary );

			$end_time = microtime( true );
			if ( $end_time - $start_time < 0.0001 ) {
				$sec = '0.0001未満';
			} else {
				$sec = round( $end_time - $start_time, 6 );
			}
			echo $list_idx . ':array_multisort end<br>処理時間：' . $sec . ' 秒<br><br>';

			// test code /////////////
			foreach ( $qa_page_version_hist_ary as $value ) {
				echo('id:'.$value['version_id']);
			}
			//////////////////////////
			
			// 最大のバージョンナンバーを求める（ソートされているのでインデックス0は最大値）
			$ver_no_max = (int) $qa_page_version_hist_ary[0]['version_no'];

			$ver_no_now          = null;
			$ins_date_unixtime   = null;
			$ins_date            = null;
			$period_to           = null;
			$period_from         = null;

			foreach ( $qa_page_version_hist_ary as $qa_page_version_hist_idx => $qa_page_version_hist ) {
				// 初回検索時
				if ( $qa_page_version_hist_idx === 0 ) {
					$ver_no_now                       = (int) $qa_page_version_hist['version_no'];
					$ins_date_unixtime                = $qahm_time->str_to_unixtime( $qa_page_version_hist['insert_datetime'] );
					$ins_date                         = $qahm_time->unixtime_to_str( $ins_date_unixtime, 'Y-m-d' );
					$period_to                        = $qahm_time->today_str();
					$period_from                      = $ins_date;
					$list[$list_idx]['version_index'] = $ver_no_now;
				}

				// version_infoの初期化
				if ( empty( $list[$list_idx]['version_info'][$qa_page_version_hist['version_no']] ) ) {
					$list[$list_idx]['version_info'][$qa_page_version_hist['version_no']] = array();
					$version_info = &$list[$list_idx]['version_info'][$qa_page_version_hist['version_no']];

					$version_info['data_all'] = 0;
					foreach ( QAHM_DEVICES as $qahm_dev ) {
						$version_info['data_' . $qahm_dev['name']] = 0;
						$version_info['version_id_' . $qahm_dev['name']] = null;
					}
				} else {
					$version_info = &$list[$list_idx]['version_info'][$qa_page_version_hist['version_no']];
				}

				// version_infoの設定
				$data_num = 0;
				$ins_date_unixtime = $qahm_time->str_to_unixtime( $qa_page_version_hist['insert_datetime'] );
				$ins_date          = $qahm_time->unixtime_to_str( $ins_date_unixtime, 'Y-m-d' );
				if ( $ver_no_now > (int) $qa_page_version_hist['version_no']) {
					$ver_no_now  = (int) $qa_page_version_hist['version_no'];
					$period_to   = $period_from;
					$period_from = $ins_date;
				}

				// 最新バージョンの検索時のみ、データ数を見る（view_pvを検索する）
				if ( $ver_no_max === $ver_no_now ) {
					echo 'version_id:' . $qa_page_version_hist['version_id'] . '<br>';

					$table_name    = $qahm_db->prefix . 'view_pv';
					$query         = 'SELECT access_time,is_raw_p,is_raw_e FROM ' . $table_name . ' WHERE version_id = %d';
					//$table_name    = $qahm_db->prefix . 'qa_pv_log';
					//$query         = 'SELECT access_time,raw_p,raw_e FROM ' . $table_name . ' WHERE version_id = %d';
					echo $list_idx . ':qa_pv_log_ary get_results start<br>';
					$start_time = microtime( true );
					$qa_pv_log_ary = $qahm_db->get_results( $qahm_db->prepare( $query, $qa_page_version_hist['version_id'] ), ARRAY_A );
					$end_time = microtime( true );
					if ( $end_time - $start_time < 0.0001 ) {
						$sec = '0.0001未満';
					} else {
						$sec = round( $end_time - $start_time, 6 );
					}
					echo $list_idx . ':qa_pv_log_ary get_results end<br>処理時間：' . $sec . ' 秒<br><br>';
					
					echo('<pre>');
					var_dump($qa_pv_log_ary);
					echo('</pre>');

					echo $list_idx . ':qa_pv_log_ary search start<br>';
					$start_time = microtime( true );
					if ( $qa_pv_log_ary ) {
						$fetch_now_utime = $qahm_time->now_unixtime();
						$older_utime     = $fetch_now_utime;
						foreach ( $qa_pv_log_ary as $qa_pv_log ) {
							if ( $qa_pv_log['is_raw_p'] || $qa_pv_log['is_raw_e'] ) {
							//if ( $qa_pv_log['raw_p'] || $qa_pv_log['raw_e'] ) {
								$data_num++;
								$access_utime = $qahm_time->str_to_unixtime( $qa_pv_log['access_time'] );
								if ( $access_utime < $older_utime ) {
									$older_utime = $access_utime;
								}
							}
						}
						if ( $older_utime !== $fetch_now_utime ) {
						$period_from = $qahm_time->unixtime_to_str( $older_utime, 'Y-m-d' );
						}
					}
					$end_time = microtime( true );
					if ( $end_time - $start_time < 0.0001 ) {
						$sec = '0.0001未満';
					} else {
						$sec = round( $end_time - $start_time, 6 );
					}
					echo $list_idx . ':qa_pv_log_ary search end<br>処理時間：' . $sec . ' 秒<br><br>';
				}

				if ( empty( $version_info['period_from'] ) ) {
					$version_info['period_from'] = $period_from;
					$version_info['period_to']   = $period_to;
				}

				$dev_name = $this->device_id_to_device_name( $qa_page_version_hist['device_id'] );
				$version_info[ 'version_id_' . $dev_name ] = (int) $qa_page_version_hist['version_id'];
				if ( $data_num > 0 ) {
					$version_info[ 'data_' . $dev_name ] = $data_num;
					$version_info[ 'data_all' ] += $data_num;
				}
			}
		}

		return $list;
	}
	*/

} // end of class
