<?php
/**
 * 
 *
 * @package qa_heatmap
 */

$qahm_admin_page_help = new QAHM_Admin_Page_Help();

class QAHM_Admin_Page_Help extends QAHM_Admin_Page_Base {

	// スラッグ
	const SLUG = QAHM_NAME . '-help';

	// nonce
	const NONCE_ACTION = self::SLUG . '-nonce-action';
	const NONCE_NAME   = self::SLUG . '-nonce-name';

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		parent::__construct();
	}
	
	/**
	 * 初期化
	 */
	public function enqueue_scripts( $hook_suffix ) {
		if( $this->hook_suffix !== $hook_suffix ||
			! $this->is_enqueue_jquery()
		) {
			return;
		}

		// enqueue_style
		$this->common_enqueue_style();

		// enqueue script
		$this->common_enqueue_script();
	
		// inline script
		$scripts = $this->get_common_inline_script();
		wp_add_inline_script( QAHM_NAME . '-common', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', obj );', 'before' );

		// localize
		$localize = $this->get_common_localize_script();
		wp_localize_script( QAHM_NAME . '-common', QAHM_NAME . 'l10n', $localize );
	}

	/**
	 * ページの表示
	 * ヘルプページは内容が薄いのでcssを直書きしている
	 */
	public function create_html() {
		$img_dir_url = plugin_dir_url(__FILE__) . 'img/';
		$lang_set = get_bloginfo('language');
		?>

		<style>
			.qahm-help-head {
				font-size: 1.2em;
    			margin: 0 0 6px 0;
			}
			.qahm-help-text {
				margin: 0;
			}
			.qahm-help-item {
				margin-bottom: 24px;
			}

			#qahm-help-debug {
				padding: 16px;
				margin-top: 60px;
				background-color: #fff;
				max-width: 600px;
			}

			#qa-help h2{
				padding: 0 1em .5em;
				border-bottom: 1px solid #999;
			}
			#qa-help h3{
				padding: 0 1em;
			}
			#qa-help p{
				padding: .5em 1em .5em 1em ;
			}
			#qa-help div.help01 ul{
				list-style: disc;
				margin-left: 35px;
			}
			#qa-help div.h-frame{
				background-color: #fff; border: 1px solid #999;
				margin-bottom: 1em;
			}
			#qa-help div.help02{
			background-image: <?php echo 'url(' . $img_dir_url . 'help-q2.png)'; ?>;
			background-repeat: no-repeat;
			background-position: right bottom;
			}
			#qa-help div.help03 h3{
				margin: 5px;
			}
			#qa-help div.help03 p{
				margin-top: 0;
			}
			#qa-help div.help04 table{
			margin: 1em;
			border-collapse: collapse;
			border-spacing: 0 0px;
			word-break: break-word;
			}
			#qa-help div.help04 table th {
			padding: .3em;
			text-align: left;
			font-weight: normal;
			background-color: #F4F4F4;
			border: 1px solid #999999;
			width: 30%;
			}
			#qa-help div.help04 table td {
			padding: .3em;
			border: 1px solid #999999;
			}
			#qa-help .wavy-line {
				text-decoration: wavy underline;
			}
			@media only screen and (min-width: 576px) {
				#qa-help div.help04 table th {
				width: 11em;
				}
			}
		</style>

		<div id="<?php esc_attr_e( basename( __FILE__, '.php' ) ); ?>" class="qahm-admin-page">
			<div class="wrap">
				<h1>QA <?php _e( 'Help', 'qa-heatmap-analytics' ); ?></h1>
				<?php $this->view_announce_html(); ?>
				<h2 style="font-size: 1.2em; margin-bottom: 40px;"><?php _e( 'Your Installed Version of QA Heatmap Analytics', 'qa-heatmap-analytics' ); ?>: <?php echo esc_html( QAHM_PLUGIN_VERSION ); ?></h2>

				<div id="qa-help">

					<?php 
						if ( $lang_set !== 'ja' ) :
					?>
					<div class="english-note-important">
						<h4>[Important!]</h4>
						<p>Our Support Site is <span style="font-weight: bold;">written in Japanese</span>. To read them using a translate service of browsers would be really appreciated.<br>
							You can also contact us and ask questions at our <a href="https://wordpress.org/support/plugin/qa-heatmap-analytics/" target="_blank" rel="noopener">WordPress Support Forum</a>.</p>
					</div>
					<?php endif; ?>

					<div class="h-frame help03">		
						<h2><?php _e( 'Contact Us', 'qa-heatmap-analytics' ); ?></h2>
						<h3><a href="https://wordpress.org/support/plugin/qa-heatmap-analytics/" target="_blank" rel="noopener"><?php _e( 'WordPress Support Forum', 'qa-heatmap-analytics' ); ?></a></h3>
						<p><?php _e( 'If you have a question, ask at the WordPress support forum. Our support team will answer and help you; as well as other kind people will (because it is the WordPress official forum:)).', 'qa-heatmap-analytics' ); ?><br>
						<?php _e( '(*You need to log in with your WordPress.org account to use the forum. If you do not have one, please create an account.)', 'qa-heatmap-analytics' ); ?></p>
						<h3><a href="https://mem.quarka.org/memberhome/" target="_blank" rel="noopener"><?php _e( 'Contact our Support Team Directly (login needed)', 'qa-heatmap-analytics' ); ?></a></h3>
						<p><?php _e( 'Those who subscribe to a paid plan can use a direct-contact-form in our support site. Please log in and fill the form.', 'qa-heatmap-analytics' ); ?><br>
						<?php _e( 'The "Debug" information which will be needed for technical questions such as about plugin operations is at the bottom of this Help screen.', 'qa-heatmap-analytics' ); ?></p>
					</div>

					<div class="h-frame help02">
						<h2><?php _e( 'User Guide', 'qa-heatmap-analytics' ); ?></h2>
						<h3><a href="https://mem.quarka.org/manual/" target="_blank" rel="noopener"><?php _e( 'Support Site / QA Manuals', 'qa-heatmap-analytics' ); ?></a></h3>
						<p><?php _e( 'Get information about how to use and set up QA heatmap analytics, troubleshooting, etc.', 'qa-heatmap-analytics' ); ?></p>
						<h3><a href="https://quarka.org/howto-install/" target="_blank" rel="noopener"><?php _e( 'Installation and Setting Method', 'qa-heatmap-analytics' ); ?></a></h3>
						<p><?php _e( 'Occasionally, depending on your web environment, phenomena such as "data is not collected" or "an error occurs" may occur. Check this page if the installation is not successful.', 'qa-heatmap-analytics' ); ?></p>
						<h3><a href="https://quarka.org/faq/" target="_blank" rel="noopener"><?php _e( 'FAQ Pages', 'qa-heatmap-analytics' ); ?></a></h3>
						<p><?php _e( 'Listing up frequently asked questions such as compatibility with plugins, differences between a free and a paid plan.', 'qa-heatmap-analytics' ); ?></p>
					</div>

					<?php if ( $lang_set === 'ja' ) : ?>
					<div class="h-frame help01">
						<h2>よくあるお問い合わせ</h2>
						<ul>
						<li><a href="https://mem.quarka.org/manual/cautions-for-installing/" target="_blank" rel="noopener">インストール・有効化したが翌日になってもデータがとれていない</a></li>
						<li><a href="https://mem.quarka.org/manual/data0-problem-2/" target="_blank" rel="noopener">データ数が0になってしまう</a></li>
						<li><a href="https://quarka.org/faq/#plugins" target="_blank" rel="noopener">他のプラグインと干渉することはありませんか？</a></li>
						<li><a href="https://mem.quarka.org/manual/defarence-between-qa-ga/" target="_blank" rel="noopener">Google Analyticsとユーザー数や滞在時間の数値が違う</a></li>
						<li><a href="https://quarka.org/faq/#howtouse" target="_blank" rel="noopener">インストール後ヒートマップのデータがいつ見られる？</a></li>
						</ul>
						<p>その他、<a href="https://mem.quarka.org/manual/" target="_blank" rel="noopener">QAマニュアル</a>もご参照ください。</p>
					</div>
					<?php endif; ?>

							
					<div class="h-frame help04">
						<h2><?php _e( 'Glossary', 'qa-heatmap-analytics' ); ?></h2>
						<p><?php _e( 'This briefly explains the terms that appear on the QA screens.', 'qa-heatmap-analytics' ); ?><br>
						<?php printf( esc_html__( 'For more details, see %1$s QA Manuals %2$s in our support (written in Japanese).', 'qa-heatmap-analytics' ), '<a href="https://mem.quarka.org/manual/" target="_blank" rel="noopener">', '</a>'); ?></p>
						<table>
							<tbody>
								<tr>
									<th><?php _e( 'Scroll Map', 'qa-heatmap-analytics' ); ?></th>
									<td><?php _e( 'Indicates "whether this page was read all the way down."', 'qa-heatmap-analytics' ); ?><br>
									<?php _e( 'As you place (and move) the mouse cursor on Scroll Map, it tells you the number of users who were staying to watch on that horizontal line of the page.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php _e( 'Attention Map', 'qa-heatmap-analytics' ); ?></th>
									<td><?php _e( 'Indicates "what content the users are interested in."', 'qa-heatmap-analytics' ); ?><br>
									<?php _e( 'Frequently read parts are displayed in red like a (temperature) heat map.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php _e( 'Click Heatmap', 'qa-heatmap-analytics' ); ?></th>
									<td><?php _e( 'Indicates "where the users are clicking."', 'qa-heatmap-analytics' ); ?><br>
									<?php _e( 'The place a user clicks is likely to be the point of interest. The parts that have been clicked a lot are displayed in red.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php _e( 'Click Count Map', 'qa-heatmap-analytics' ); ?></th>
									<td><?php _e( 'Indicates "how many times this button or link has been pressed."', 'qa-heatmap-analytics' ); ?><br>
									<?php _e( 'You can see trends such as banner clicks.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php _e( 'Average Time on Page', 'qa-heatmap-analytics' ); ?></th>
									<td><?php _e( 'Shows the average amount of time spent on the page per person, calcurated by: sum total of time on page / amount of data.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php _e( 'Time on Site', 'qa-heatmap-analytics' ); ?></th>
									<td><?php printf( esc_html__( 'The time while a user actually browses your site is measured with an accuracy of +-3 seconds; this is the point %1$s differs from Google Analytics %2$s.', 'qa-heatmap-analytics' ), '<span class="wavy-line">', '</span>' ); ?><br>
									<?php _e( 'QA Heatmap Analytics doesn\'t count time while a user is browsing another site or using other applications, so you can see an accurate time range that the user stayed.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php _e( 'Replay<br>(Session Replay)', 'qa-heatmap-analytics' ); ?></th>
									<td><?php _e( 'Session Replay is a function that shows you how the page was viewed in a session, likely to follow the user\'s line of sight.', 'qa-heatmap-analytics' ); ?><br>
									<?php _e( 'Press a "play" button of each session row in the list, which you can find at "Real Time" or "Sessions & Heatmap" on "Home" screen, to replay.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
								<tr>
									<th><?php _e( 'Download TSV Data', 'qa-heatmap-analytics' ); ?></th>
									<td><?php _e( 'QA Heatmap Analytics records every pageview*, and you can download those data!', 'qa-heatmap-analytics' ); ?><br>
									<?php _e( 'Download the file from "Overview" section for the whole data, or from "Sessions & Heatmap" section for the data of a specific page you want.', 'qa-heatmap-analytics' ); ?><br>
									<?php _e( '*Pageview data include: os, browser, page url and title, device category, utm_source, source_domain, utm_medium, utm_campaign, session_number, access_time, ordinal of pv in a session, speed_msec, browse_sec, is_last pv (or not), is_new-user (or returning).', 'qa-heatmap-analytics' ); ?><br>
									<?php _e( '**TSV is a text format in which each value is separated with tabs.', 'qa-heatmap-analytics' ); ?></td>
								</tr>
							</tbody>
						</table>
					</div>



				</div>

			</div>
		</div>

		
		
		
		<!-- Start debug -->
		<?php
			global $wpdb, $wp_version;
		
			$my_theme     = wp_get_theme();
			$site_plugins = get_plugins();
			$plugin_names = [];
		
			foreach( $site_plugins as $main_file => $plugin_meta ) {
				if ( ! is_plugin_active( $main_file ) ) {
					continue;
				}
				$plugin_names[] = sanitize_text_field( $plugin_meta['Name'] . ' ' . $plugin_meta['Version'] );
			}
			
			$options = '';
			/*
			foreach ( QAHM_OPTIONS as $key => $value ) {
				$options .= '<p><strong>' . $key . ':</strong><br>' . $this->wrap_get_option( $key ) . '</p>';
			}
			*/
			$options .= '<p><strong>Plugin version:</strong><br>' . QAHM_PLUGIN_VERSION . '</p>';

			$cron_status = $this->wrap_get_contents( $this->get_data_dir_path() . 'cron_status' );
		?>
		<div id="qahm-help-debug">
			<h3>Debug</h3>
			<hr>
			<p><strong>Server IP address:</strong><br><?php echo filter_input( INPUT_SERVER, 'SERVER_ADDR' ); ?></p>
			<p><strong>PHP version:</strong><br><?php echo phpversion(); ?></p>
			<p><strong>PHP extensions:</strong><br><?php echo implode( ', ', get_loaded_extensions() ); ?></p>
			<p><strong>Database version:</strong><br><?php echo $wpdb->get_var( "SELECT VERSION();" ); ?></p>
			<p><strong>InnoDB availability:</strong><br><?php echo $wpdb->get_var( "SELECT SUPPORT FROM INFORMATION_SCHEMA.ENGINES WHERE ENGINE = 'InnoDB';" ); ?></p>
			<p><strong>WordPress version:</strong><br><?php echo $wp_version; ?></p>
			<p><strong>Multisite:</strong><br><?php echo ( function_exists( 'is_multisite' ) && is_multisite() ) ? 'Yes' : 'No'; ?></p>
			<p><strong>Active plugins:</strong><br><?php echo implode( ', ', $plugin_names ); ?></p>
			<p><strong>Theme:</strong><br><?php echo $my_theme->get( 'Name' ) . ' (' . $my_theme->get('Version') . ') by ' . $my_theme->get('Author'); ?></p>
			<?php echo $options; ?>
			<p><strong>qalog.txt:</strong><br><?php echo $this->get_data_dir_url( 'log' ) . 'qalog.txt'; ?></p>
			<p><strong>cron_status:</strong><br><?php echo $cron_status; ?></p>
			<p><strong>max_execution_time:</strong><br><?php echo ini_get( 'max_execution_time' ); ?></p>
		</div>
		<!-- End debug -->

		<?php
	}
} // end of class
