<?php
/**
 * 
 *
 * @package qa_heatmap
 */

$qahm_admin_page_home = new QAHM_Admin_Page_Home();

class QAHM_Admin_Page_Home extends QAHM_Admin_Page_Base {

	// スラッグ
	const SLUG = QAHM_NAME . '-home';

	// nonce
	const NONCE_ACTION = self::SLUG . '-nonce-action';
	const NONCE_NAME   = self::SLUG . '-nonce-name';

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		parent::__construct();
		$this->regist_ajax_func( 'ajax_get_session_num' );
		$this->regist_ajax_func( 'ajax_get_realtime_list' );
		$this->regist_ajax_func( 'ajax_get_json' );
		$this->regist_ajax_func( 'ajax_url_to_page_id' );
	}

	/**
	 * 初期化
	 */
	public function enqueue_scripts( $hook_suffix ) {
		if( $this->hook_suffix !== $hook_suffix ||
			! $this->is_enqueue_jquery()
		) {
			return;
		}

		$css_dir_url = $this->get_css_dir_url();
		$js_dir_url  = $this->get_js_dir_url();
		
		// enqueue style
		$this->common_enqueue_style();
		wp_enqueue_style( QAHM_NAME . '-admin-page-home-common-css', $css_dir_url . 'admin-page-home-common.css', array( QAHM_NAME . '-reset' ), QAHM_PLUGIN_VERSION );	
		wp_enqueue_style( QAHM_NAME . '-admin-page-home-realtime-css', $css_dir_url. 'admin-page-home-realtime.css', null, QAHM_PLUGIN_VERSION );
		wp_enqueue_style( QAHM_NAME . '-admin-page-home-stats-css', $css_dir_url. 'admin-page-home-stats.css', null, QAHM_PLUGIN_VERSION );	
		wp_enqueue_style( QAHM_NAME . '-admin-page-home-ds-css', $css_dir_url. 'admin-page-home-ds.css', null, QAHM_PLUGIN_VERSION );
		wp_enqueue_style( QAHM_NAME . '-daterangepicker-css', $css_dir_url . 'lib/date-range-picker/daterangepicker.css', null, QAHM_PLUGIN_VERSION );
	
		// enqueue script
		$this->common_enqueue_script();
		wp_enqueue_script( QAHM_NAME . '-admin-page-realtime', $js_dir_url . 'admin-page-realtime.js', array( QAHM_NAME . '-admin-common' ), QAHM_PLUGIN_VERSION );
		wp_enqueue_script( QAHM_NAME . '-table', $js_dir_url . 'table.js', null, QAHM_PLUGIN_VERSION );
		wp_enqueue_script( QAHM_NAME . '-progress-bar', $js_dir_url . '/progress-bar-exec.js', null, QAHM_PLUGIN_VERSION );
		wp_enqueue_script( QAHM_NAME . '-chart', $js_dir_url . 'lib/chart/chart.min.js', null, QAHM_PLUGIN_VERSION, false );
		wp_enqueue_script( QAHM_NAME . '-moment', $js_dir_url . 'lib/moment/moment.min.js', array( QAHM_NAME . '-admin-common' ), QAHM_PLUGIN_VERSION, false );
		wp_enqueue_script( QAHM_NAME . '-moment-with-locales', $js_dir_url . 'lib/moment/moment-with-locales.min.js', array( QAHM_NAME . '-moment' ), QAHM_PLUGIN_VERSION, false );	
		wp_enqueue_script( QAHM_NAME . '-daterangepicker', $js_dir_url . 'lib/date-range-picker/daterangepicker.js', array( QAHM_NAME . '-moment', QAHM_NAME . '-moment-with-locales' ), QAHM_PLUGIN_VERSION, false );
		wp_enqueue_script( QAHM_NAME . '-admin-page-home-datasearch', $js_dir_url . 'admin-page-home-datasearch.js', null, QAHM_PLUGIN_VERSION );
		wp_enqueue_script( QAHM_NAME . '-admin-page-statistics', $js_dir_url . 'admin-page-statistics.js', array( QAHM_NAME . '-table', QAHM_NAME . '-chart', QAHM_NAME . '-moment-with-locales', QAHM_NAME . '-daterangepicker', QAHM_NAME . '-admin-page-home-datasearch' ), QAHM_PLUGIN_VERSION );
		wp_enqueue_script( QAHM_NAME . '-cap-create', $js_dir_url . 'cap-create.js', array( QAHM_NAME . '-admin-common' ), QAHM_PLUGIN_VERSION );
	
		// inline script
		$scripts = $this->get_common_inline_script();
		$scripts['nonce_api']    = wp_create_nonce( QAHM_Data_Api::NONCE_API );
		$scripts['wp_time_adj'] = get_option('gmt_offset');
		$scripts['wp_lang_set'] =  get_bloginfo('language');
		wp_add_inline_script( QAHM_NAME . '-common', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', obj );', 'before' );

		// localize
		$localize = $this->get_common_localize_script();
		$localize['table_tanmatsu'] 		= __( 'Device', 'qa-heatmap-analytics' );
		$localize['table_ridatsujikoku'] 	= __( 'Time Session End', 'qa-heatmap-analytics' );
		$localize['table_id'] 				= __( 'ID', 'qa-heatmap-analytics' );
		$localize['table_id_tooltip']		= __( 'An ID which was uniquely assigned by QA to a user.', 'qa-heatmap-analytics' );
		$localize['table_1page_me'] 		= __( 'First Page Viewed', 'qa-heatmap-analytics' );
		$localize['table_ridatsu_page'] 	= __( 'Last Page Viewed', 'qa-heatmap-analytics' );
		$localize['table_sanshoumoto'] 		= __( 'Referral', 'qa-heatmap-analytics' );
		$localize['table_pv'] 				= __( 'PVs', 'qa-heatmap-analytics' );
		$localize['table_site_taizaijikan'] = __( 'Time on Site', 'qa-heatmap-analytics' );
		$localize['table_saisei'] 			= __( 'Replay', 'qa-heatmap-analytics' );
		$localize['table_page_title'] 		= __( 'Page Title', 'qa-heatmap-analytics' );
		$localize['table_data_total'] 		= __( 'Data Total', 'qa-heatmap-analytics' );
		$localize['table_page_version'] 	= __( 'Page Version : Span', 'qa-heatmap-analytics' );


		$localize['graph_users'] 			= _x( 'Users', 'a label in a graph', 'qa-heatmap-analytics' );
		$localize['graph_sessions'] 		= _x( 'Sessions', 'a lebel in a graph', 'qa-heatmap-analytics' );
		$localize['graph_pvs'] 				= _x( 'Pageviews', 'a label in a graph', 'qa-heatmap-analytics' );

		$localize['calender_kinou'] 		= _x( 'Yesterday', 'a word in a date range picker', 'qa-heatmap-analytics' );
		$localize['calender_kako7days'] 	= _x( 'Last 7 Days', 'words in a date range picker', 'qa-heatmap-analytics' );
		$localize['calender_kako30days'] 	= _x( 'Last 30 Days', 'words in a date range picker', 'qa-heatmap-analytics' );
		$localize['calender_kongetsu'] 		= _x( 'This Month', 'words in a date range picker', 'qa-heatmap-analytics' );
		$localize['calender_sengetsu'] 		= _x( 'Last Month', 'words in a date range picker', 'qa-heatmap-analytics' );
		$localize['calender_erabu'] 		= _x( 'Custom Range', 'words in a date range picker', 'qa-heatmap-analytics' );
		$localize['calender_cancel'] 		= _x( 'Cancel', 'a word in a date range picker', 'qa-heatmap-analytics' );
		$localize['calender_ok'] 			= _x( 'Apply', 'a word in a date range picker', 'qa-heatmap-analytics' );
		$localize['calender_kara']			= _x( '-', 'a connector between dates in a date range picker', 'qa-heatmap-analytics' );

		$localize['download_msg1']			= __( 'Download the data from %1$s to %2$s.', 'qa-heatmap-analytics' );
		$localize['download_msg2']			= __( '*If the data size is too large, depending on the server, it may not be possible to download. In that case, try shortening the date range.', 'qa-heatmap-analytics' );
		$localize['download_done_nodata']	= __( 'No data between %1$s and %2$s.', 'qa-heatmap-analytics' );
		$localize['download_error1']		= __( 'A communication error occurred when acquiring data.', 'qa-heatmap-analytics' );
		$localize['download_error2']		= __( 'It may be acquired too much data. Please shorten the date range and try again. (It depends on the server, but in general, it would be better to make the total number of PVs for the period less than 10,000.)', 'qa-heatmap-analytics' );
		$localize['ds_download_msg1']		= __( 'Download the data of pageviews which filtered.', 'qa-heatmap-analytics' );
		$localize['ds_download_error1']		= __( 'No data to download.', 'qa-heatmap-analytics' );
		$localize['ds_download_done_nodata'] = __( 'No filtered data.', 'qa-heatmap-analytics' );


		$localize['ds_cyusyutsu_kensu'] 	= _x( 'session(s) found.', 'displaying the number of filtered sessions', 'qa-heatmap-analytics' );
		$localize['ds_cyusyutsu_button'] 	= _x( 'Filter and Search', 'value of the button in narrow-down-data-section', 'qa-heatmap-analytics' );
		$localize['ds_cyusyutsu_cyu'] 		= _x( 'Filtering...', 'value of the button in narrow-down-data-section', 'qa-heatmap-analytics' );
		$localize['ds_cyusyutsu_error1'] 	= _x( ': NO data found.', 'error message1 in narrow-down-data-section', 'qa-heatmap-analytics' );
		$localize['ds_cyusyutsu_error2'] 	= _x( 'A communication error occurred when acquiring data. It may be acquired too much data. Please narrow down the condition and try again.', 'error message2 in narrow-down-data-section', 'qa-heatmap-analytics' );
		$localize['ds_cyusyutsu_error3'] 	= _x( 'Showing for the last %d day(s) data because of too many data.', 'error message1 in narrow-down-data-section', 'qa-heatmap-analytics' );
		$localize['ds_free_plan_msg1']	 	= _x( 'Would like data from all the pages?', 'a message appears if only one page can be measured.', 'qa-heatmap-analytics' );
		$localize['ds_free_plan_msg2']	 	= _x( '%1$s Upgrade to a subscribed plan %2$s, to get all the pages recorded and measured automatically.', 'a message appears if only one page can be measured.', 'qa-heatmap-analytics' );


		$localize['tjs_open_filter']  			= esc_attr_x( 'Open all filter', 'a button appears above a table', 'qa-heatmap-analytics' );
		$localize['tjs_close_filter'] 			= esc_attr_x( 'Close all filter', 'a button appears above a table', 'qa-heatmap-analytics' );
		$localize['tjs_clear_filter'] 			= _x( 'CLEAR all filter', 'a-button-like words appears above a table', 'qa-heatmap-analytics' );
		$localize['tjs_komejirushi']	 		= _x( '*', 'a mark before a sentence', 'qa-heatmap-analytics' );
		$localize['tjs_howto_use_filter'] 		= _x( 'How to Use this QA Filter (written in Japanese)', 'words appears above a table', 'qa-heatmap-analytics' );
		$localize['tjs_sort']					= _x( 'Sort', 'word(s) in a table filter funcion', 'qa-heatmap-analytics' );
		$localize['tjs_filter'] 				= _x( 'Filter', 'word(s) in a table filter funcion', 'qa-heatmap-analytics' );
		$localize['tjs_word_for_filter'] 		= esc_attr_x( 'filtering word', 'word(s) in a table filter funcion', 'qa-heatmap-analytics' );
		$localize['tjs_include'] 				= _x( 'include', 'word(s) in a table filter funcion', 'qa-heatmap-analytics' );
		$localize['tjs_not_include'] 			= _x( 'exclude', 'word(s) in a table filter function', 'qa-heatmap-analytics' );
		$localize['tjs_select_all'] 			= _x( 'Select all', 'word(s) in a table filter function', 'qa-heatmap-analytics' );
		$localize['tjs_filter_equal'] 			= _x( '(equal)', 'word(s) in a table filter function', 'qa-heatmap-analytics' );
		$localize['tjs_filter_lt'] 				= _x( '(and less)', 'word(s) in a table filter function', 'qa-heatmap-analytics' );
		$localize['tjs_filter_gt'] 				= _x( '(and more)', 'word(s) in a table filter function', 'qa-heatmap-analytics' );
		$localize['tjs_filter_data_tani_currency'] = _x( 'yen', 'word(s) in a table filter function', 'qa-heatmap-analytics' );
		$localize['tjs_filter_data_tani_second']   = _x( 'second(s)', 'word(s) in a table filter function', 'qa-heatmap-analytics' );
		$localize['tjs_alert_msg_over500'] 		= _x( '"Select all" cannot be used when over 500 selectors exist. Please filter and make them less.', 'an alert in a table filter function', 'qa-heatmap-analytics' );
		$localize['tjs_alert_msg_over100']		= _x( '"Select all" cannot be used when over 100 selectors exist. Please filter and make them less.', 'an alert in a table filter function', 'qa-heatmap-analytics' );


		wp_localize_script( QAHM_NAME . '-common', QAHM_NAME . 'l10n', $localize );
	}

	/**
	 * ページの表示
	 */
	public function create_html() {
		if( ! $this->is_enqueue_jquery() ) {
			$this->view_not_enqueue_jquery_html();
			return;
		}
		
		if( $this->is_maintenance() ) {
			$this->view_maintenance_html();
			return;
		}

		$lang_set = get_bloginfo('language');	
		?>
		<div id="<?php esc_attr_e( basename( __FILE__, '.php' ) ); ?>" class="qahm-admin-page">
			<div class="wrap">
				<h1>QA <?php _e( 'Home', 'qa-heatmap-analytics' ); ?></h1>

				<?php $this->view_rss_feed(); ?>
				<?php $this->view_announce_html(); ?>

				<p><?php printf( esc_html__( 'See Help for the brief explanation of words such as "time on site". Or visit %1$s our support site %2$s (written in Japanese).', 'qa-heatmap-analytics' ), '<a href="https://mem.quarka.org/manual" target="_blank" rel="noopener">', '</a>' ); ?></p>


				<div class="tabs">
					<input id="tab_realtime" type="radio" name="tab_item" checked>
					<label class="tab_item" for="tab_realtime"><span style="margin-right: 4px;"><i class="far fa-clock"></i></span><?php _e( 'Real Time', 'qa-heatmap-analytics' ); ?></label>
					<input id="tab_stats" type="radio" name="tab_item">
					<label class="tab_item" for="tab_stats"><span style="margin-right: 4px;"><i class="fas fa-chart-bar"></i></span><?php _e( 'Overview', 'qa-heatmap-analytics' ); ?></label>
					<input id="tab_ds" type="radio" name="tab_item">
					<label class="tab_item" for="tab_ds"><span style="margin-right: 4px;"><i class="fas fa-search"></i></span><?php _e( 'Sessions & Heatmap', 'qa-heatmap-analytics' ); ?></label>


					<div class="tab_content" id="tab_realtime_content">
						<div class="tab_content_description">
							<div id="tday_container">

								<div id="tday_top">
									<div class="tday_now_box">
										<div class="tday_now_number">
											<div class="tday_now_title"><?php _e( 'Users in Last 30 Min.', 'qa-heatmap-analytics' ); ?></div>
											<div id="session_num" class="tday_now_rtn"></div>
										</div><!-- now_number_end -->
										<div class="tday_now_number">
											<div class="tday_now_title"><?php _e( 'Users Per Minute', 'qa-heatmap-analytics' ); ?></div>
											<div  id="session_num_1min" class="tday_now_rtn"></div>
										</div><!-- now_number_end -->


										<div class="tday_terminal_box">
										</div>
										<!-- terminal_box_end -->
									</div><!-- now_box_end -->
								</div><!-- top_end -->

								<hr>
								<div id="tday_upper">
									<div class="tday_upper_title">
										<div class="tday_title_icon tday_bg_red"><img src="<?php echo plugin_dir_url( __FILE__ ); ?>img/icon_01.png" alt="Today"></div>
										<div class="tday_upper_txt">
											<div class="tday_upper_eng color_red"><?php _e( 'Last update', 'qa-heatmap-analytics' ); ?>:
												<span id="update_time"></span>
												<script>
													// time();
													function time(){
														var now = new Date();
														var formatnow = now.toLocaleString().slice(0,-3);
														document.getElementById("time").innerHTML = formatnow;
													}
													// setInterval('time()',60000);
												</script>
											</div>
											<div class="tday_upper_jpn"><?php _e( 'Recorded Visiting in the Last 2 Days', 'qa-heatmap-analytics' ); ?></div>
											<p>
											<?php _e( '*There would be empty space below when no "recorded visiting" data to show.', 'qa-heatmap-analytics' ); ?>
										</p>
										</div>
									</div><!-- upper_title_end -->

									<div>
										<p><?php _e( 'To watch Session Replay, click a "play button" in each session row; Replay culumn.', 'qa-heatmap-analytics' ); ?><br>
										<?php _e( 'You can sort and filter the session data by using a "filter" function of the table. (Please don\'t forget to press "magnifying glass"-symbol for filtering!)', 'qa-heatmap-analytics' ); ?></p>
									</div>

									<!--<div class="tday_data_box">-->
									<!--<img src="--><?php //echo plugin_dir_url( __FILE__ ); ?><!--img/graf.jpg">-->
									<!--</div>--><!-- data_box_end -->
									<div id="tday-table-progbar"></div>
									<div class="tday_scroll-table" id="tday_table"></div>
								</div><!-- upper_end -->
								
							</div>
						</div>
					</div><!-- end of .tab_content, #tab_realtime_content-->


					<div class="tab_content" id="tab_stats_content">
							<div class="tab_content_description">
									<div id="float-scrn-prog"></div>

									<div id="stats_container" class="tab_container">

											<div>
													<div class="tab_content_item_thinbottom">
														<div class="left-blue_div">
															<p><span style="margin-right: 4px;"><i class="fas fa-calendar-check"></i></span><?php _e( 'Date Range', 'qa-heatmap-analytics' ); ?></p>
															<div class="datepicker-frame">
																	<div class="chart-reportrange">
																			<i class="fa fa-calendar"></i>&nbsp;
																			<span></span> <i class="fa fa-caret-down"></i>
																	</div>
															</div>
														</div>
														<h3 class="back-blue-grad-center_h3 text-center"><?php _e( 'Visits Graph', 'qa-heatmap-analytics' ); ?></h3>
															<div class="qa-chart-access">
																	<div id="chart1-legend" class="grf-legend"></div>
																	<div class="chart-container">
																	<canvas id="statsChart1"></canvas>
																	</div>
															</div>
															<div class="stats_textalign-right">
																<p style="font-size:90%;"><?php _e( '*If you are a non-subscription member,  the data retention period is the last two months. (The graph shows as zero before that.)', 'qa-heatmap-analytics' ); ?><br>
																　-> <a href="https://quarka.org/plan/" target="_blank" rel="noopener noreferrer"><?php _e( 'Upgrade to a subscribe plan.', 'qa-heatmap-analytics' ); ?></a><?php if ( $lang_set !== 'ja' ) { _e( '(written in Japanese)', 'qa-heatmap-analytics' ); } ?><p>
															</div>
													</div>
													<div class="tab_content_item">
															<h3 class="back-blue-grad_h3"><?php _e( 'Period Total', 'qa-heatmap-analytics' ); ?></h3>
															<div class="stats-open" id="stats-total">
																	<div class="qa-chart-access-view">
																			<div class="qa-view-count-box">
																					<span class="dashicons dashicons-groups"></span>
																					<span id="qa-num-readers" class="qa-view-count qa-count"><span style="font-size:24px;">counting...</span></span>
																					<span class="qa-view-count-info qa-stats-tooltip" data-statstt-text="<?php esc_attr_e( 'Total number of Users who have initiated at least one session during the date range.', 'qa-heatmap-analytics' ); ?>"><?php _e( 'Users', 'qa-heatmap-analytics' ); ?></span>
																			</div>
																			<div class="qa-view-count-box">
																					<span class="stats-icons"><i class="fas fa-door-open"></i></span>
																					<span id="qa-num-sessions" class="qa-view-count qa-count"><span style="font-size:24px;">counting...</span></span>
																					<span class="qa-view-count-info qa-stats-tooltip" data-statstt-text="<?php esc_attr_e( 'Total number of Sessions within the date range. A session is the period time a user is actively engaged with your website.', 'qa-heatmap-analytics' ); ?>"><?php _e( 'Sessions', 'qa-heatmap-analytics' ); ?></span>
																			</div>
																			<div class="qa-view-count-box">
																					<span class="dashicons dashicons-admin-page"></span>
																					<span id="qa-num-pvs" class="qa-view-count qa-count"><span style="font-size:24px;">counting...</span></span>
																					<span class="qa-view-count-info qa-stats-tooltip" data-statstt-text="<?php esc_attr_e( 'Total number of Pages Viewed. Repeated views of a single page are counted.', 'qa-heatmap-analytics' ); ?>"><?php _e( 'Pageviews', 'qa-heatmap-analytics' ); ?></span>
																			</div>
																	</div>
															</div>
													</div>

													<div class="stats_textalign-right">
														<input type="button" id="csv-download-btn" value="<?php esc_attr_e( 'Download TSV Data of PV', 'qa-heatmap-analytics' ); ?>">
													</div>
											</div>

											<div style="text-align: center;">
													<button type="button" id="extraction-view-button" class="button button-primary"><?php _e( 'Sessions & Heatmap', 'qa-heatmap-analytics' ); ?> <i class="fas fa-search"></i></button>
											</div>

									</div><!-- end of #stats_container -->

							</div>
					</div><!-- end of .tab_content, #tab_stats_content-->


					<div class="tab_content" id="tab_ds_content">
						<div class="tab_content_description">
<!--								<div id="extraction-view-container" style="display:none;">-->
								<div id="extraction-view-container" class="tab_container">

									<div class="tab_content_item_thinbottom ds_item">
									<div class="left-blue_div">										
										<h3><span style="margin-right: 4px;"><i class="fas fa-search"></i></span><?php _e( 'Narrow down the data', 'qa-heatmap-analytics' ); ?></h3>
										<div class="datepicker-frame">
											<div class="ds-chart-reportrange">
												<i class="fa fa-calendar"></i>&nbsp;
												<span></span> <i class="fa fa-caret-down"></i>
											</div>
										</div>
										<div>
											<input type="text" size="100" id="jsSearchPageUrl" placeholder="https://example.com/goal/"><label for="jsSearchPageUrl"> <?php _e( 'is the page to look for.', 'qa-heatmap-analytics' ); ?></label>
										</div>
										<div class="margin-eight">
											<button type="button" id="extraction-proc-button" class="button button-primary"><?php _e( 'Filter and Search', 'qa-heatmap-analytics' ); ?></button>
										</div>
										</div>
										<div>
											<p class="sankaku underline-double" id="cyusyutsu_notice"></p>
										</div>
									</div>

									<div class="tab_content_item ds_item">
										<h3 class="back-blue_h3"><span style="margin-right: 4px;"><i class="fas fa-caret-down"></i></span><?php _e( 'Sessions which include the page filtered', 'qa-heatmap-analytics' ); ?></h3>
										<p><?php _e( 'To watch Session Replay, click a "play button" in each session row.', 'qa-heatmap-analytics' ); ?></p>

										<div id="sday-table-progbar"></div>
										<div id="sday_table"></div>

									</div>

									<div class="stats_textalign-right">
											<input type="button" id="ds-csv-download-btn" value="<?php esc_attr_e( 'Download TSV Data of Filtered PV', 'qa-heatmap-analytics' ); ?>">
											<p style="font-size: 90%"><?php printf( esc_html__( '*You can download the whole data of pageviews on %1$s Overview section %2$s.', 'qa-heatmap-analytics' ),	'<a href="#" onclick="document.getElementById(\'tab_stats\').checked = true;return false;">', '</a>' );	?></p>									
									</div>

									<div class="tab_content_item ds_item">
										<h3 class="back-blue_h3"><span style="margin-right: 4px;"><i class="fas fa-caret-down"></i></span><?php _e( 'Heatmaps which found', 'qa-heatmap-analytics' ); ?></h3>
										<p><?php _e( 'Click the number in Device column to see the heatmap.', 'qa-heatmap-analytics' ); ?></p>
										<div>
											<div id="heatmap-table-progbar"></div>
											<div id="heatmap-table"></div>
											<table id="table-heatmap-list"></table>
										</div>
										<p style="font-size: 90%; text-align: right;"><?php _e( 'You can also see heatmaps from Heatmap Manager screen.', 'qa-heatmap-analytics' ); ?></p>

									</div>
								</div>

							</div>
						</div><!-- end of .tab_content, #tab_ds_content-->

					</div><!-- end of .tabs-->
				</div>
			</div><!-- end of .qahm-admin-page-->


<?php
	}

	/**
	 * セッション数の取得
	 */
	public function ajax_get_session_num() {
		if( $this->is_maintenance() ) {
			return;
		}

		$data = array();
		$session_num = 0;
		$session_num_1min = 0;

		global $wp_filesystem;
		global $qahm_time;
		$before1min = $qahm_time->now_unixtime() - 60;
		$session_temp_dir_path = $this->get_data_dir_path( 'readers/temp' );
		if( $wp_filesystem->exists( $session_temp_dir_path ) ) {

			$session_temp_dirlist = $this->wrap_dirlist( $session_temp_dir_path );
			if ( $session_temp_dirlist ) {
				$session_num = count( $session_temp_dirlist );
				foreach ( $session_temp_dirlist as $session_temp_fileobj ) {
					if ( $session_temp_fileobj['lastmodunix'] > $before1min ) {
						++$session_num_1min;
					}
				}
			}
		}

		$data['session_num']      = $session_num;
		$data['session_num_1min'] = $session_num_1min;

		echo wp_json_encode( $data );
		die();
	}


	/**
	 * リアルタイムリストの取得
	 */
	public function ajax_get_realtime_list() {
		if( $this->is_maintenance() ) {
			return;
		}

		$data = array();
		$alldataary = array();
		$realtime_list = '';

		global $wp_filesystem;
		global $qahm_time;

		$ellipsis     = '...';
		$title_width  = 80 + mb_strlen( $ellipsis );
		$domain_width = 30 + mb_strlen( $ellipsis );

		$realtime_view_path = $this->get_data_dir_path( 'readers' ) . 'realtime_view.php';
		if ( ! $wp_filesystem->exists( $realtime_view_path ) ) {
			echo 'null';
			die();
		}

		$realtime_view_ary = unserialize( $this->wrap_get_contents( $realtime_view_path ) );
		if ( ! $realtime_view_ary ) {
			echo 'null';
			die();
		}

		$realtime_cnt = count( $realtime_view_ary['body'] );
		if ( $realtime_cnt === 0 ) {
			echo 'null';
			die();
		}

		for ( $i = 0; $i < $realtime_cnt; $i++ ) {
			$body = $realtime_view_ary['body'][$i];
			$first_title        = $body['first_title'];
			$first_title_el     = mb_strimwidth( $first_title, 0, $title_width, $ellipsis );
			$first_url          = $body['first_url'];
			$last_title         = $body['last_title'];
			$last_title_el      = mb_strimwidth( $last_title, 0, $title_width, $ellipsis );
			$last_url           = $body['last_url'];
			$last_exit_time     = $body['last_exit_time'];
			$sec_on_site        = $qahm_time->seconds_to_timestr( (int) $body['sec_on_site'] );
			$referrer           = $body['first_referrer'];
			$source_domain_html = 'direct';
			$work_base_name     = pathinfo( $body['file_name'], PATHINFO_FILENAME );

			if ( ! empty( $referrer ) ) {
				if ( 0 === strncmp( $referrer, 'http', 4 ) ) {
					$parse_url          = parse_url( $referrer );
					$ref_host           = $parse_url['host'];
					$source_domain      = mb_strimwidth( $ref_host, 0, $domain_width, $ellipsis );
					$source_domain_html = '<a href="' . esc_url( $referrer ) . '" target="_blank" class="qahm-tooltip" data-qahm-tooltip="'. esc_url( $referrer ) . '">' . esc_html( $source_domain ) . '</a>';
				} else {
					$source_domain      = mb_strimwidth( $referrer, 0, $domain_width, $ellipsis );
					$source_domain_html = esc_html( $source_domain );
				}
			}

			$device = $body['device_name'];
			if ( 'dsk' === $device ) {
				$device = 'pc';
			}

			$dataary = [];
			$dataary[] = esc_html( $device );
			$dataary[] = esc_html( $last_exit_time );
			$dataary[] = esc_url( $first_url );
			$dataary[] = esc_attr( $first_title );
			$dataary[] = esc_html( $first_title_el );
			$dataary[] = esc_url( $last_url );
			$dataary[] = esc_attr( $last_title );
			$dataary[] = esc_html( $last_title_el );
			$dataary[] = esc_url( $referrer );
			$dataary[] = esc_html( $source_domain );
			$dataary[] = esc_html( $body['page_view'] );
			$dataary[] = esc_html( $body['sec_on_site'] );
			$dataary[] = esc_attr( $work_base_name );
			$alldataary[] = $dataary;
		}

		$data['update_time']   = $qahm_time->now_str();

		//$data['realtime_list'] = $realtime_list;
		$data['realtime_list'] = $alldataary;

		echo wp_json_encode( $data );
		die();
	}

	/**
	 * 
	*/
	public function ajax_get_json() {
		if( $this->is_maintenance() ) {
			return;
		}
		
		global $wp_filesystem;
		
		$cache_path = $this->get_data_dir_path() . 'cache/';
		if ( ! $wp_filesystem->exists( $cache_path ) ) {
			$wp_filesystem->mkdir( $cache_path );
		}
		
		// リストファイル読み込み
		$list_path = $cache_path . '/statistics_list.php';
		if ( $wp_filesystem->exists( $list_path ) ) {
			$list = unserialize( $wp_filesystem->get_contents( $list_path ) );
		} else {
			$time = new QAHM_Time();
			global $wpdb;

			$yesterday_str = $time->xday_str(-1);

			$table_name = $wpdb->prefix . 'qa_pv_log';
			$query      = 'SELECT pv_id,reader_id,page_id ,device_id,source_id,medium_id,campaign_id,session_no,access_time,pv,speed_msec,browse_sec,is_last,is_newuser,is_cv_session,flag_bit,version_id FROM ' . $table_name . ' WHERE  access_time between %s AND %s';
			$preobj     = $wpdb->prepare( $query,  '2020-08-01 00:00:00', $yesterday_str . ' 23:59:59' );
			$result     = $wpdb->get_results( $preobj );

			if ( ! empty( $result ) ) {
				$newary = array();
				foreach ($result as $idx => $row ) {
					$newary[$idx]['pv_id'] = $row->pv_id;
					// reader_id
					$newary[$idx]['reader_id'] = $row->reader_id;
					$table_name = $wpdb->prefix . 'qa_readers';
					$query      = 'SELECT UAos,UAbrowser FROM ' . $table_name . ' WHERE  reader_id = %d';
					$preobj     = $wpdb->prepare( $query, $row->reader_id );
					$select     = $wpdb->get_results( $preobj );
					$newary[$idx]['UAos'] = $select[0]->UAos;
					$newary[$idx]['UAbrowser'] = $select[0]->UAbrowser;

					// page_id
					$newary[$idx]['page_id'] = $row->page_id;
					$table_name = $wpdb->prefix . 'qa_pages';
					$query      = 'SELECT url,title FROM ' . $table_name . ' WHERE  page_id = %d';
					$preobj     = $wpdb->prepare( $query, $row->page_id );
					$select     = $wpdb->get_results( $preobj );
					$newary[$idx]['url'] = $select[0]->url;
					$newary[$idx]['title'] = esc_html( $select[0]->title );

					// device_id
					$newary[$idx]['device_id'] = $row->device_id;

					// source_id
					$newary[$idx]['source_id'] = $row->source_id;
					$table_name = $wpdb->prefix . 'qa_utm_sources';
					$query      = 'SELECT utm_source,source_domain FROM ' . $table_name . ' WHERE  source_id = %d';
					$preobj     = $wpdb->prepare( $query, $row->source_id );
					$select     = $wpdb->get_results( $preobj );
					$newary[$idx]['utm_source'] = $select[0]->utm_source;
					$newary[$idx]['source_domain'] = $select[0]->source_domain;

					// medium_id
					$newary[$idx]['medium_id'] = $row->medium_id;
					if ( $row->medium_id ) {
						$table_name = $wpdb->prefix . 'qa_utm_media';
						$query      = 'SELECT utm_medium FROM ' . $table_name . ' WHERE  medium_id = %d';
						$preobj     = $wpdb->prepare( $query, $row->medium_id );
						$select     = $wpdb->get_results( $preobj );
						$newary[$idx]['utm_medium'] = $select[0]->utm_medium;
					}

					// campaign_id
					$newary[$idx]['campaign_id'] = $row->campaign_id;
					if ( $row->campaign_id ) {
						$table_name = $wpdb->prefix . 'qa_utm_campaigns';
						$query      = 'SELECT utm_campaign FROM ' . $table_name . ' WHERE  campaign_id = %d';
						$preobj     = $wpdb->prepare( $query, $row->campaign_id );
						$select     = $wpdb->get_results( $preobj );
						$newary[$idx]['utm_campaign'] = $select[0]->utm_campaign;
					}

					// others
					$newary[$idx]['session_no']    = $row->session_no;
					$newary[$idx]['access_time']   = $row->access_time;
					$newary[$idx]['pv']            = $row->pv;
					$newary[$idx]['speed_msec']    = $row->speed_msec;
					$newary[$idx]['browse_sec']    = $row->browse_sec;
					$newary[$idx]['is_last']       = $row->is_last;
					$newary[$idx]['is_newuser']    = $row->is_newuser;
					$newary[$idx]['is_cv_session'] = $row->is_cv_session;
					$newary[$idx]['flag_bit']      = $row->flag_bit;
					$newary[$idx]['version_id']    = $row->version_id;
				}
			
				$list = wp_json_encode($newary) ;
				$wp_filesystem->put_contents( $list_path, serialize( $list ) );
			}
		}
		
		echo $list;
		die();
	}

	/**
	 * urlをpage idに変換して出力
	*/
	public function ajax_url_to_page_id() {
		/*
		$nonce = filter_input( INPUT_POST, 'nonce' );
		if ( ! wp_verify_nonce( $nonce, self::NONCE_REFRESH ) ) {
			http_response_code( 400 );
			die( 'wp_verify_nonce error' );
		}
		*/

		global $wpdb;

		// スラッシュあり、なしの両方を検索
		$url1 = filter_input( INPUT_POST, 'url' );
		if( substr( $url1, -1 ) === '/' ) {
			$url2 = rtrim( $url1, '/' );
		} else {
			$url2 = $url1 . '/';
		}
		
		$query = 'SELECT page_id FROM ' . $wpdb->prefix . 'qa_pages WHERE url IN(%s,%s)';
		$res   = $wpdb->get_results( $wpdb->prepare( $query, $url1, $url2 ) );

		if ( $res ) {
			echo wp_json_encode( $res[0]->page_id );
		}
		die();
	}
} // end of class
