<?php
/**
 *
 *
 * @package qa_heatmap
 */

new QAHM_Article_Post();

class QAHM_Article_Post extends QAHM_File_Base {

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		add_action( 'save_post', array( $this, 'save_post' ), 10, 3 );
		add_action( 'admin_head', array( $this, 'update_version' ) ); // save_postでキャプチャフラグを立てた場合に実行
	}


	/**
	 *
	 */
	public function save_post( $post_id, $post, $update ) {
		// 自動保存対策
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}
		// ゴミ箱に入れた時に更新されないよう対策
		if ( empty( $post ) ) {
			return;
		}
		// 記事新規作成画面を開いたときなどに、実行されないようにする対策
		if ( ! $update ) {
			return;
		}

		$type = 'p';
		if ( $post->post_type === 'page' ) {
			$type = 'page_id';
		}

		// ページのバージョンアップはrec_flagに関係なく行う
		// $post->guidはパーマリンクの設定とは違うURLが取得されるためコメントアウト

		// 今の仕様では自動バージョンアップ処理を入れない。よってこの部分をコメントアウトしている
		// ※後々実装予定
		/*
		$this->wrap_update_option(
			'post_update',
			array(
				'type'  => $type,
				'id'    => $post_id,
				//'url'   => $post->guid,
				'title' => $post->post_title
			)
		);
		*/

		// save_postのアクションフックは多重実行される可能性があるためremoveする
		remove_action( 'save_post', array( $this, 'update' ) );
	}


	/**
	 * 
	 */
	public function update_version() {
		$post_update = $this->wrap_get_option( 'post_update' );
		if ( ! $post_update ) {
			return;
		}
		$this->wrap_update_option( 'post_update', '' );

		global $wpdb;
		global $qahm_time;

		$type      = $post_update['type'];
		$id        = $post_update['id'];
		$url       = $this->get_base_url( $type, $id );
		$title     = $post_update['title'];
		$today_str = $qahm_time->today_str();
		$now_str   = $qahm_time->now_str();

		// db更新
		$table_name = $wpdb->prefix . 'qa_pages';
		$query      = 'select page_id from ' . $table_name . ' where url_hash=%s';
		$qa_pages   = $wpdb->get_results( $wpdb->prepare( $query, hash( 'fnv164', $url ) ) );

		// qa_pagesがこの時点で存在しなければINSERT
		// インサートはひとつずつループで行うと失敗する可能性があるので、バルクで行うよう処理を変更する必要あり
		$page_id = null;
		if ( $qa_pages ) {
			// url_hashで検索しているのでこの方法で良い
			$page_id = (int) $qa_pages[0]->page_id;
		} else {
			$wpdb->insert(
				$table_name,
				array(
					'tracking_id' => $this->get_tracking_id(),
					'wp_qa_type'  => $type,
					'wp_qa_id'    => $id,
					'url'         => $url,
					'url_hash'    => hash( 'fnv164', $url ),
					'title'       => $title,
					'update_date' => $today_str,
				),
				array(
					'%s',
					'%s',
					'%d',
					'%s',
					'%s',
					'%s',
					'%s',
				)
			);

			$page_id = $wpdb->insert_id;
		}

		if ( $page_id ) {
			// 最新バージョンを調べる。存在しなければバージョン1作成
			// 存在していれば全デバイスそのバージョンを作成＆新規バージョン追加
			$table_name           = $wpdb->prefix . 'qa_page_version_hist';
			$query                = 'select version_id,device_id,version_no from ' . $table_name . ' where page_id=%d and version_no=(select max(version_no) from ' . $table_name . ' where page_id=%d)';
			$qa_page_version_hist = $wpdb->get_results( $wpdb->prepare( $query, $page_id, $page_id ) );

			$dev_id_ary = array();
			$cur_ver_no = 0;

			if ( $qa_page_version_hist ) {
				foreach ( $qa_page_version_hist as $hist ) {
					$dev_id_ary[] = $hist->device_id;
					$cur_ver_no   = $hist->version_no;
				}
			}

			// 今のバージョンは全デバイス分データを作成
			foreach ( QAHM_DEVICES as $qahm_dev ) {
				$dev_name  = $this->device_id_to_device_name( $qahm_dev['id'] );
				$options   = $this->get_stream_options( $dev_name );
				$base_html = file_get_contents( $url, false, stream_context_create( $options ) );

				if ( $qa_page_version_hist ) {
					$find = false;
					if ( $dev_id_ary !== array() ) {
						foreach ( $dev_id_ary as $dev_id ) {
							if ( $qahm_dev['id'] === (int) $dev_id ) {
								$find = true;
								break;
							}
						}
					}

					// 現行バージョンで作られていないデバイスのversion histを作成
					if ( ! $find ) {
						$wpdb->insert(
							$table_name,
							array(
								'page_id'         => $page_id,
								'device_id'       => $qahm_dev['id'],
								'version_no'      => $cur_ver_no,
								'base_html'       => $base_html,
								'update_date'     => $today_str,
								'insert_datetime' => $now_str,
							),
							array(
								'%d',
								'%d',
								'%d',
								'%s',
								'%s',
								'%s',
							)
						);
					}
				}

				// バージョン追加
				$wpdb->insert(
					$table_name,
					array(
						'page_id'         => $page_id,
						'device_id'       => $qahm_dev['id'],
						'version_no'      => $cur_ver_no + 1,
						'base_html'       => $base_html,
						'update_date'     => $today_str,
						'insert_datetime' => $now_str,
					),
					array(
						'%d',
						'%d',
						'%d',
						'%s',
						'%s',
						'%s',
					)
				);
			}
		}
	}
}
