<?php
/**
 * rawデータ関連のajax通信処理をまとめたクラス
 *
 * @package qa_heatmap
 */

new QAHM_Behavioral_Data();

class QAHM_Behavioral_Data extends QAHM_File_Data {

	const NONCE_INIT       = 'init';
	const NONCE_BEHAVIORAL = 'behavioral';

	/**
	 * コンストラクタ
	 */
	public function __construct() {
		$this->regist_ajax_func( 'ajax_init_user_data' );
		$this->regist_ajax_func( 'ajax_update_msec' );
		$this->regist_ajax_func( 'ajax_record_behavioral_data' );
		
		add_action( 'init', array( $this, 'init_wp_filesystem' ) );
	}

	/**
	 * 初期化
	 */
	public function ajax_init_user_data() {
		try {
			if ( $this->is_maintenance() ) {
				throw new Exception( 'It is under maintenance.' );
			}

			$nonce = filter_input( INPUT_POST, 'nonce' );
			if ( ! wp_verify_nonce( $nonce, self::NONCE_INIT ) ) {
				throw new Exception( 'The nonce value is invalid.' );
			}

			global $qahm_log;
			global $qahm_time;
			global $wp_filesystem;

			$data = array();

			// urlをここで取得するとajaxのURLになるためjsから渡している
			$url           = filter_input( INPUT_POST, 'url' );
			$title         = filter_input( INPUT_POST, 'title' );
			$type          = filter_input( INPUT_POST, 'type' );
			$id            = filter_input( INPUT_POST, 'id' );
			$window_width  = filter_input( INPUT_POST, 'window_width' );
			$window_height = filter_input( INPUT_POST, 'window_height' );
			$country       = filter_input( INPUT_POST, 'country' );

			$ip            = filter_input( INPUT_ENV, 'REMOTE_ADDR', FILTER_VALIDATE_IP );
			$ua            = filter_input( INPUT_SERVER, 'HTTP_USER_AGENT' );
			if( ! $ua ) {
				$ua = $_SERVER['HTTP_USER_AGENT'];
			}
			$dev_name      = $this->user_agent_to_device_name( $ua );
			//$ref           = filter_input( INPUT_SERVER, 'HTTP_REFERER' );
			$ref           = filter_input( INPUT_POST, 'referrer' );
			if ( empty( $ref ) ) {
				//$ref           = filter_input( INPUT_SERVER, 'HTTP_REFERER' );
				$ref = 'direct';
			}
			$qa_id         = filter_input( INPUT_COOKIE, 'qa_id' );
			$qa_session    = filter_input( INPUT_COOKIE, 'qa_session' );
			$today_str     = $qahm_time->today_str();

			// validate
			if( ! $this->validate_qa_type( $type ) ) {
				throw new Exception( 'The value of $type is invalid.' );
			}
			if( ! $this->validate_number( $id ) ) {
				throw new Exception( 'The value of $id is invalid.' );
			}
			if( ! $this->validate_number( $window_width ) ) {
				throw new Exception( 'The value of $window_width is invalid.' );
			}
			if( ! $this->validate_number( $window_height ) ) {
				throw new Exception( 'The value of $window_height is invalid.' );
			}

			// utm_***の設定＆urlの一部パラメーターを削除して保存できるよう対応
			parse_str( parse_url( $url, PHP_URL_QUERY ), $query_ary );
			$utm_source       = array_key_exists( 'utm_source', $query_ary ) ? $query_ary[ 'utm_source' ] : '';
			$utm_medium       = array_key_exists( 'utm_medium', $query_ary ) ? $query_ary[ 'utm_medium' ] : '';
			$utm_campaign     = array_key_exists( 'utm_campaign', $query_ary ) ? $query_ary[ 'utm_campaign' ] : '';
			$utm_term         = array_key_exists( 'utm_term', $query_ary ) ? $query_ary[ 'utm_term' ] : '';
			$user_original_id = '';

			if ( array_key_exists( 'gclid', $query_ary ) ) {
				if ( ! $utm_source ) {
					$utm_source = 'google';
				}
				if ( ! $utm_medium ) {
					$utm_medium = 'cpc';
				}
			}

			if ( array_key_exists( 'fbclid', $query_ary ) ) {
				if ( ! $utm_medium ) {
					$utm_medium = 'social';
				}
				if ( ! $utm_source ) {
					$utm_source = 'facebook';
				}
			}
		
			$url = $this->opt_url_param( $url );

			$readers_dir        = $this->get_data_dir_path( 'readers' );
			$readers_temp_dir   = $this->get_data_dir_path( 'readers/temp/' );
			$readers_finish_dir = $this->get_data_dir_path( 'readers/finish/' );

			// sessionデータ作成
			$session_temp_ary = null;
			$is_new_user      = 0;

			if ( $qa_id ) {
				// 保存対象のセッションファイルを調べる。まずはtempディレクトリ
				$file_info = $this->get_latest_readers_file_info( $readers_temp_dir, $qa_id );

				if ( $file_info ) {
					$before_30min = $qahm_time->now_unixtime() - ( 60 * 30 );
					if ( $file_info['lastmodunix'] < $before_30min ) {
						// 作られてから30分以上経過している場合はcronが止まっている可能性があるので、session_no+1で新規ファイル作成
						if ( $file_info['day_str'] === $today_str ) {
							$session_num  = $file_info['session_num'] + 1;
						} else {
							$session_num = 1;
						}
						$readers_name = $qa_id . '_' . $today_str . '_' . $session_num;

					} else {
						// 作られてから30分経過していない場合は前のファイルに追記書き込み
						$session_num  = $file_info['session_num'];
						$readers_name = $qa_id . '_' . $file_info['day_str'] . '_' . $session_num;
						$session_temp_ary = unserialize( $this->wrap_get_contents( $readers_temp_dir . $readers_name . '.php' ) );
					}

				} else {
					// tempディレクトリにファイルがない場合はfinishディレクトリを確認
					$file_info = $this->get_latest_readers_file_info( $readers_finish_dir, $qa_id );
					
					if ( $file_info ) {
						if ( $file_info['day_str'] === $today_str ) {
							$session_num  = $file_info['session_num'] + 1;
						} else {
							$session_num = 1;
						}

					} else {
						$session_num  = 1;
					}

					$readers_name = $qa_id . '_' . $today_str . '_' . $session_num;
				}

			} else {
				$qa_id        = $qahm_time->now_str( 'ymdHis' ) . hash( 'fnv164', $ip . $window_width . $window_height . $ua );
				$session_num  = 1;
				$readers_name = $qa_id . '_' . $today_str . '_' . $session_num;
				setcookie( 'qa_id', $qa_id, time() + 60 * 60 * 24 * 365 * 2, '/' );

				// qa_idも_gaもcookieに存在していなければ新規ユーザー
				if ( ! filter_input( INPUT_COOKIE, '_ga' ) ) {
					$is_new_user = 1;
				}
			}
			
			setrawcookie( 'qa_session', $url, time() + 60 * 30, '/' );

			// session temp data
			if ( ! $session_temp_ary ) {
				$session_temp_ary = array();
				$session_temp_ary['head']['version']        = 1;
				$session_temp_ary['head']['tracking_id']    = $this->get_tracking_id();
				$session_temp_ary['head']['device_name']    = $dev_name;
				$session_temp_ary['head']['is_new_user']    = $is_new_user;
				$session_temp_ary['head']['user_agent']     = $ua;
				$session_temp_ary['head']['first_referrer'] = $ref;
				$session_temp_ary['head']['utm_source']     = $utm_source;
				$session_temp_ary['head']['utm_medium']     = $utm_medium;
				$session_temp_ary['head']['utm_campaign']   = $utm_campaign;
				$session_temp_ary['head']['utm_term']       = $utm_term;
				$session_temp_ary['head']['original_id']    = $user_original_id;
				$session_temp_ary['head']['country']        = $country;
			}

			$body = array(
				'page_url'    => $url,
				'page_title'  => $title,
				'page_type'   => $type,
				'page_id'     => $id,
				'access_time' => $qahm_time->now_unixtime(),
				'page_speed'  => 0,
			);

			if ( ! isset( $session_temp_ary['body'] ) ) {
				$session_temp_ary['body'] = array();
			}
			$data['readers_name']       = $readers_name;
			$data['readers_body_index'] = array_push( $session_temp_ary['body'], $body ) - 1;

			$this->wrap_put_contents( $readers_temp_dir . $readers_name . '.php', serialize( $session_temp_ary ) );

			// qahm測定対象外のページの場合は生データを作らない
			if ( $this->is_qahm_page( $type ) ) {

				// 読者のPV数を取得。検索上限PV数は10000（仮）
				$raw_dir = $this->get_raw_dir_path( $type, $id, $dev_name );
				$limit = 10000;
				for( $i = 1; $i < $limit; $i++ ) {
					if ( ! $wp_filesystem->exists( $raw_dir . $readers_name . '_' . $i . '-p.php' ) ) {
						$pv_num  = $i;
						break;
					}
				}
				
				$data['raw_name'] = $readers_name . '_' . $pv_num;
			}

			echo wp_json_encode( $data );

		} catch ( Exception $e ) {
			http_response_code( 500 );
			echo $e->getMessage();

		} finally {
			die();
		}
	}


	/**
	 * msecを更新
	 */
	public function ajax_update_msec() {
		try {
			$readers_name       = filter_input( INPUT_POST, 'readers_name' );
			$readers_body_index = (int) filter_input( INPUT_POST, 'readers_body_index' );
			$speed_msec         = (int) filter_input( INPUT_POST, 'speed_msec' );
			
			$readers_temp_dir = $this->get_data_dir_path( 'readers/temp/' );
			$readers_data_ary = unserialize( $this->wrap_get_contents( $readers_temp_dir . $readers_name . '.php' ) );

			if ( isset( $readers_data_ary['body'][$readers_body_index]['page_speed'] ) ) {
				$readers_data_ary['body'][$readers_body_index]['page_speed'] = $speed_msec;
				$this->wrap_put_contents( $readers_temp_dir . $readers_name . '.php', serialize( $readers_data_ary ) );
			}

		} catch ( Exception $e ) {
			http_response_code( 500 );
			echo $e->getMessage();

		} finally {
			die();
		}
	}

	/**
	 * URLパラメーターをsession tempに格納するurl用に最適化
	 * http_build_queryは強制的にエンコードされるので使わない
	 */
	private function opt_url_param( $url, $del_param_ary=Array() ){
		$url_exp = explode( '?', $url );

		if( ! isset( $url_exp[1] ) ){
			return $url;
		}
		parse_str( $url_exp[1], $query_ary );

		$query_str = '';
		foreach( $query_ary as $key => $value ){
			if(
				$key !== 'gclid' && 
				$key !== '_ga' && 
				$key !== 'uid' &&
				strpos( $key, 'utm_' ) !== 0 &&
				$key !== 'fbclid'
			){
				if ( $query_str ) {
					$query_str .= '&';
				}
				$query_str .= $key . '=' . $value;
			}
		}

		return $query_str ? $url_exp[0] . '?' . $query_str : $url_exp[0];
	}


	// 指定したQA IDを持つひとにとって、最新のreadersファイルを取得
	private function get_latest_readers_file_info( $tar_dir, $qa_id ) {
		global $qahm_time;

		// 一番新しいreadersファイルの情報を格納する配列
		$file_info = array();
		
		$file_list = $this->wrap_dirlist( $tar_dir );
		if ( $file_list ) {
			foreach ( $file_list as $file ) {
				if ( strpos( $file['name'], $qa_id . '_' ) === false ) {
					continue;
				}

				if ( ! preg_match( '/' . $qa_id . '_(.*)_(.*)\.php/', $file['name'], $match ) ) {
					continue;
				}
	
				$day_str     = $match[1];
				$session_num = (int) $match[2];

				if ( ! empty( $file_info ) ) {
					$xday_num = $qahm_time->xday_num( $file_info['day_str'], $day_str );

					if ( $xday_num > 0 ) {
						$session_num = 1;
					} elseif ( $xday_num === 0 ) {
						if ( $file_info['session_num'] >= $session_num ) {
							continue;
						}
					} else {
						continue;
					}
				}

				$file_info['name']        = $file['name'];
				$file_info['lastmodunix'] = $file['lastmodunix'];
				$file_info['session_num'] = $session_num;
				$file_info['day_str']     = $day_str;
			}
		}

		return empty( $file_info ) ? null : $file_info;
	}


	/**
	 * 行動データ作成
	 */
	public function ajax_record_behavioral_data() {
		try {
			if ( $this->is_maintenance() ) {
				throw new Exception( 'It is under maintenance.' );
			}

			$nonce = filter_input( INPUT_POST, 'nonce' );
			if ( ! wp_verify_nonce( $nonce, self::NONCE_BEHAVIORAL ) ) {
				throw new Exception( 'The nonce value is invalid.' );
			}

			global $qahm_log;
			global $wp_filesystem;

			$is_pos   = filter_input( INPUT_POST, 'is_pos' );
			$is_pos   = ( $is_pos === 'true' ) ? true : false;
			$is_click = filter_input( INPUT_POST, 'is_click' );
			$is_click = ( $is_click === 'true' ) ? true : false;
			$is_event = filter_input( INPUT_POST, 'is_event' );
			$is_event = ( $is_event === 'true' ) ? true : false;

			$raw_name       = filter_input( INPUT_POST, 'raw_name' );
			$type           = filter_input( INPUT_POST, 'type' );
			$id             = filter_input( INPUT_POST, 'id' );
			$ua             = filter_input( INPUT_POST, 'ua' );
			if( ! $ua ) {
				$ua = $_SERVER['HTTP_USER_AGENT'];
			}
			$dev_name       = $this->user_agent_to_device_name( $ua );
			$file_dir       = $this->get_raw_dir_path( $type, $id, $dev_name );

			// validate
			if ( ! $file_dir ) {
				throw new Exception( 'Failed to specify the directory for raw data.' );
			}
			if ( ! $this->validate_qa_type( $type ) ) {
				throw new Exception( 'The value of $type is invalid.' );
			}
			if ( ! $this->validate_number( $id ) ) {
				throw new Exception( 'The value of $id is invalid.' );
			}

			$output = 'output data /';
			if ( $is_pos ) {
				$percent_height = json_decode( filter_input( INPUT_POST, 'percent_height' ), true );
				$is_scroll_max  = filter_input( INPUT_POST, 'is_scroll_max' );

				// validate
				for ( $i = 0; $i < 100; $i++ ) {
					if ( ! $this->validate_number( $percent_height[$i] ) ) {
						throw new Exception( 'The value of $percent_height[$i] is invalid.' );
					}
				}

				// パスを作成
				$file_name = $raw_name . '-p';
				$file_path = $file_dir . $file_name . '.php';
				
				$file_contents = '';
				$file_contents = $this->get_file_data_string(
					array(
						self::DATA_HEADER_VERSION => self::DATA_CURRENT_VERSION['POS'],
					),
					$file_contents
				);

				for ( $i = 0; $i < 100; $i++ ) {
					if ( $percent_height[ $i ] > 0 ) {
						$file_contents = $this->get_file_data_string(
							array(
								self::DATA_POS_1['PERCENT_HEIGHT'] => $i,
								self::DATA_POS_1['TIME_ON_HEIGHT'] => $percent_height[ $i ],
							),
							$file_contents
						);
					}
				}
				if ( $is_scroll_max === 'true' ) {
					$file_contents = $this->get_file_data_string(
						array(
							self::DATA_POS_1['PERCENT_HEIGHT'] => 'a',
						),
						$file_contents
					);
				}

				// 位置データは毎回上書き
				if ( ! $wp_filesystem->put_contents( $file_path, $file_contents ) ) {
					$qahm_log->error( '$wp_filesystem->put_contents()に失敗しました。パス：' . $file_path );
				}

				$output .= ' p /';
			}


			if ( $is_click ) {
				$click_log      = json_decode( filter_input( INPUT_POST, 'click_log' ), true );

				// validate
				for ( $i = 0, $click_log_cnt = count( $click_log ); $i < $click_log_cnt; $i++ ) {
					if ( ! $this->validate_number( $click_log[$i][self::DATA_CLICK_1['SELECTOR_X']] ) ) {
						throw new Exception( 'The value of $click_log is invalid.' );
					}

					if ( ! $this->validate_number( $click_log[$i][self::DATA_CLICK_1['SELECTOR_Y']] ) ) {
						throw new Exception( 'The value of $click_log is invalid.' );
					}					
				}

				// パスを作成
				$file_name = $raw_name . '-c';
				$file_path = $file_dir . $file_name . '.php';

				// ファイルが存在しなければ1行目から
				$file_contents = '';
				if ( ! $wp_filesystem->exists( $file_path ) ) {
					$file_contents = $this->get_file_data_string(
						array(
							self::DATA_HEADER_VERSION => self::DATA_CURRENT_VERSION['CLICK'],
						),
						$file_contents
					);
				} else {
					$file_contents = $wp_filesystem->get_contents( $file_path );
				}

				for ( $i = 0, $click_log_cnt = count( $click_log ); $i < $click_log_cnt; $i++ ) {
					$file_contents = $this->get_file_data_string( $click_log[ $i ], $file_contents );
				}

				// ファイルに追記
				if ( ! $wp_filesystem->put_contents( $file_path, $file_contents ) ) {
					$qahm_log->error( '$wp_filesystem->put_contents()に失敗しました。パス：' . $file_path );
				}

				$output .= ' c /';
			}

			
			if( $is_event ){
				$window_inner_w = filter_input( INPUT_POST, 'window_inner_w' );
				$window_inner_h = filter_input( INPUT_POST, 'window_inner_h' );
				$event_log      = json_decode( filter_input( INPUT_POST, 'event_log' ), true );

				// validate
				if ( ! $this->validate_number( $window_inner_w ) ) {
					throw new Exception( 'The value of $window_inner_w is invalid.' );
				}

				if ( ! $this->validate_number( $window_inner_h ) ) {
					throw new Exception( 'The value of $window_inner_h is invalid.' );
				}

				for ( $i = 0, $event_log_cnt = count( $event_log ); $i < $event_log_cnt; $i++ ) {
					$event_type = $event_log[$i][self::DATA_EVENT_1['TYPE']];
					if ( strlen( $event_type ) !== 1 ) {
						throw new Exception( 'The value of $event_log is invalid.' );
					}

					if ( ! $this->validate_number( $event_log[$i][self::DATA_EVENT_1['TIME']] ) ) {
						throw new Exception( 'The value of $event_log is invalid.' );
					}

					switch ( $event_type ) {
						case 'c':
							if ( ! $this->validate_number( $event_log[$i][self::DATA_EVENT_1['CLICK_X']] ) ) {
								throw new Exception( 'The value of $event_log is invalid.' );
							}
		
							if ( ! $this->validate_number( $event_log[$i][self::DATA_EVENT_1['CLICK_Y']] ) ) {
								throw new Exception( 'The value of $event_log is invalid.' );
							}
							break;

						case 's':
							if ( ! $this->validate_number( $event_log[$i][self::DATA_EVENT_1['SCROLL_Y']] ) ) {
								throw new Exception( 'The value of $event_log is invalid.' );
							}
							break;

						case 'm':
							if ( ! $this->validate_number( $event_log[$i][self::DATA_EVENT_1['MOUSE_X']] ) ) {
								throw new Exception( 'The value of $event_log is invalid.' );
							}

							if ( ! $this->validate_number( $event_log[$i][self::DATA_EVENT_1['MOUSE_Y']] ) ) {
								throw new Exception( 'The value of $event_log is invalid.' );
							}
							break;
						
						case 'r':
							if ( ! $this->validate_number( $event_log[$i][self::DATA_EVENT_1['RESIZE_X']] ) ) {
								throw new Exception( 'The value of $event_log is invalid.' );
							}

							if ( ! $this->validate_number( $event_log[$i][self::DATA_EVENT_1['RESIZE_Y']] ) ) {
								throw new Exception( 'The value of $event_log is invalid.' );
							}
							break;
						
						default:
							throw new Exception( 'The value of $event_log is invalid.' );
					}
				}
				
				// パスを作成
				$file_name = $raw_name . '-e';
				$file_path = $file_dir . $file_name . '.php';

				// ファイルが存在しなければ1行目から
				$file_contents = '';
				if ( ! $wp_filesystem->exists( $file_path ) ) {
					$file_contents = $this->get_file_data_string(
						array(
							self::DATA_HEADER_VERSION            => self::DATA_CURRENT_VERSION['EVENT'],
							self::DATA_EVENT_1['WINDOW_INNER_W'] => $window_inner_w,
							self::DATA_EVENT_1['WINDOW_INNER_H'] => $window_inner_h,
						),
						$file_contents
					);
				} else {
					$file_contents = $wp_filesystem->get_contents( $file_path );
				}

				for ( $i = 0, $event_log_cnt = count( $event_log ); $i < $event_log_cnt; $i++ ) {
					$file_contents = $this->get_file_data_string( $event_log[ $i ], $file_contents );
				}

				if ( ! $wp_filesystem->put_contents( $file_path, $file_contents ) ) {
					$qahm_log->error( '$wp_filesystem->put_contents()に失敗しました。パス：' . $file_path );
				}
				$output .= ' e /';
			}

			// ファイルの整合性を合わせるためにファイルの保存はここで一気にする

			echo $output;
		
		} catch ( Exception $e ) {
			http_response_code( 500 );
			echo $e->getMessage();

		} finally {
			die();
		}
	}

	// 数値のチェック
	private function validate_qa_type( $qa_type ) {
		switch( $qa_type ) {
			case 'home':
			case 'page_id':
			case 'p':
			case 'cat':
			case 'tag':
			case 'tax':
				return true;
			default:
				return false;
		}
	}

	// 数値のチェック
	private function validate_number( $val, $type = 'numeric' ) {
		switch( $type ) {
			case 'numeric':
				return is_numeric( $val );
			case 'int':
				return is_int( $val );
			case 'float':
				return is_float( $val );
			default:
				return false;
		}
	}
} // end of class


