<?php
/**
 * qa関連のdataをひいてくるAPI
 * qahm-dbクラスを通じてデータを取得する。
 * 初期APIとしては、自プラグインデータ閲覧画面からのajaxリクエストを裁くAPIを想定するが、今後は、他のプラグインやウェブサービス上からデータを取得できるようにする可能性がある
 * （しかし、その場合はバージョン管理が必須。かつ要セキュリティ対応？プラグインはDB直接参照できるのでセキュリティは不要な気もする。そもそもデータは匿名化している）
 * どちらにせよ、初期はバージョン管理が容易な自プラグインのためのAPIだけを作成する。
 * 他所からの参照にはajaxだけでなくURLによるget やdeleteなどのmethod対応も考えられ、WordPressのRESTを使う可能性もあり、別ファイルで管理すべきだと思われる
 * @package qa_heatmap
 */

$qahm_data_api = new QAHM_Data_Api();
class QAHM_Data_Api extends QAHM_Db {
	/**
	 *
	 */
	const NONCE_API  = 'api';
	public function __construct() {
		$this->regist_ajax_func( 'ajax_select_data' );

		// wordpressはコアの初期化→dbの初期化→その他の初期化（プラグイン含む）といった流れになるので
		// コンストラクタの時点でおそらくwpdbが読み込まれているはず
		// ダメならフックを用いて以下の処理の実行タイミングを変えるべき imai
		global $wpdb;
		$this->prefix = $wpdb->prefix;
	}

	/**
	 * public function
	 */
	public function ajax_select_data() {
		$nonce = filter_input( INPUT_POST, 'nonce' );
		if ( ! wp_verify_nonce( $nonce, self::NONCE_API) || $this->is_maintenance() ) {
//			http_response_code( 400 );
//			die( 'nonce error' );
		}
		// 全パラメーターを取得する
		$table      = $this->alltrim( filter_input( INPUT_POST, 'table' ) );
		$column     = $this->alltrim( filter_input( INPUT_POST, 'select' ) );
		$date_or_id = mb_strtolower( trim( filter_input( INPUT_POST, 'date_or_id' ) ) );
		$count      = $this->alltrim( filter_input( INPUT_POST, 'count' ) );
		$where      = $this->alltrim( filter_input( INPUT_POST, 'where' ) );

		if ( $count === 'true' || $count == 1 ) {
			$count = true;
		} else {
			$count = false;
		}

		$resary     = $this->select_data( $table, $column, $date_or_id, $count, $where );
//		http_response_code( 409 );
//		die( (string)$resary );

		header("Content-type: application/json; charset=UTF-8");
		echo wp_json_encode($resary);
		die();
	}

	/**
	 * public function
	 */
	public function select_data( $table, $column, $date_or_id, $count = false, $where = '' ) {
		global $qahm_db;

		//table名の補完
		if ( strpos( $table, $this->prefix) === false ) {
			$table = $this->prefix . $table;
		}

		//必須フィールドのチェック
		$colname = [];
		$is_err  = false;
		$is_datetime = false;
		switch ($table) {
			case $this->prefix . 'view_pv':
			case $this->prefix . 'vr_view_pv':
				$colname['date'] = 'access_time';
				$colname['id']   = 'pv_id';
				$is_datetime     = true;
				break;
			
			case $this->prefix . 'summary_days_access':
				$colname['date'] = 'date';
				$colname['id']   = '';
				$is_datetime     = false;
				break;

			case $this->prefix . 'qa_page_version_hist':
				$colname['date'] = 'update_date';
				$colname['id'] = 'version_id';
				break;
				
			case $this->prefix . 'qa_readers':
				$colname['date'] = 'update_date';
				$colname['id']   = 'reader_id';
				break;
				
			case $this->prefix . 'qa_utm_sources':
				$colname['id'] = 'source_id';
				break;
			
			case $this->prefix . 'qa_pages':
				$colname['date'] = 'update_date';
				$colname['id']   = 'page_id';
				break;

			case $this->prefix . 'qa_pv_log':
				$colname['date'] = 'access_time';
				$colname['id']   = 'pv_id';
				$is_datetime     = true;
				break;

			case $this->prefix . 'vr_view_session':
				return $this->get_vr_view_session( $column, $date_or_id, $where, $count );

			default:
				$is_err = true;
				break;
		}
		if ( $is_err ) {
			http_response_code( 401 );
			die( 'table error' );
		}
		//カラムのチェック
		if ($column !== '*') {
			$table_allcol  = $this->show_column( $table );
			$columns       = explode(',', $column );
			if ( 1 < count($columns) && $count === true ) {
				http_response_code( 402 );
				die( 'count too many colmuns error' );
			}
			foreach ( $columns as $col ) {
				if (! in_array($col, $table_allcol)) {
					http_response_code( 402 );
					die( 'colmuns error' );
				}
			}
		}

		//カラムの完成
		if ( $count ) {
			$column = 'count(' . $column . ')';
		}

		// 最初のクエリを作成
		if ( preg_match('/^date\s*=\s*between\s*([0-9]{4}.[0-9]{2}.[0-9]{2})\s*and\s*([0-9]{4}.[0-9]{2}.[0-9]{2})$/', $date_or_id, $datestrs ) ) {
			//$basequery = 'select ' . $column . ' from ' . $table . ' where ' . $colname[ 'date' ] . ' = %s between %s';
			$basequery = 'select ' . $column . ' from ' . $table . ' where ' . $colname[ 'date' ] . ' between %s and %s';
			$aday = $datestrs[1];
			$bday = $datestrs[2];
			if ( $is_datetime ) {
				if ( strpos( $aday, ':') === false ) {
					$aday = $aday . ' 00:00:00';
				}
				if ( strpos( $bday, ':') === false ) {
					$bday = $bday . ' 23:59:59';
				}
			}
			$query  = $this->prepare( $basequery, $aday, $bday );
		} elseif ( preg_match('/^id\s*=\s*([0-9]*)$/', $date_or_id, $idnum ) ) {
			$basequery = 'select ' . $column . ' from ' . $table . ' where ' . $colname[ 'id' ] . ' = %d';
			$query = $this->prepare( $basequery, $idnum[1] );
		} else {
			$is_err = true;
		}
		if ( $is_err ) {
			http_response_code( 408 );
			die( $aday . $bday . 'date_or_id error' );
		}

		if ( $count ) {
			return $this->get_var( $query );
		} else {
			return $this->get_results( $query );
		}
	}

	/**
	 * private function
	 */
	private function alltrim( $string ) {
		return str_replace(' ', '', $string);
	}

	private function get_all_parameter() {
		$postarray = array();
		if ( isset( $_POST ) && count( $_POST ) ) {
			$postarray = $_POST;
		}

		$getarray = array();
		if( isset( $_GET ) && count( $_GET ) ){
			$getarray = $_GET;
		}

		$paramarray = array_merge( $getarray, $postarray );

		foreach ( $paramarray as $key => $value ) {
			$paramarray[ $key ] = $this->h( $value );
		}
		return $paramarray;
	}

	private function get_get_parameter() {
		$getarray = array();
		if( isset( $_GET ) && count( $_GET ) ){
			$getarray = $_GET;
		}

		$paramarray = $getarray;

		foreach ( $paramarray as $key => $value ) {
			$paramarray[ $key ] = $this->h( $value );
		}
		return $paramarray;
	}

	private function get_post_parameter() {
		$postarray = array();
		if ( isset( $_POST ) && count( $_POST ) ) {
			$postarray = $_POST;
		}

		$paramarray = $postarray;

		foreach ( $paramarray as $key => $value ) {
			$paramarray[ $key ] = $this->h( $value );
		}
		return $paramarray;
	}


	/**
	 * private function
	 */
	private function h( $string ) {
		if ( is_array( $string ) ) {
			return array_map( "$this->h", $string );
		} else {
			return htmlspecialchars( $string, ENT_QUOTES, 'UTF-8' );
		}
	}
}
