<?php
/**
 * QAデータ（DBやファイルに入っている）にアクセスするためのラッパークラス
 * wpdbと似たコマンドを揃え、SQLに対応するが、セキュリティ対策も含め、決まったいくつかのコマンド(限定されたSELECTなど）しか受け付けないようにする。
 * またwpdbのprepareは単にSQLインジェクション対策を施したSQL文字列を返すだけなので（つまり純粋なDBのprepareとは違う）、そのまま活用することができる。
 * QAの関数群は、このqahm_dbを利用することでデータがどこに保存されているのかを意識せずに任意のデータをひくことができる。
 * @package qa_heatmap
 */

$qahm_db = new QAHM_Db();
class QAHM_Db extends QAHM_File_Data {
	
	public $prefix;

	/**
	 * const
	 */
	const QAHM_VIEW_PV_COL = array (
		'pv_id',
		'reader_id',
		'UAos',
		'UAbrowser',
		'page_id',
		'url',
		'title',
		'device_id',
		'source_id',
		'utm_source',
		'source_domain',
		'medium_id',
		'utm_medium',
		'campaign_id',
		'utm_campaign',
		'session_no',
		'access_time',
		'pv',
		'speed_msec',
		'browse_sec',
		'is_last',
		'is_newuser',
		'version_id',
		'is_raw_p',
		'is_raw_c',
		'is_raw_e',
	);
	
	const QAHM_VR_VIEW_PV_COL = array (
		'pv_id',
		'reader_id',
		'UAos',
		'UAbrowser',
		'page_id',
		'url',
		'title',
		'device_id',
		'source_id',
		'utm_source',
		'source_domain',
		'medium_id',
		'utm_medium',
		'campaign_id',
		'utm_campaign',
		'session_no',
		'access_time',
		'pv',
		'speed_msec',
		'browse_sec',
		'is_last',
		'is_newuser',
		'version_id',
		'is_raw_p',
		'is_raw_c',
		'is_raw_e',
		'version_no',
	);

	public function __construct() {
		$mkdummy = 1;

		// wordpressはコアの初期化→dbの初期化→その他の初期化（プラグイン含む）といった流れになるので
		// コンストラクタの時点でおそらくwpdbが読み込まれているはず
		// ダメならフックを用いて以下の処理の実行タイミングを変えるべき imai
		global $wpdb;
		$this->prefix = $wpdb->prefix;
	}

	/**
	 * prepare
	 */
	public function prepare( $query, ...$args ) {
		global $wpdb;
		return $wpdb->prepare( $query, ...$args );
	}

	/**
	 * result
	 */
	public function get_results( $query = null, $output = OBJECT ) {
		global $wpdb;

		//switch function from table name
		$tb_view_pv    = $this->prefix . 'view_pv';
		$tb_vr_view_pv = $this->prefix . 'vr_view_pv';
		$tb_view_ver   = $this->prefix . 'view_page_version_hist';
		$tb_days_access = $this->prefix . 'summary_days_access';
		$tb_days_access_detail = $this->prefix . 'summary_days_access_detail';
		if ( preg_match('/from ' . $tb_vr_view_pv . '/i', $query ) ) {
			return $this->get_results_view_pv( $query, true );
		} elseif ( preg_match('/from ' . $tb_view_pv . '/i', $query ) ) {
			return $this->get_results_view_pv( $query );
		} elseif ( preg_match('/from ' . $tb_view_ver . '/i', $query ) ) {
			return $this->get_results_view_page_version_hist( $query );
//		} elseif( preg_match('/from ' . $tb_days_access_detail . ' /i', $query ) ) {
//			return $this->get_results_days_access_detail( $query );
		} elseif( preg_match('/from ' . $tb_days_access . ' /i', $query ) ) {
			return $this->get_results_days_access( $query );
        } else {
			return $wpdb->get_results( $query, $output );
		}
	}


	/**
	 * result
	 */
	public function get_vr_view_session( $column, $date, $where, $count = false ) {
		global $wp_filesystem;
		global $qahm_time;
		/*

		メモ

		idの優先順位（数が少ない方から優先 / いまはpage_idのみ対応）
		reader_id
		page_id
		version_id
		campaign_id
		source_id
		medium_id

		*/

		// date 必須のため、適切な形になっていないならnullを返す
		// もしも単一の日付で検索することが今後あるなら「if ( strptime( $date_or_id, '%Y-%m-%d' ) ) {」で判定すれば良さそう
		if ( ! preg_match( '/^date\s*=\s*between\s*([0-9]{4}.[0-9]{2}.[0-9]{2})\s*and\s*([0-9]{4}.[0-9]{2}.[0-9]{2})$/', $date, $date_strings ) ) {
			null;
		}

		$s_daystr   = $date_strings[1];
		$e_daystr   = $date_strings[2];
		if ( ! $s_daystr || ! $e_daystr ) {
			return null;
		}
		$s_unixtime = $qahm_time->str_to_unixtime( $s_daystr . ' 00:00:00' );
		$e_unixtime = $qahm_time->str_to_unixtime( $e_daystr . ' 23:59:59' );

		$date_period_ary = new DatePeriod(
			new DateTime( $s_daystr . ' 00:00:00' ),
			new DateInterval('P1D'),
			new DateTime( $e_daystr . ' 23:59:59' )
	   );

		// column配列を作成
		$column = str_replace( ' ', '', $column );
		$column_ary = explode( ',', $column );

		$view_dir = $this->get_data_dir_path( 'view' );
		$traking_id = $this->get_tracking_id();
		$viewpv_dir  = $view_dir . $traking_id . '/view_pv/';
		$viewpv_idx_dir  = $viewpv_dir . 'index/';
		$verhist_dir = $view_dir . $traking_id . '/version_hist/';


		// AND に対応するときは事前にexplode関数で配列に分割するとよさそう

		// whereの構文が対応不可ならnullを返す
		preg_match_all( '/[^ =]+/', $where, $matches );
		if ( ! isset( $matches[0][0] ) || ! isset( $matches[0][1] ) ) {
			return null;
		}

		$id_type = $matches[0][0];
		$id_num  = $matches[0][1];
		if ( ! is_numeric( $id_num ) ) {
			return null;
		}
		$id_num = (int) $id_num;

		$file_name  = '';
		switch ( $id_type ) {
		case 'page_id':
			$file_name = '_pageid.php';
			break;
		default:
			return null;
		}

		$search_range = 100000;
		$search_max   = 10000000;
		if ( $id_num > $search_max ) {
			return null;
		}
		for ( $i = 1; $i < $search_max; $i += $search_range ) {
			if ( $i <= $id_num && $i + $search_range > $id_num ) {
				$file_name = $i . '-' . ( $i + $search_range - 1 ) . $file_name;
				break;
			}
		}

		if ( ! $wp_filesystem->exists( $viewpv_idx_dir . $file_name ) ) {
			return null;
		}

		$pageid_idx_ary = unserialize( $this->wrap_get_contents( $viewpv_idx_dir . $file_name ) );
		$viewpv_idx_ary = array();

		foreach ( $date_period_ary as $value ) {
			$date_period = $value->format('Y-m-d');
			$viewpv_idx_ary[$date_period] = $pageid_idx_ary[$id_num][$date_period];
		}

		$viewpv_ary = null;
		$viewpv_dirlist = $this->wrap_dirlist( $viewpv_dir );
		if ( ! $viewpv_dirlist ) {
			return null;
		}
		
		$ret_ary = array();
		$ret_cnt = 0;
		$searched_pv_id = array();
		$view_pv_idx_ary_keys = array_keys( $viewpv_idx_ary );
		$view_pv_idx_cnt = 0;
		$view_pv_idx_max = count( $viewpv_idx_ary );
		foreach ( $viewpv_dirlist as $viewpv_fileobj ) {
			$view_pv_file_name = $viewpv_fileobj['name'];

			// この時点でbetweenの最小値、最大値と日付比較を行うことで、更に高速化できそう

			for ( $i = $view_pv_idx_cnt; $i < $view_pv_idx_max; $i++ ) {
				$key = $view_pv_idx_ary_keys[$i];
				if( substr( $view_pv_file_name, 0, 10 ) !== $key ) {
					continue;
				}

				$viewpv_ary = unserialize( $this->wrap_get_contents( $viewpv_dir . $viewpv_fileobj['name'] ) );
				
				foreach ( $viewpv_ary as $idx => $viewpv ) {

					foreach ( $viewpv_idx_ary[$key] as $viewpv_idx ) {
						if ( (int) $viewpv['pv_id'] === $viewpv_idx ) {
							// 前回調べたか確認し、既に調べていたら（セッションに組み込まれていたら）スルー
							if ( ! empty( $searched_pv_id ) ) {
								$find = false;
								foreach( $searched_pv_id as $pv_id ) {
									if ( $pv_id === $viewpv['pv_id'] ) {
										$find = true;
										break;
									}
								}
								if( $find ) {
									break;
								}
							}
							$searched_pv_id = array();	// 毎回調べる度にクリアする
	
							// セッションの構築
							$session_ary = array();
							$pv_no = (int) $viewpv['pv'];
	
							// 前のPV
							if ( $pv_no > 1 ) {
								$pv_idx = $idx - $pv_no + 1;
								while ( $pv_idx < $idx ) {
									if ( $count ) {
										$ret_cnt++;
									} else {
										if ( $column === '*' ) {
											if ( $viewpv_ary[$pv_idx]['version_id'] ) {
												$verhist_filename = $viewpv_ary[$pv_idx]['version_id'] . '_version.php';
												$verhist_file = $this->wrap_get_contents( $verhist_dir . $verhist_filename );
												if ( $verhist_file ) {
													$verhist_ary = unserialize( $verhist_file );
													$viewpv_ary[$pv_idx]['version_no'] = $verhist_ary[0]->version_no;
												}
											}
	
											$session_ary[] = $viewpv_ary[$pv_idx];
										} else {
											$temp_ary = array();
											foreach ( $column_ary as $column_val ) {
												switch ( $column_val ) {
													case 'version_no':
														if ( $viewpv_ary[$pv_idx]['version_id'] ) {
															$verhist_filename = $viewpv_ary[$pv_idx]['version_id'] . '_version.php';
															$verhist_file = $this->wrap_get_contents( $verhist_dir . $verhist_filename );
															if ( $verhist_file ) {
																$verhist_ary = unserialize( $verhist_file );
																$temp_ary[$column_val] = $verhist_ary[0]->version_no;
															}
														}
														break;
														
													default:
														$temp_ary[$column_val] = $viewpv_ary[$pv_idx][$column_val];
														break;
												}
											}
											$session_ary[] = $temp_ary;
										}
									}
									$pv_idx++;
								}
							}
	
							// 今以降のPV。念のためPVのループ上限は10000に設定
							$first_search = true;
							for ( $pv_cnt = 0; $pv_cnt < 10000; $pv_cnt++ ) {
								$pv_idx = $idx + $pv_cnt;
								if ( $first_search ) {
									$first_search = false;
								} else {
									$searched_pv_id[] = $viewpv_ary[$pv_idx]['pv_id'];
								}
	
								if ( $count ) {
									$ret_cnt++;
								}
								else {
									if ( $column === '*' ) {
										if ( $viewpv_ary[$pv_idx]['version_id'] ) {
											$verhist_filename = $viewpv_ary[$pv_idx]['version_id'] . '_version.php';
											$verhist_file = $this->wrap_get_contents( $verhist_dir . $verhist_filename );
											if ( $verhist_file ) {
												$verhist_ary = unserialize( $verhist_file );
												$viewpv_ary[$pv_idx]['version_no'] = $verhist_ary[0]->version_no;
											}
										}

										$session_ary[] = $viewpv_ary[$pv_idx];
									} else {
										$temp_ary = array();
										foreach ( $column_ary as $column_val ) {
											switch ( $column_val ) {
												case 'version_no':
													if ( $viewpv_ary[$pv_idx]['version_id'] ) {
														$verhist_filename = $viewpv_ary[$pv_idx]['version_id'] . '_version.php';
														$verhist_file = $this->wrap_get_contents( $verhist_dir . $verhist_filename );
														if ( $verhist_file ) {
															$verhist_ary = unserialize( $verhist_file );
															$temp_ary[$column_val] = $verhist_ary[0]->version_no;
														}
													}
													break;

												default:
													$temp_ary[$column_val] = $viewpv_ary[$pv_idx][$column_val];
													break;
											}
										}
										$session_ary[] = $temp_ary;
									}
								}

								if ( $viewpv_ary[ $pv_idx ]['is_last'] ) {
									break;
								}
							}

							$ret_ary[] = $session_ary;
						}
					}
				}

				$view_pv_idx_cnt++;
			}
			if ( $view_pv_idx_cnt >= $view_pv_idx_max ) {
				break;
			}
		}

		if ( $count ) {
			return $ret_cnt;
		} else {
			if ( empty( $ret_ary ) ) {
				return null;
			} else {
				return $ret_ary;
			}
		}
	}


	public function query( $query ) {
		global $wpdb;
		return $wpdb->query( $query );
	}

	// get_varに問題がありそうなので使わなくなった
	public function get_var( $query = null, $x = 0, $y = 0 ) {
		global $wpdb;

		//switch function from table name
		$tb_view_pv    = $this->prefix . 'view_pv';
		$tb_vr_view_pv = $this->prefix . 'vr_view_pv';
		$tb_view_ver   = $this->prefix . 'view_page_version_hist';
		if ( preg_match('/from ' . $tb_vr_view_pv . '/i', $query ) ) {
			return $this->get_results_view_pv( $query, true );
		} elseif ( preg_match('/from ' . $tb_view_pv . '/i', $query ) ) {
			return $this->get_results_view_pv( $query );
		} elseif ( preg_match('/from ' . $tb_view_ver . '/i', $query ) ) {
			return $this->get_results_view_page_version_hist( $query );
		} else {
			return $wpdb->get_var( $query, $x, $y );
		}
	}

	/**
	 * useful
	 */
	public function show_column( $table ) {
		global $wpdb;
		$retary = [];
		switch ($table) {
			case 'view_pv':
				$retary = self::QAHM_VIEW_PV_COL;
				break;
				
			case 'vr_view_pv':
				$retary = self::QAHM_VR_VIEW_PV_COL;
				break;

			default:
				$is_table = false;
				foreach ( QAHM_DB_OPTIONS as $table_ver_name ) {
					$tablename = str_replace('_version', '', $table_ver_name);
					if ($tablename === $table) {
						$is_table = true;
					}
				}
				if ($is_table) {
					$res = $wpdb->get_results(' columns from '.$tablename);
					foreach ($res as $line) {
						$retary[] = $line['Field'];
					}
				}
				break;
		}
		return $retary;
	}
	/**
	 * protected
	 */
	protected function get_results_view_pv( $query = null, $is_vr = false ) {
		global $qahm_time;

		//view_pv table is in file
		$data_dir = $this->get_data_dir_path();
		$traking_id = $this->get_tracking_id();
		$view_dir = $data_dir . 'view/';
		//view_pv dirlist
		$viewpv_dir  = $view_dir . $traking_id . '/view_pv/';
		$viewpv_idx_dir  = $viewpv_dir . '/index/';
		$allfiles = $this->wrap_dirlist( $viewpv_dir );
		$verhist_dir = $view_dir . $traking_id . '/version_hist/';

		// columns
		preg_match( '/select (.*) from/i', $query, $sel_columns );
		$sel_column = str_replace( ' ', '', $sel_columns[1] );
		$columns_ary = [];
		if ( $sel_column === '*' ) {
			$columns_ary[0] = '*';
		} elseif ( preg_match('/count\((.*)\)/', $sel_column,$counts ) ) {
			$columns_ary[0] = 'count';
			$columns_ary[1] = $counts[1];
		} else {
			$columns_ary = explode( ',', $sel_column );
		}
		$is_error = false;
		$results_ary = [];
		$countall   = 0;
		// where date
		if (preg_match('/where access_time between.*([0-9]{4}.[0-9]{2}.[0-9]{2}.[0-9]{2}.[0-9]{2}.[0-9]{2}).*and.*([0-9]{4}.[0-9]{2}.[0-9]{2}.[0-9]{2}.[0-9]{2}.[0-9]{2})/i', $query, $date_strings )) {
			$s_unixtime = $qahm_time->str_to_unixtime( $date_strings[1] );
			$e_unixtime = $qahm_time->str_to_unixtime( $date_strings[2] );

			foreach ( $allfiles as $file ) {
				$filename = $file['name'];
				if ( is_file( $viewpv_dir . $filename ) ) {
					$f_date   = substr( $filename, 0, 10) . ' 00:00:00';

					$f_unixtime = $qahm_time->str_to_unixtime( $f_date );
					if ( $s_unixtime <= $f_unixtime && $f_unixtime <= $e_unixtime ) {
						$tmpary = unserialize( $this->wrap_get_contents( $viewpv_dir . $filename ) );
						//データは連想配列に入っている
						//取得するデータによって処理をわける
						switch ( $columns_ary[0] ) {
							case 'count':
								if ( $columns_ary[1] === '*' ){
									$countall = $countall + count( $tmpary );
								} else {
									foreach ( $tmpary as $bodyary ) {
										if ( $bodyary[$columns_ary[1]] ){
											++$countall;
										}
									}
								}
								break;

							case '*':
								foreach ( $tmpary as &$bodyary ) {
									if ( $is_vr ) {
										$verhist_filename = $bodyary['version_id'] . '_version.php';
										$verhist_file = $this->wrap_get_contents( $verhist_dir . $verhist_filename );
										if ( $verhist_file ) {
											$verhist_ary = unserialize( $verhist_file );
											$bodyary['version_no'] = $verhist_ary[0]->version_no;
										}
									}
								}
								$results_ary[] = $tmpary;
								break;

							default:
								foreach ( $tmpary as $bodyary ) {
									$lineary = [];
									foreach ($columns_ary as $column ) {
										if ( $is_vr ) {
											switch ( $column ) {
												case 'version_no':
													$verhist_filename = $bodyary['version_id'] . '_version.php';
													$verhist_file = $this->wrap_get_contents( $verhist_dir . $verhist_filename );
													if ( $verhist_file ) {
														$verhist_ary = unserialize( $verhist_file );
														$lineary[$column] = $verhist_ary[0]->version_no;
													}
													break;

												default:
													$lineary[$column] = $bodyary[$column];
													break;
											}
										} else {
											$lineary[$column] = $bodyary[$column];
										}
									}
									$results_ary[] = $lineary;
								}
								break;
						}
					}
				}
			}


		// where pv_id
		} elseif ( preg_match('/where pv_id = ([0-9]*)/i', $query, $pvidary ) ){
			$pv_id = (int) $pvidary[1];
			for ( $iii = 0; $iii < count( $allfiles ); $iii++ ) {
				$filename = $allfiles[$iii]['name'];
				$fnameexp = explode('_', $filename);
				$pvno_exp = explode('-', $fnameexp[1]);
				$s_pvidno = (int) $pvno_exp[0];
				$e_pvidno = (int) $pvno_exp[1];
				if ($s_pvidno <= $pv_id && $pv_id <= $e_pvidno ){
					$tmpary = unserialize( $this->wrap_get_contents( $viewpv_dir . $filename ) );
					//データは連想配列に入っている
					//取得するデータによって処理をわける
					for ($jjj = 0; $jjj < count($tmpary); $jjj++) {
						if ( (int)$tmpary[$jjj]['pv_id'] === $pv_id ) {
							switch ( $columns_ary[0] ) {
								case 'count':
									if ( $columns_ary[1] === '*' ){
										$countall = 1;
									} else {
										if ( $tmpary[$jjj][$columns_ary[1]] ){
											$countall = 1;
										}
									}
									break;

								case '*':
									$results_ary[] = $tmpary[$jjj];
									break;

								default:
									$lineary = [];
									foreach ($columns_ary as $column ) {
										$lineary[$column] = $tmpary[$jjj][$column];
									}
									$results_ary[] = $lineary;
									break;
							}
							$jjj = 	count($tmpary) + 888;
							$iii = 	count( $allfiles ) + 888;
						}
					}
				}
			}
		
		// where version_id
		} elseif ( preg_match('/where version_id = ([0-9]*)/i', $query, $veridary ) ){
			$version_id = (int) $veridary[1];

			$veridx_file = $this->get_index_file_contents( $viewpv_idx_dir, 'versionid', $version_id );
			if ( ! $veridx_file ) {
				return false;
			}

			$iii = 0;
			$allfiles_cnt = count( $allfiles );
			foreach ( $veridx_file as $date => $id_ary ) {
				for ( ; $iii < $allfiles_cnt; $iii++ ) {
					$filename = $allfiles[$iii]['name'];
					if ( ! is_file( $viewpv_dir . $filename ) ) {
						continue;
					}
					
					if( substr( $filename, 0, 10 ) !== $date ) {
						continue;
					}
					
					$tmpary = unserialize( $this->wrap_get_contents( $viewpv_dir . $filename ) );
					
					$jjj = 0;
					$tmp_cnt = count($tmpary);
					foreach ( $id_ary as $id ) {
						//データは連想配列に入っている
						//取得するデータによって処理をわける
						for ( ; $jjj < $tmp_cnt; $jjj++ ) {
							if ( (int) $tmpary[$jjj]['pv_id'] === (int) $id ) {
								switch ( $columns_ary[0] ) {
									case 'count':
										if ( $columns_ary[1] === '*' ){
											$countall = 1;
										} else {
											if ( $tmpary[$jjj][$columns_ary[1]] ){
												$countall = 1;
											}
										}
										break;

									case '*':
										$results_ary[] = $tmpary[$jjj];
										break;

									default:
										$lineary = [];
										foreach ($columns_ary as $column ) {
											$lineary[$column] = $tmpary[$jjj][$column];
										}
										$results_ary[] = $lineary;
								}

								$jjj++;
								break;
							}
						}
					}

					$iii++;
					break;
				}
			}

		// 必須項目の指定がないのでエラー
		} else {
			$is_error = true;
		}

		//返値をセット
		if ( $is_error ) {
			return false;
		} else {
			if ( $columns_ary[0] === 'count' ) {
				return $countall;
			}else{
				return $results_ary;
			}
		}
	}


	/**
	 * protected
	 */
	protected function get_results_view_page_version_hist( $query = null ) {
		global $qahm_time;

		//view_pv table is in file
		$data_dir = $this->get_data_dir_path();
		$traking_id = $this->get_tracking_id();
		$view_dir = $data_dir . 'view/';
		//view_pv dirlist
		$verhist_dir = $view_dir . $traking_id . '/version_hist/';
		$verhist_idx_dir = $view_dir . $traking_id . '/version_hist/index/';
		$allfiles = $this->wrap_dirlist( $verhist_dir );

		// columns
		preg_match( '/select (.*?) /i', $query, $sel_columns );
		$sel_column = str_replace( ' ', '', $sel_columns[1] );
		$columns_ary = array();
		if ( $sel_column === '*' ) {
			$columns_ary[0] = '*';
		} elseif ( preg_match('/count\((.*)\)/', $sel_column,$counts ) ) {
			$columns_ary[0] = 'count';
			$columns_ary[1] = $counts[1];
		} else {
			$columns_ary = explode( ',', $sel_column );
		}
		$is_error = false;
		$results_ary = [];
		$countall   = 0;
		// where date
		if (preg_match('/where update_date between.*([0-9]{4}.[0-9]{2}.[0-9]{2}.[0-9]{2}.[0-9]{2}.[0-9]{2}).*and.*([0-9]{4}.[0-9]{2}.[0-9]{2}.[0-9]{2}.[0-9]{2}.[0-9]{2})/i', $query, $date_strings )) {
			$s_unixtime = $qahm_time->str_to_unixtime( $date_strings[1] );
			$e_unixtime = $qahm_time->str_to_unixtime( $date_strings[2] );

			foreach ( $allfiles as $file ) {
				$filename = $file['name'];
				if ( is_file( $verhist_dir . $filename ) ) {
					$f_date   = substr( $filename, 0, 10) . ' 00:00:00';

					$f_unixtime = $qahm_time->str_to_unixtime( $f_date );
					if ( $s_unixtime <= $f_unixtime && $f_unixtime <= $e_unixtime ) {
						$tmpary = unserialize( $this->wrap_get_contents( $verhist_dir . $filename ) );
						//データは連想配列に入っている
						//取得するデータによって処理をわける
						switch ( $columns_ary[0] ) {
							case 'count':
								if ( $columns_ary[1] === '*' ){
									$countall = $countall + count( $tmpary );
								} else {
									foreach ( $tmpary as $bodyary ) {
										if ( $bodyary[$columns_ary[1]] ){
											++$countall;
										}
									}
								}
								break;

							case '*':
								$results_ary[] = $tmpary;
								break;

							default:
								foreach ( $tmpary as $bodyary ) {
									$lineary = [];
									foreach ($columns_ary as $column ) {
										$lineary[$column] = $bodyary[$column];
									}
									$results_ary[] = $lineary;
								}
								break;
						}
					}
				}
			}


		// where version_id
		} elseif ( preg_match('/where version_id = ([0-9]*)/i', $query, $veridary ) ){
			$version_id = (int) $veridary[1];
			$filename = $version_id . '_version.php';
			$tmpary = unserialize( $this->wrap_get_contents( $verhist_dir . $filename ) );
			
			// stdClassから連想配列にキャスト変換
			$tmpary = json_decode(json_encode($tmpary), true);

			//データは連想配列に入っている
			//取得するデータによって処理をわける
			for ( $jjj = 0; $jjj < count($tmpary); $jjj++ ) {
				if ( (int) $tmpary[$jjj]['version_id'] === $version_id ) {
					switch ( $columns_ary[0] ) {
						case 'count':
							if ( $columns_ary[1] === '*' ){
								$countall = 1;
							} else {
								if ( $tmpary[$jjj][$columns_ary[1]] ){
									$countall = 1;
								}
							}
							break;

						case '*':
							$results_ary[] = $tmpary[$jjj];
							break;

						default:
							$lineary = [];
							foreach ($columns_ary as $column ) {
								$lineary[$column] = $tmpary[$jjj][$column];
							}
							$results_ary[] = $lineary;
							break;
					}
				}
			}
		} elseif ( preg_match('/where page_id = ([0-9]*)/i', $query, $pageidary ) ){
			$page_id = (int) $pageidary[1];

			$id_ary = $this->get_index_file_contents( $verhist_idx_dir, 'pageid', $page_id );
			if ( ! $id_ary ) {
				return false;
			}

			foreach ( $id_ary as $id ) {
				$filename = $id . '_version.php';
				if ( is_file( $verhist_dir . $filename ) ) {
					$tmpary = unserialize( $this->wrap_get_contents( $verhist_dir . $filename ) );
					
					// stdClassから連想配列にキャスト変換
					$tmpary = json_decode(json_encode($tmpary), true);

					//データは連想配列に入っている
					//取得するデータによって処理をわける
					for ( $jjj = 0; $jjj < count($tmpary); $jjj++ ) {
						if ( (int) $tmpary[$jjj]['page_id'] === (int) $page_id ) {
							switch ( $columns_ary[0] ) {
								case 'count':
									if ( $columns_ary[1] === '*' ){
										$countall = 1;
									} else {
										if ( $tmpary[$jjj][$columns_ary[1]] ){
											$countall = 1;
										}
									}
									break;

								case '*':
									$results_ary[] = $tmpary[$jjj];
									break;

								default:
									$lineary = [];
									foreach ($columns_ary as $column ) {
										$lineary[$column] = $tmpary[$jjj][$column];
									}
									$results_ary[] = $lineary;
									break;
							}
						}
					}
				}
			}
		// 必須項目の指定がないのでエラー
		} else {
			$is_error = true;
		}

		//返値をセット
		if ( $is_error ) {
			return false;
		} else {
			if ( $columns_ary[0] === 'count' ) {
				return $countall;
			}else{
				return $results_ary;
			}
		}
	}

	protected function get_results_days_access( $query = null ) {
		global $qahm_time;
		//view_pv table is in file
		$data_dir = $this->get_data_dir_path();
		$view_dir          = $data_dir . 'view/';
		$traking_id        = $this->get_tracking_id();
		$myview_dir        = $view_dir . $traking_id . '/';
		$summary_dir = $myview_dir . 'summary/';
		$summary_days_access_file = $summary_dir . 'days_access.php';
		global $wp_filesystem;
		// columns
		preg_match( '/select (.*) from/i', $query, $sel_columns );
		$sel_column = str_replace( ' ', '', $sel_columns[1] );
		$columns_ary = [];
		if ( $sel_column === '*' ) {
			$columns_ary[0] = '*';
		} elseif ( preg_match('/count\((.*)\)/', $sel_column,$counts ) ) {
			$columns_ary[0] = 'count';
			$columns_ary[1] = $counts[1];
		} else {
			$columns_ary = explode( ',', $sel_column );
		}
		$is_error = false;
		$results_ary = [];
		$countall   = 0;

        // アクセス一覧を読み込む。存在しなければfalseを返す
		if ( $wp_filesystem->exists( $summary_days_access_file ) ) {
			$days_access_ary = unserialize( $this->wrap_get_contents( $summary_days_access_file ) );
		} else {
			return false;
		}
		// where date
		if (preg_match('/where date between.*([0-9]{4}.[0-9]{2}.[0-9]{2}).*and.*([0-9]{4}.[0-9]{2}.[0-9]{2})/i', $query, $date_strings )) {
			$s_daystr   = $date_strings[1];
			$e_daystr   = $date_strings[2];
			$s_unixtime = $qahm_time->str_to_unixtime( $s_daystr . ' 00:00:00' );
			$e_unixtime = $qahm_time->str_to_unixtime( $e_daystr . ' 23:59:59' );

			$is_loop = true;
			$loopcnt = 0;
			while ( $is_loop ) {
			    $rowdate      = $days_access_ary[$loopcnt]['date'];
			    $row_unixtime = $qahm_time->str_to_unixtime( $rowdate . ' 00:00:00' );

			    if ( $s_unixtime <= $row_unixtime && $row_unixtime <= $e_unixtime) {
                    //データは連想配列に入っている
                    //取得するデータによって処理をわける
                    switch ( $columns_ary[0] ) {
                        case 'count':
                            if ( $columns_ary[1] === '*' ){
                                ++$countall;
                            } else {
                                if ( $days_access_ary[$loopcnt][$columns_ary[1]] ){
                                    ++$countall;
                                }
                            }
                            break;

                        case '*':
                            $results_ary[] = $days_access_ary[$loopcnt];
                            break;

                        default:
                            $lineary = [];
                            foreach ($columns_ary as $column ) {
                                $lineary[$column] = $days_access_ary[$loopcnt][$column];
                            }
                            $results_ary[] = $lineary;
                            break;
                    }

                }
                if ( $e_unixtime < $row_unixtime ) {
                    $is_loop = false;
                }
			    if ( $rowdate === $e_daystr ) {
                    $is_loop = false;
				}
                ++$loopcnt;
			    if ( $loopcnt >= count( $days_access_ary ) ) {
                    $is_loop = false;
				}
            }

		// where pv_id
		// 必須項目の指定がないのでエラー
		} else {
			$is_error = true;
		}

		//返値をセット
		if ( $is_error ) {
			return false;
		} else {
			if ( $columns_ary[0] === 'count' ) {
				return $countall;
			}else{
				return $results_ary;
			}
		}
	}

	private function get_index_file_contents( $dir_path, $id_type, $id_num ) {
		$file_name    = null;
		$search_range = 100000;
		$search_max   = 10000000;
		if ( $id_num > $search_max ) {
			return null;
		}
		for ( $i = 1; $i < $search_max; $i += $search_range ) {
			if ( $i <= $id_num && $i + $search_range > $id_num ) {
				$file_name = $i . '-' . ( $i + $search_range - 1 ) . '_' . $id_type . '.php';
				break;
			}
		}

		if ( ! $file_name || ! $this->wrap_exists( $dir_path . $file_name ) ) {
			return null;
		}

		$file = unserialize( $this->wrap_get_contents( $dir_path . $file_name ) );
		return $file[$id_num];
	}
}
