<?php
/**
 *
 *
 * @package qa_heatmap
 */

class QAHM_File_Base extends QAHM_Base {
	protected function wrap_exists( $path ) {
		global $wp_filesystem;
		return $wp_filesystem->exists( $path );
	}

	/**
	 * ディレクトリの存在チェック＆ディレクトリを作成
	 * ディレクトリが既に存在した場合や新規作成した場合にtrueを返す
	 */
	protected function wrap_mkdir( $path ) {
		global $wp_filesystem;
		if ( $wp_filesystem->exists( $path ) ) {
			return true;
		} else {
			return $wp_filesystem->mkdir( $path );
		}
	}

	protected function wrap_dirlist( $path ) {
		global $wp_filesystem;
		$ret_ary = array();
		if ( is_readable( $path ) ) {

			if ( defined( 'FS_METHOD' ) ) {
				switch ( FS_METHOD ) {
					case 'ftpext':
						$files = $wp_filesystem->dirlist( $path );
						foreach ( $files as $file ) {
							// 「.」「..」以外のファイルを出力
							$lastmodunix = filemtime( $path . $file['name'] );
							$ret_ary[]   = array(
								'name'        => $file['name'],
								'lastmodunix' => $lastmodunix,
							);
						}
						break;

					default:
						// ディレクトリ内のファイルを取得
						$files = scandir( $path );
						foreach ( $files as $file_name ) {
							// 「.」「..」以外のファイルを出力
							if ( ! preg_match( '/^(\.|\.\.)$/', $file_name ) ) {
								$lastmodunix = filemtime( $path . $file_name );
								$ret_ary[]   = array(
									'name'        => $file_name,
									'lastmodunix' => $lastmodunix,
								);
							}
						}
						break;
				}
			} else {
				$files = scandir( $path );
				foreach ( $files as $file_name ) {
					// 「.」「..」以外のファイルを出力
					if ( ! preg_match( '/^(\.|\.\.)$/', $file_name ) ) {
						$lastmodunix = filemtime( $path . $file_name );
						$ret_ary[]   = array(
							'name'        => $file_name,
							'lastmodunix' => $lastmodunix,
						);
					}
				}
			}
		}
		if ( ! empty( $ret_ary ) ) {
			return $ret_ary;
		} else {
			return false;
		}
	}

	/**
	 * wp_remote_getをqahm用にラップした関数
	 * 失敗した場合WP_Errorが返る
	 */
	protected function wrap_remote_get( $url, $dev_name='dsk' ) {
		$bot = QAHM_NAME . 'bot/' . QAHM_PLUGIN_VERSION;

		// デバイスによるユーザーエージェント指定
		switch ( $dev_name ) {
			case 'smp':
				$ua = 'User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 12_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/69.0.3497.91 Mobile/15E148 Safari/605.1' . ' ' . $bot;
				break;
			case 'tab':
				$ua = 'User-Agent: Mozilla/5.0 (iPad; CPU OS 12_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/12.0 Mobile/15E148 Safari/604.1' . ' ' . $bot;
				break;
			default:
				$ua = 'User-Agent: Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11' . ' ' . $bot;
				break;
		}

		$args = array(
			'user-agent' => $ua,
			'timeout'    => 60,
	    	'sslverify'  => false
		);

		return wp_remote_get( $url, $args );
	}

	/**
	 * qahm共通のfile_get_contentsのstream_context_create内で使用するオプションを取得
	 * ここは後々remote_getを使う形に変更
	 */
	protected function get_stream_options( $dev_name ) {
		$bot = QAHM_NAME . 'bot/' . QAHM_PLUGIN_VERSION;

		// デバイスによるユーザーエージェント指定
		switch ( $dev_name ) {
			case 'smp':
				$ua = 'User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 12_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) CriOS/69.0.3497.91 Mobile/15E148 Safari/605.1' . ' ' . $bot;
				break;
			case 'tab':
				$ua = 'User-Agent: Mozilla/5.0 (iPad; CPU OS 12_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/12.0 Mobile/15E148 Safari/604.1' . ' ' . $bot;
				break;
			default:
				$ua = 'User-Agent: Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.79 Safari/535.11' . ' ' . $bot;
				break;
		}

		$options = array(
			'http' => array(
				'method'           => 'GET',
				'header'           => $ua,
				'timeout'          => 10,
				'ignore_errors'    => true,
			),
			'ssl' => array(
				'verify_peer'      => false,
				'verify_peer_name' => false,
			)
		);

		return $options;
	}
	

	/**
	 * rawデータのディレクトリのパスを取得
	 */
	protected function get_raw_dir_path( $type, $id, $dev_name, $tracking_id = null ) {
		// $wp_filesystemオブジェクトの呼び出し
		global $wp_filesystem;

		$dir = $this->get_data_dir_path();
		if ( ! $this->wrap_mkdir( $dir ) ) {
			return false;
		}
		
		// トラッキングIDがnullなら今使用しているWPのトラッキングID
		if ( ! $tracking_id ) {
			$tracking_id = $this->get_tracking_id();
		}
		$dir .= $tracking_id . '/';
		if ( ! $this->wrap_mkdir( $dir ) ) {
			return false;
		}

		if ( $type ) {
			$dir .= $type . '/';
			if ( ! $this->wrap_mkdir( $dir ) ) {
				return false;
			}
		}

		$dir .= $id . '/';
		if ( ! $this->wrap_mkdir( $dir ) ) {
			return false;
		}

		$dir .= 'temp/';
		if ( ! $this->wrap_mkdir( $dir ) ) {
			return false;
		}

		$dir .= $dev_name . '/';
		if ( ! $this->wrap_mkdir( $dir ) ) {
			return false;
		}

		return $dir;
	}

	/**
	 * ディレクトリのURL or パスから要素を求める
	 */
	protected function get_raw_dir_elem( $url ) {
		$url_exp = explode( '/', $url );

		$data_num = null;
		for ( $i = 0; $i < count( $url_exp ); $i++ ) {
			// dataフォルダの位置を求める
			if ( $url_exp[ $i ] === 'data' ) {
				$data_num = $i;
				break;
			}
		}
		if ( $data_num === null || ! isset( $url_exp[ $i + 4 ] ) ) {
			return null;
		}
		if ( ! $url_exp[ $i + 5 ] ) {
			return null;
		}

		$data         = array();
		$data['type'] = $url_exp[ $i + 2 ];
		$data['id']   = $url_exp[ $i + 3 ];
		$data['ver']  = $url_exp[ $i + 4 ];
		$data['dev']  = $url_exp[ $i + 5 ];
		return $data;
	}

	/**
	 * タイプとIDから元URLを取得
	 */
	protected function get_base_url( $type, $id ) {
		switch ( $type ) {
			case 'home':
				return home_url( '/' );
			case 'page_id':
			case 'p':
				return get_permalink( $id );
			case 'cat':
				return get_category_link( $id );
			case 'tag':
				return get_tag_link( $id );
			case 'tax':
				return get_term_link( $id );
			default:
				return null;
		}
	}

	/**
	 * ユーザーエージェントからデバイス名に変換
	 */
	public function user_agent_to_device_name( $ua ) {
		// スマホからのアクセス
		if ( stripos( $ua, 'iphone' ) !== false || // iphone
			stripos( $ua, 'ipod' ) !== false || // ipod
			( stripos( $ua, 'android' ) !== false && stripos( $ua, 'mobile' ) !== false ) || // android
			( stripos( $ua, 'windows' ) !== false && stripos( $ua, 'mobile' ) !== false ) || // windows phone
			( stripos( $ua, 'firefox' ) !== false && stripos( $ua, 'mobile' ) !== false ) || // firefox phone
			( stripos( $ua, 'bb10' ) !== false && stripos( $ua, 'mobile' ) !== false ) || // blackberry 10
			( stripos( $ua, 'blackberry' ) !== false ) // blackberry
			) {
			return 'smp';
		}
		// タブレット
		// mobileという文字が含まれていないAndroid端末はすべてタブレット
		elseif ( stripos( $ua, 'android' ) !== false || stripos( $ua, 'ipad' ) !== false ) {
			return 'tab';
		} else {
			return 'dsk';
		}
	}

	/**
	 * ユーザーエージェントからOS名に変換
	 */
	public function user_agent_to_os_name( $ua ) {
		if (preg_match('/Windows NT 10.0/', $ua)) {
			return 'Windows 10';
		} elseif (preg_match('/Windows NT 6.3/', $ua)) {
			return 'Windows 8.1';
		} elseif (preg_match('/Windows NT 6.2/', $ua)) {
			return 'Windows 8';
		} elseif (preg_match('/Windows NT 6.1/', $ua)) {
			return 'Windows 7';
		} elseif (preg_match('/Mac OS X ([0-9\._]+)/', $ua, $matches)) {
			return 'Mac OS X ' . str_replace('_', '.', $matches[1]);
		} elseif (preg_match('/Linux ([a-z0-9_]+)/', $ua, $matches)) {
			return 'Linux ' . $matches[1];
		} elseif (preg_match('/OS ([a-z0-9_]+)/', $ua, $matches)) {
			return 'iOS ' . str_replace('_', '.', $matches[1]);
		} elseif (preg_match('/Android ([a-z0-9\.]+)/', $ua, $matches)) {
			return 'Android ' . $matches[1];
		} else {
			return 'Unknown';
		}
	}

	/**
	 * ユーザーエージェントからブラウザ名に変換
	 */
	public function user_agent_to_browser_name( $ua ) {
		if (preg_match('/(Iron|Sleipnir|Maxthon|Lunascape|SeaMonkey|Camino|PaleMoon|Waterfox|Cyberfox)\/([0-9\.]+)/', $ua, $matches)) {
			return $matches[1] . $matches[2];
		} elseif (preg_match('/Edge\/([0-9\.]+)/', $ua, $matches)) {
			return 'Edge' . ' ' . $matches[2];
		} elseif (preg_match('/(^Opera|OPR).*\/([0-9\.]+)/', $ua, $matches)) {
			return 'Opera' . ' ' . $matches[2];
		} elseif (preg_match('/Chrome\/([0-9\.]+)/', $ua, $matches)) {
			return 'Chrome' . ' ' . $matches[1];
		} elseif (preg_match('/Firefox\/([0-9\.]+)/', $ua, $matches)) {
			return 'Firefox' . ' ' . $matches[1];
		} elseif (preg_match('/(MSIE\s|Trident.*rv:)([0-9\.]+)/', $ua, $matches)) {
			return 'Internet Explorer' . ' ' . $matches[2];
		} elseif (preg_match('/\/([0-9\.]+)(\sMobile\/[A-Z0-9]{6})?\sSafari/', $ua, $matches)) {
			return 'Safari' . ' ' . $matches[1];
		} else {
			return 'Unknown';
		}
	}

	/**
	 * デバイスIDをデバイス名に変換
	 */
	protected function device_id_to_device_name( $id ) {
		foreach ( QAHM_DEVICES as $qahm_dev ) {
			if ( $qahm_dev['id'] === (int) $id ) {
				return $qahm_dev['name'];
			}
		}

		return false;
	}

	/**
	 * デバイス名をデバイスIDに変換
	 */
	protected function device_name_to_device_id( $name ) {
		foreach ( QAHM_DEVICES as $qahm_dev ) {
			if ( $qahm_dev['name'] === $name ) {
				return $qahm_dev['id'];
			}
		}

		return false;
	}


	/**
	 * $wp_filesystem->put_contentsのラップ関数。ファイル書込用
	 */
	protected function wrap_put_contents( $file, $data) {
		global $wp_filesystem;
		$newstr  = '<?php http_response_code(404);exit; ?>'.PHP_EOL;
		$newstr .= $data;
		return $wp_filesystem->put_contents( $file, $newstr );

	}

	/**
	 * $wp_filesystem->get_contentsのラップ関数。ファイル読み出し用。1行目の 404を抜く
	 */
	function wrap_get_contents( $file ) {
		global $wp_filesystem;
		$string = $wp_filesystem->get_contents( $file );
		if ( $string ) {
			if ( strpos ( $string, 'http_response_code(404)' ) ) {
				return substr(strstr( $string, PHP_EOL ),1);
			} else {
				return $string;
			}
		} else {
			return false;
		}
	}

	/**
	 * $wp_filesystem->get_contents_arrayのラップ関数。ファイル読み出し用。1行目の 404を抜く
	 */
	function wrap_get_contents_array( $file ) {
		global $wp_filesystem;
		$ary    = $wp_filesystem->get_contents_array( $file );
		if ( $ary ) {
			$retary = array();
			$maxcnt = count( $ary );
			if ( strpos( $ary[0],'http_response_code(404)' ) ) {
				$startn = 1;
			} else {
				$startn = 0;
			}
			for ( $iii = $startn; $iii < $maxcnt; $iii++ ) {
				$retary[] = $ary[$iii];
			}
			return $retary;
		} else {
			return false;
		}
	}

	/**
	 * wp_json_encodeのラップ関数。ファイル書込用
	 */
	protected function wrap_json_encode( $data ) {
		return wp_json_encode( $data );
	}


	/**
	 * json_decodeのラップ関数。ファイル読み出し用。1行目の 404を抜く
	 * 簡易的なserialzeデータのチェックも行う
	 * この関数を使用する際は引数$dataの先頭に$aryの中身のいずれかの値を入れないこと
	 */
	function wrap_json_decode( $data ) {
		$str = substr( $data, 0, 2 );
		$ary = array( 'a:', 'b:', 'd:', 'i:', 'O:', 's:' );
		if( in_array( $str, $ary, true ) ) {
			return unserialize( $data );
		}else{
			return json_decode( $data );
		}
	}
}
