<?php
/**
 *
 *
 * @package qa_heatmap
 */

//
// ライセンスに関するクラス
//

$qahm_license = new QAHM_License();

// ライセンスを管理するクラス
class QAHM_License extends QAHM_File_Base {

	const FREE_MEASURE_PAGE = 1;        // 無料ユーザーの最大計測ページ数

	// プラン
	const PLAN = array(
		'free'          => 0,
		'friend'        => 1,
		'personal'      => 2,
		'light'         => 3,
		'business'      => 4,
		'businessplus'  => 5,
		'agent'         => 6,
		'enterprise'    => 7
	);

	// メッセージのレベル
	const MESSAGE_LEVEL = array(
		'success'       => 0,
		'info'          => 1,
		'warning'       => 2,
		'error'         => 3
	);

	// メッセージの通知領域
	const MESSAGE_VIEW = array(
		'admin'         => 0,				// 管理画面全体。こちらを選択した場合はqahmのプラグイン名も先頭に自動付与
		'license'       => 1,				// 管理画面のライセンス認証画面のみ
		'hidden'        => 2				// 非表示
	);

	// valuables
	static public $dom = '';

	public function __construct() {
		add_action( 'admin_notices', array( $this, 'view_message' ) );

		$wp_url    = get_option('home');
		$parse_url = parse_url( $wp_url );
		$ref_host  = $parse_url['host'];
		self::$dom = $ref_host;
		
		add_action( 'init', array( $this, 'init_wp_filesystem' ) );
	}

	/*
	 * ライセンスシステムのメッセージを管理画面に表示
	 */
	public function view_message() {
		$msg_ary  = $this->wrap_get_option( 'license_message' );
		if ( ! $msg_ary ) {
			return;
		}

		$plugin_name = QAHM_PLUGIN_NAME . ' : ';
		foreach ( $msg_ary as &$msg ) {
			$is_view = true;
			switch ( $msg['view'] ) {
				case self::MESSAGE_VIEW['admin']:
					break;

				case self::MESSAGE_VIEW['license']:
					require_once dirname( __FILE__ ) . '/class-qahm-admin-page-license.php';
					$page = filter_input( INPUT_GET, 'page' );
					if ( QAHM_Admin_Page_License::SLUG !== $page ) {
						$is_view = false;
					}
					$plugin_name = '';
					break;
	
				case self::MESSAGE_VIEW['hidden']:
				default:
					$is_view = false;
					break;
			}
			if ( ! $is_view ) {
				continue;
			}

			$class_level = '';
			switch ( $msg['level'] ) {
				case self::MESSAGE_LEVEL['success']:
					if ( self::MESSAGE_VIEW['license'] !== $msg['view'] ) {
						continue 2;
					}
					$class_level = 'notice-success ';
					break;
				case self::MESSAGE_LEVEL['info']:
					$class_level = 'notice-info ';
					break;
				case self::MESSAGE_LEVEL['warning']:
					$class_level = 'notice-warning ';
					break;
				case self::MESSAGE_LEVEL['error']:
					$class_level = 'notice-error ';
					break;
			}
			
			echo '<div class="qahm-license-message notice ' . esc_attr( $class_level ) . 'is-dismissible" data-no="' . $msg['no'] . '">';
			echo '<p>' . esc_html( $plugin_name . $msg['message'] ) .'</p>';
			echo '</div>';
		}
	}

	/**
	 * プラグイン側で設定したいメッセージの出力 & ログの出力
	 * この関数はwp_remote_postで返ってきたjsonの中身（配列）を入れるのではない
	 * 引数によってjsonのメッセージと同じ形式のデータを作りlicese_messageに格納する
	 * この関数により作られたメッセージのメッセージナンバーは空とする。
	 */
	private function set_plugin_message ( $level, $msg, $view, $log = '' ) {
		$msg_ary = array(
			'no'      => '',
			'level'   => $level,
			'message' => $msg,
			'view'    => $view
		);
		$this->wrap_update_option( 'license_message', array( $msg_ary ) );
		
		if ( $log ) {
			global $qahm_log;
			$qahm_log->error( $log );
		}
	}

	/**
	 * jsonで返されたメッセージ配列をプラグイン用に最適化してreturn
	 */
	private function opt_json_message ( $json_msg_ary, $view ) {
		foreach ( $json_msg_ary as &$json_msg ) {
			$json_msg['level'] = self::MESSAGE_LEVEL[ $json_msg['level'] ];
			$json_msg['view']  = $view;
		}
		return $json_msg_ary;
	}
	
	/**
	 * ライセンス認証の通信処理。戻り値はjsonデータ。通信に失敗した場合はfalse
	 */
	private function remote_post ( $url, $args, $view ) {
		$level = self::MESSAGE_LEVEL['error'];
		$msg   = __( 'An error occurred during authentication. Please activate again.', 'qa-heatmap-analytics' );

		$allmix    = wp_remote_post( $url, array( 'body' => $args ) );

		if ( is_wp_error( $allmix ) ) {
			$this->set_plugin_message( $level, $msg, $view, __( 'License authentication error', 'qa-heatmap-analytics' ) . ' : wp_remote_post > is_wp_error' );
			return false;
		}

		$ret_body = wp_remote_retrieve_body( $allmix );
		if ( ! $ret_body ) {
			$this->set_plugin_message( $level, $msg, $view, __( 'License authentication error', 'qa-heatmap-analytics' ) . ' : wp_remote_post > wp_remote_retrieve_body' );
			return false;
		}

		$res_code = wp_remote_retrieve_response_code( $allmix );
		if ( ! $res_code ) {
			$this->set_plugin_message( $level, $msg, $view, __( 'License authentication error', 'qa-heatmap-analytics' ) . ' : wp_remote_post > wp_remote_retrieve_response_code' );
			return false;

		} elseif ( $res_code !== 200 ) {
			$this->set_plugin_message( $level, $msg, $view, __( 'License authentication error', 'qa-heatmap-analytics' ) . ' : http status code ' . $res_code );
			return false;
		}

		$json_array  = json_decode( $ret_body, true );
		if ( $json_array === null ) {
			$this->set_plugin_message( $level, $msg, $view, __( 'License authentication error', 'qa-heatmap-analytics' ) . ' : wp_remote_post > json_decode' );
			return false;
		}

		return $json_array;
	}

	/**
	 * アクティベート（認証）
	 * $viewではエラーの通知領域をMESSAGE_VIEWの中から設定可能
	 */
	public function activate( $key, $uid, $view = self::MESSAGE_VIEW['admin'] ) {
		global $qahm_time;
		$this->wrap_update_option( 'license_activate_time', $qahm_time->now_unixtime() );

		$posturl     = 'https://mem.quarka.org/tsushin/';
		$parm        = array();
		$parm['sec'] = 'license';
		$parm['cmd'] = 'check';
		$parm['ver'] = QAHM_PLUGIN_VERSION;
		$parm['dom'] = self::$dom;
		$parm['uid'] = $uid;
		$parm['key'] = $key;
		
		$json_array = $this->remote_post( $posturl, $parm, $view );
		if ( ! $json_array ) {
			return false;
		}
		if ( QAHM_DEBUG >= QAHM_DEBUG_LEVEL['debug'] ) {
			//print_r( $json_array );
		}

		$is_success =  false;
		if ( $json_array['is_success'] ) {
			$is_success = true;
			$this->change_paid( $json_array['val'], $json_array['bin'] );
			$json_array['msg'][0]['message'] = __( 'License authentication was successful.', 'qa-heatmap-analytics' );
			$json_array['msg'][0]['level']   = 'success';
			$json_array['msg'][0]['no']      = 0;

			if( $view === self::MESSAGE_VIEW['admin'] ) {
				$view = self::MESSAGE_VIEW['hidden'];
			} elseif ( $view === self::MESSAGE_VIEW['license'] ) {
				// ライセンス認証画面の紙吹雪エフェクト用
				$json_array['msg'][0]['confetti'] = true;
			}
		} else {
			$this->change_free();
		}
		
		if ( $json_array['msg'] ) {
			$msg = $this->opt_json_message( $json_array['msg'], $view );
			$this->wrap_update_option( 'license_message', $msg );
		} else {
			$this->wrap_update_option( 'license_message', '' );
		}
		return $is_success;
	}

	// このドメインのライセンスを削除する
	public function deactivate( $key, $uid, $view = self::MESSAGE_VIEW['admin'] ) {
		$posturl     = 'https://mem.quarka.org/tsushin/';
		$parm        = array();
		$parm['sec'] = 'license';
		$parm['cmd'] = 'deactivate';
		$parm['dom'] = self::$dom;
		$parm['uid'] = $uid;
		$parm['key'] = $key;

		$json_array = $this->remote_post( $posturl, $parm, $view );
		if ( ! $json_array ) {
			return false;
		}
		if ( QAHM_DEBUG >= QAHM_DEBUG_LEVEL['debug'] ) {
			//print_r( $json_array );
		}

		$is_success =  false;
		if ( $json_array['is_success'] ) {
			$is_success = true;
			$this->change_free();
			$json_array['msg'][0]['message'] = __( 'The license has been deactivated. All functions enabled by the license have been disabled, and the target of saving the heatmap has been changed to Home.', 'qa-heatmap-analytics' );
			$json_array['msg'][0]['level']   = 'success';
			$json_array['msg'][0]['no']      = 0;

			if( $view === self::MESSAGE_VIEW['admin'] ) {
				$view = self::MESSAGE_VIEW['hidden'];
			}
		}
		
		if ( $json_array['msg'] ) {
			$msg = $this->opt_json_message( $json_array['msg'], $view );
			$this->wrap_update_option( 'license_message', $msg );
		} else {
			$this->wrap_update_option( 'license_message', '' );
		}
		return $is_success;
	}

	// 有償化
	public function change_paid( $val, $bin ) {
		global $wp_filesystem;

		// プランをプラグイン用のIDで格納。これは数値で比較できるようにするため。
		$this->wrap_update_option( 'license_plan', self::PLAN[$val['plan']] );

		$option = null;
		if ( array_key_exists( 'option', $val ) ) {
			$option = $val['option'];
		}

		if ( $val['plan'] === 'free' ) {
			$this->wrap_update_option( 'heatmap_measure_max', 1 );
		} elseif ( $val['plan'] === 'friend' ) {
			$this->wrap_update_option( 'heatmap_measure_max', $option['measure'] );
		} else {
			$this->wrap_update_option( 'heatmap_measure_max', -1 );
		}

		if ( ! $bin ) {
			return;
		}

		// js 作成
		foreach ( $bin as $data ) {
			if ( $data['extension'] === 'js' ) {
				global $wp_filesystem;
				$path = $this->get_data_dir_path() . 'js/';
				if ( ! $wp_filesystem->exists( $path ) ) {
					$wp_filesystem->mkdir( $path );
				}
				
				$path .= $data['name'];
				$body  = base64_decode( $data['body'] );

				global $wp_filesystem;
				$wp_filesystem->put_contents( $path, $body );
			}
		}
	}

	// 無償化
	public function change_free() {
		global $wp_filesystem;
		global $qahm_recterm;

		// rectermのフラグを変更
		$recs = $qahm_recterm->get_all_record();
		foreach ( $recs as &$rec ) {
			if ( $rec['type'] === 'home' ) {
				$rec['rec_flag'] = true;
			} else {
				$rec['rec_flag'] = false;
			}
		}
		$qahm_recterm->bulk_insert( $recs );

		// js 削除
		$dir_path = $this->get_data_dir_path() . 'js/';
		$list = $this->wrap_dirlist( $dir_path );
		if ( $list ) {
			foreach ( $list as $file ) {
				if( strncmp( $file['name'], 'powerup-', strlen( 'powerup-' ) ) === 0 ) {
					$wp_filesystem->delete( $dir_path . $file['name'] );
				}
			}
		}

		// wp_optionsのパラメーターを初期化
		$this->wrap_update_option( 'license_plan', self::PLAN['free'] );
		$this->wrap_update_option( 'data_save_month', 2 );
		$this->wrap_update_option( 'heatmap_measure_max', 1 );
		$this->wrap_update_option( 'is_raw_save_all', false );
		$this->wrap_update_option( 'access_role', 'administrator' );
	}
}
