<?php
/**
 * プラグインで読み込むファイルの管理
 *
 * @package qa_heatmap
 */

new QAHM_Load();

// ファイルのロードや管理画面のメニューを管理するクラス
class QAHM_Load extends QAHM_File_Base {

	public function __construct() {
		// 翻訳ファイルの読み込み
		load_plugin_textdomain( 'qa-heatmap-analytics', false, plugin_basename( dirname( __FILE__ ) ) . '/languages' );

		// 管理画面にプラグインメニューを追加＆CSS, JS読み込み
		add_action( 'admin_menu', array( $this, 'create_plugin_menu' ) );

		// 管理画面以外のCSS, JS読み込み
		add_action( 'wp_enqueue_scripts', array( $this, 'load_user_scripts' ) );
	}

	// 読者用のファイル読み込み
	public function load_user_scripts() {
		if ( $this->is_bot() ) {
			return;
		}

		if ( is_home() || is_front_page() ) {
			$type = 'home';
			$id   = 1;
		} elseif ( is_page() ) {
			$type = 'page_id';
			$id   = get_the_id();
		} elseif ( is_single() ) {
			$type = 'p';
			$id   = get_the_id();
		} elseif ( is_category() ) {
			$type = 'cat';
			$id   = get_query_var( 'cat' );
		} elseif ( is_tag() ) {
			$type = 'tag';
			$id   = get_query_var( 'tag_id' );
		} elseif ( is_tax() ) {
			$type = 'tax';
			$id   = get_queried_object_id();
		} else {
			$type = '';
			$id   = '';
		}

		$js_dir_url = $this->get_js_dir_url();
		$maruyama = defined( 'QAHM_MARUYAMA' ) ? true : false;
		$imai     = defined( 'QAHM_IMAI' ) ? true : false;
		$yuko     = defined( 'QAHM_YUKO' ) ? true : false;

		if ( is_user_logged_in() ) {
			// ログインユーザー アドミンバーの処理が今はないのでコメント中
			// enqueue script
			//wp_enqueue_script( QAHM_NAME . '-common',  $js_dir_url . 'common.js', array( 'jquery' ), QAHM_PLUGIN_VERSION );
			//wp_enqueue_script( QAHM_NAME . '-load-screen',  $js_dir_url . 'load-screen.js', array( QAHM_NAME . '-common' ), QAHM_PLUGIN_VERSION );
			//wp_enqueue_script( QAHM_NAME . '-cap-create',  $js_dir_url . 'cap-create.js', array( QAHM_NAME . '-load-screen' ), QAHM_PLUGIN_VERSION );

			// inline script
			$scripts = array();
			$scripts['ajax_url']          = admin_url( 'admin-ajax.php' );
			$scripts['const_debug_level'] = QAHM_DEBUG_LEVEL;
			$scripts['const_debug']       = QAHM_DEBUG;
			$scripts['license_plan']      = $this->wrap_get_option( 'license_plan' );
			$scripts['type']              = $type;
			$scripts['id']                = $id;
			$scripts['version_id']        = 1;	// あとで変更するかも imai
			$scripts['km']                = $maruyama;
			$scripts['yi']                = $imai;
			$scripts['ym']                = $yuko;
			wp_add_inline_script( QAHM_NAME . '-common', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', obj );', 'before' );

		} else {
			// 一般ユーザー データ測定用
			// enqueue script
			wp_enqueue_script( QAHM_NAME . '-polyfill-object-assign',  $js_dir_url . 'polyfill/object_assign.js', null, QAHM_PLUGIN_VERSION );
			wp_enqueue_script( QAHM_NAME . '-behavioral-data-init',  $js_dir_url . 'behavioral-data-init.js', array( QAHM_NAME . '-polyfill-object-assign' ), QAHM_PLUGIN_VERSION );
			wp_enqueue_script( QAHM_NAME . '-common',  $js_dir_url . 'common.js', array( 'jquery', QAHM_NAME . '-behavioral-data-init' ), QAHM_PLUGIN_VERSION );
			wp_enqueue_script( QAHM_NAME . '-behavioral-data-record',  $js_dir_url . 'behavioral-data-record.js', array( QAHM_NAME . '-common', QAHM_NAME . '-behavioral-data-init' ), QAHM_PLUGIN_VERSION, true );

			// inline script
			$scripts = array();
			$scripts['ajax_url']          = admin_url( 'admin-ajax.php' );
			$scripts['nonce_init']        = wp_create_nonce( QAHM_Behavioral_Data::NONCE_INIT );
			$scripts['nonce_behavioral']  = wp_create_nonce( QAHM_Behavioral_Data::NONCE_BEHAVIORAL );
			$scripts['const_debug_level'] = QAHM_DEBUG_LEVEL;
			$scripts['const_debug']       = QAHM_DEBUG;
			$scripts['type']              = $type;
			$scripts['id']                = $id;
			$scripts['km']          = $maruyama;
			$scripts['yi']              = $imai;
			$scripts['ym']              = $yuko;
			wp_add_inline_script( QAHM_NAME . '-behavioral-data-init', 'var ' . QAHM_NAME . ' = ' . QAHM_NAME . ' || {}; let obj = ' . wp_json_encode( $scripts ) . '; ' . QAHM_NAME . ' = Object.assign( ' . QAHM_NAME . ', obj );', 'before' );
		}
	}

	/**
	 * 管理画面のQAHMメニュー
	 * ここから各管理ページのファイル読み込みフックを呼び出している
	 */
	public function create_plugin_menu() {
		if ( ! is_user_logged_in() || ! is_admin() ) {
			return false;
		}

		global $qahm_admin_page_home;
		global $qahm_admin_page_realtime;
		global $qahm_admin_page_statistics;
		global $qahm_admin_page_heatmap;
		global $qahm_admin_page_config;
		global $qahm_admin_page_license;
		global $qahm_admin_page_help;

		// プラグインメインメニュー
		add_menu_page(
			QAHM_PLUGIN_NAME,
			QAHM_PLUGIN_NAME,
			'manage_options',
			QAHM_Admin_Page_Home::SLUG,
			array( $qahm_admin_page_home, 'create_html' ),
			plugin_dir_url( __FILE__ ) . 'img/quokka-icon.png',
			99
		);

		// ホーム
		$qahm_admin_page_home->hook_suffix = add_submenu_page(
			QAHM_Admin_Page_Home::SLUG,
			__( 'Home', 'qa-heatmap-analytics' ),
			__( 'Home', 'qa-heatmap-analytics' ),
			'manage_options',
			QAHM_Admin_Page_Home::SLUG,
			array( $qahm_admin_page_home, 'create_html' )
		);
		add_action( 'admin_enqueue_scripts', array( $qahm_admin_page_home, 'enqueue_scripts' ) );

		/*
		// リアルタイムビュー（旧ダッシュボード）
		$qahm_admin_page_realtime->hook_suffix = add_submenu_page(
			QAHM_Admin_Page_Home::SLUG,
			__( 'Real-Time View', 'qa-heatmap-analytics' ),
			__( 'Real-Time View', 'qa-heatmap-analytics' ),
			'manage_options',
			QAHM_Admin_Page_Realtime::SLUG,
			array( $qahm_admin_page_realtime, 'create_html' )
		);
		add_action( 'admin_enqueue_scripts', array( $qahm_admin_page_realtime, 'enqueue_scripts' ) );
		
		// サイト統計情報
		$qahm_admin_page_statistics->hook_suffix = add_submenu_page(
			QAHM_Admin_Page_Home::SLUG,
			__( 'Site Statistics', 'qa-heatmap-analytics' ),
			__( 'Site Statistics', 'qa-heatmap-analytics' ),
			'manage_options',
			QAHM_Admin_Page_Statistics::SLUG,
			array( $qahm_admin_page_statistics, 'create_html' )
		);
		add_action( 'admin_enqueue_scripts', array( $qahm_admin_page_statistics, 'enqueue_scripts' ) );
		*/

		// ヒートマップ管理
		$qahm_admin_page_heatmap->hook_suffix = add_submenu_page(
			QAHM_Admin_Page_Home::SLUG,
			__( 'Heatmap Manager', 'qa-heatmap-analytics' ),
			__( 'Heatmap Manager', 'qa-heatmap-analytics' ),
			'manage_options',
			QAHM_Admin_Page_Heatmap::SLUG,
			array( $qahm_admin_page_heatmap, 'create_html' )
		);
		add_action( 'admin_enqueue_scripts', array( $qahm_admin_page_heatmap, 'enqueue_scripts' ) );

		// ライセンス認証
		$qahm_admin_page_license->hook_suffix = add_submenu_page(
			QAHM_Admin_Page_Home::SLUG,
			__( 'Licence Activation', 'qa-heatmap-analytics' ),
			__( 'Licence Activation', 'qa-heatmap-analytics' ),
			'manage_options',
			QAHM_Admin_Page_License::SLUG,
			array( $qahm_admin_page_license, 'create_html' )
		);
		add_action( 'admin_enqueue_scripts', array( $qahm_admin_page_license, 'enqueue_scripts' ) );

		// 設定
		$qahm_admin_page_config->hook_suffix = add_submenu_page(
			QAHM_Admin_Page_Home::SLUG,
			__( 'Settings', 'qa-heatmap-analytics' ),
			__( 'Settings', 'qa-heatmap-analytics' ),
			'manage_options',
			QAHM_Admin_Page_Config::SLUG,
			[$qahm_admin_page_config, 'create_html']
		);
		add_action( 'admin_enqueue_scripts', array( $qahm_admin_page_config, 'enqueue_scripts' ) );

		// ヘルプ
		$qahm_admin_page_help->hook_suffix = add_submenu_page(
			QAHM_Admin_Page_Home::SLUG,
			__( 'Help', 'qa-heatmap-analytics' ),
			__( 'Help', 'qa-heatmap-analytics' ),
			'manage_options',
			QAHM_Admin_Page_Help::SLUG,
			array( $qahm_admin_page_help, 'create_html' )
		);
		add_action( 'admin_enqueue_scripts', array( $qahm_admin_page_help, 'enqueue_scripts' ) );

		if ( defined( 'QAHM_IMAI' ) ) {
			// imai test
			require_once dirname( __FILE__ ) . '/class-qahm-admin-page-imai.php';
			$qahm_admin_page_imai->hook_suffix = add_submenu_page(
				QAHM_Admin_Page_Home::SLUG,
				'imai test',
				'imai test',
				'manage_options',
				QAHM_Admin_Page_Imai::SLUG,
				array( $qahm_admin_page_imai, 'create_html' )
			);
			add_action( 'admin_enqueue_scripts', array( $qahm_admin_page_imai, 'enqueue_scripts' ) );
		}
	}

	public function is_bot() {
		$bot = array(
			'Googlebot',
			'msnbot',
			'bingbot',
			'Yahoo! Slurp',
			'Y!J',
			'facebookexternalhit',
			'Twitterbot',
			'Applebot',
			'Linespider',
			'Baidu',
			'YandexBot',
			'Yeti',
			'dotbot',
			'rogerbot',
			'AhrefsBot',
			'MJ12bot',
			'SMTBot',
			'BLEXBot',
			'linkdexbot',
			'SemrushBot',
			'360Spider',
			'spider',
			'YoudaoBot',
			'DuckDuckGo',
			'Daum',
			'Exabot',
			'SeznamBot',
			'Steeler',
			'Sonic',
			'BUbiNG',
			'Barkrowler',
			'GrapeshotCrawler',
			'MegaIndex.ru',
			'archive.org_bot',
			'TweetmemeBot',
			'PaperLiBot',
			'admantx-apacas',
			'SafeDNSBot',
			'TurnitinBot',
			'proximic',
			'ICC-Crawler',
			'Mappy',
			'YaK',
			'CCBot',
			'Pockey',
			'psbot',
			'Feedly',
			'Superfeedr bot',
			'ltx71',
			'Mail.RU_Bot',
			'Linguee Bot',
			'DuckDuckBot',
			'bidswitchbot',
			'applebot',
			'istellabot',
			'integralads',
			'jet-bot',
			'trendictionbot',
			'blogmuraBot',
			'NetSeer crawler',
			QAHM_NAME . 'bot',
		);

		// 正規表現用に配列を置換
		for ( $i = 0, $bot_len = count( $bot ); $i < $bot_len; $i++ ) {
			$bot[ $i ] = str_replace( '.', '\.', $bot[ $i ] );
			$bot[ $i ] = str_replace( '-', '\-', $bot[ $i ] );
		}

		$ua = filter_input( INPUT_SERVER, 'HTTP_USER_AGENT' );
		//if( ! $ua ) {
		//	$ua = $_SERVER['HTTP_USER_AGENT'];
		//}
		if( $ua ) {
			if ( preg_match( '/' . implode( '|', $bot ) . '/', $ua ) ) {
				return true;
			}
		}
		return false;
	}
}
