<?php
try {
	$version_id        = (int) filter_input( INPUT_GET, 'version_id' );
	$temp_dir          = dirname( __FILE__ ) . '/temp/';
	$wp_load_temp_path = $temp_dir . 'wp-load-path.php';
	$wp_load_rel_path  = file_get_contents( $wp_load_temp_path );
	if ( ! $wp_load_rel_path ) {
		throw new Exception( 'Error loading wp_load.php' );
	}

	require_once $wp_load_rel_path;

	// GETパラメーター判定
	if ( ! $version_id ) {
		throw new Exception( 'Query string has no value.' );
	}

	global $qahm_time;
	global $wp_filesystem;
	$heatmap_view_work_dir = $qahm_view_heatmap->get_data_dir_path( 'heatmap-view-work' );
	$heatmap_view_work_url = $qahm_view_heatmap->get_heatmap_view_work_dir_url();
	$file_info             = $heatmap_view_work_dir . $version_id . '-info.php';
	if ( ! $wp_filesystem->exists( $file_info ) ) {
		throw new Exception( 'heatmap view info file does not exist.' );
	}
	$content_info_ary = $wp_filesystem->get_contents_array( $file_info );


	// info ファイル読み込み
	foreach ( $content_info_ary as $content_info ) {
		$exp_info = explode( '=', $content_info );
		switch ( $exp_info[0] ) {
			case 'data_num':
				$data_num = (int) trim( $exp_info[1] );
				break;
			case 'wp_qa_type':
				$wp_qa_type = trim( $exp_info[1] );
				break;
			case 'wp_qa_id':
				$wp_qa_id = (int) trim( $exp_info[1] );
				break;
			case 'version_no':
				$version_no = (int) trim( $exp_info[1] );
				break;
			case 'device_name':
				$device_name = trim( $exp_info[1] );
				break;
			case 'time_on_page':
				$time_on_page = (float) trim( $exp_info[1] );
				$time_on_page = $qahm_time->seconds_to_timestr( $time_on_page );
				$time_on_page = substr( $time_on_page, strlen('00:') );
				break;
		}
	}
	$is_heatmap = false;
	if ( $data_num > 0 ) {
		$is_heatmap = true;
	}


	// ログイン判定
	if ( ! is_user_logged_in() ) {
		throw new Exception( __( 'Please try to log in your WordpPress.' ) );
	}
	$user   = wp_get_current_user();
	$roles  = (array) $user->roles;
	$access = get_option( QAHM_OPTION_PREFIX . 'access_role', 'administrator' );

	switch ( $roles[0] ) {
		case 'administrator':
			if ( ! ( 'administrator' === $access || 'editor' === $access ) ) {
				throw new Exception( __( 'You do not have access privileges.' ) );
			}
			break;
		case 'editor':
			if ( ! ( 'editor' === $access ) ) {
				throw new Exception( __( 'You do not have access privileges.' ) );
			}
			break;
		default:
			throw new Exception( __( 'You do not have access privileges.' ) );
	}

	// 翻訳ファイルの読み込み
	load_plugin_textdomain( 'qa-heatmap-analytics', false, plugin_basename( dirname( __FILE__ ) ) . '/languages' );
	
	$ajax_url       = admin_url( 'admin-ajax.php' );
	$plugin_dir_url = plugin_dir_url( __FILE__ );

	// データ数
	$data_num_title   = __( 'Amount of Data', 'qa-heatmap-analytics' );
	$data_num_tooltip = esc_attr__( 'The amount of data recorded on this page to develop the heatmap. The value will be close to the number of PV, but it will not be recorded in the case such as it bounces in a few seconds.', 'qa-heatmap-analytics' );
	$data_num_icon    = '<i class="fas fa-users"></i>';

/*
	$text_data_num = esc_html__( 'Number of data', 'qa-heatmap-analytics' );
	$text_data_num = '<i class="fas fa-users"></i> ' . $text_data_num . ': ' . $data_num;
	$text_data_num = '<span class="qahm-tooltip-bottom" data-qahm-tooltip="' . $qahm_view_heatmap->qa_langesc_attr__( 'このページでヒートマップデータを記録した数です。PV数に近い値になりますが、数秒で直帰した場合などは記録されません。', 'qa-heatmap-analytics' ) . '">' . $text_data_num . '</span>';
*/
	$html_bar        = '<ul>';
	$html_bar_mobile = '<ul>';
	
	if ( $is_heatmap ) {
		
		// スクロールマップ
		$scro_map_title   = __( 'Scroll Map', 'qa-heatmap-analytics' );
		$scro_map_tooltip = esc_attr__( 'Indicates "whether this page was read all the way down." As you place (and move) the mouse cursor on Scroll Map, it tells you the number of users who were staying to watch on that horizontal line of the page.', 'qa-heatmap-analytics' );
//		$scro_map_tooltip = esc_attr__( 'Indicates "whether this page was read all the way down." The top is viewed by 100% of users, and generally the number of users decreases as it goes down.', 'qa-heatmap-analytics' );
		$scro_map_icon    = '<img src="' . $qahm_view_heatmap->get_img_dir_url() . 'scroll-map.svg">';

		// アテンションマップ
		$atte_map_title   = __( 'Attention Map', 'qa-heatmap-analytics' );
		$atte_map_tooltip = esc_attr__( 'Indicates "what content the users are interested in." Frequently read parts are displayed in red like a heat map.', 'qa-heatmap-analytics' );
		$atte_map_icon    = '<img src="' . $qahm_view_heatmap->get_img_dir_url() . 'attention-map.svg">';

		// クリックヒートマップ
		$heat_map_title   = __( 'Click Heatmap', 'qa-heatmap-analytics' );
		$heat_map_tooltip = esc_attr__( 'Indicates "where the users are clicking." The place a user clicks is likely to be the point of interest. The parts that have been clicked a lot are displayed in red.', 'qa-heatmap-analytics' );
		$heat_map_icon    = '<img src="' . $qahm_view_heatmap->get_img_dir_url() . 'click-heat-map.svg">';

		// クリックカウントマップ		
		$count_map_title   = __( 'Click Count Map', 'qa-heatmap-analytics' );
		$count_map_tooltip = esc_attr__( 'Indicates "how many times this button or link has been pressed." You can see trends such as banner clicks.', 'qa-heatmap-analytics' );
		$count_map_icon    = '<img src="' . $qahm_view_heatmap->get_img_dir_url() . 'click-count-map.svg">';

		// 平均滞在
		$time_on_page_title   = __( 'Average Time on Page', 'qa-heatmap-analytics' );
		$time_on_page_tooltip = esc_attr__( 'Shows the average amount of time spent on the page per person, derived from the heatmap data.', 'qa-heatmap-analytics' );
		$time_on_page_icon    = '<i class="fas fa-user-clock"></i>';

		// 「この位置まで注目」
		$on_this_line_watched = __( 'On this line of page watched by:', 'qa-heatmap-analytics' );
		$on_this_line_watched_ninzu = _x( 'people', 'counting number (unit) of people', 'qa-heatmap-analytics' );

		// html構築
		$html_bar .= $qahm_view_heatmap->get_html_bar_checkbox( 'qahm-bar-read', $scro_map_icon . $scro_map_title, $scro_map_tooltip, false, false );
		$html_bar .= $qahm_view_heatmap->get_html_bar_checkbox( 'qahm-bar-attention', $atte_map_icon . $atte_map_title, $atte_map_tooltip, true, false );
		$html_bar .= $qahm_view_heatmap->get_html_bar_checkbox( 'qahm-bar-heatmap', $heat_map_icon . $heat_map_title, $heat_map_tooltip, true, false );
		$html_bar .= $qahm_view_heatmap->get_html_bar_checkbox( 'qahm-bar-click', $count_map_icon . $count_map_title, $count_map_tooltip, false, false );
		$html_bar .= $qahm_view_heatmap->get_html_bar_text( 'qahm-bar-data-num', $data_num_icon . $data_num_title . ': ' . $data_num, $data_num_tooltip );
		$html_bar .= $qahm_view_heatmap->get_html_bar_text( 'qahm-bar-avg-time-on-page', $time_on_page_icon . $time_on_page_title . ': ' . $time_on_page, $time_on_page_tooltip );
		
		$html_bar_mobile .= $qahm_view_heatmap->get_html_bar_checkbox( 'qahm-bar-read', $scro_map_icon, $scro_map_tooltip, false, true );
		$html_bar_mobile .= $qahm_view_heatmap->get_html_bar_checkbox( 'qahm-bar-attention', $atte_map_icon, $atte_map_tooltip, true, true );
		$html_bar_mobile .= $qahm_view_heatmap->get_html_bar_checkbox( 'qahm-bar-heatmap', $heat_map_icon, $heat_map_tooltip, true, true );
		$html_bar_mobile .= $qahm_view_heatmap->get_html_bar_checkbox( 'qahm-bar-click', $count_map_icon, $count_map_tooltip, false, true );
		$html_bar_mobile .= $qahm_view_heatmap->get_html_bar_text( 'qahm-bar-data-num', $data_num_icon . $data_num, $data_num_tooltip, true );
		$html_bar_mobile .= $qahm_view_heatmap->get_html_bar_text( 'qahm-bar-avg-time-on-page', $time_on_page_icon . $time_on_page, $time_on_page_tooltip );

	} else {
		$html_bar        .= $qahm_view_heatmap->get_html_bar_text( 'qahm-bar-data-num', $data_num_icon . $data_num_title . $data_num, $data_num_tooltip );
		$html_bar_mobile .= $qahm_view_heatmap->get_html_bar_text( 'qahm-bar-data-num', $data_num_icon . $data_num, $data_num_tooltip );
	}
	
	$html_bar        .= '</ul>';
	$html_bar_mobile .= '</ul>';
	$plugin_version   = QAHM_PLUGIN_VERSION;

} catch ( Exception $e ) {
	echo '<!DOCTYPE html><html><head><meta charset="utf-8"></head><body>';
	echo '<p>Error : ' . esc_html( $e->getMessage() ) . '</p>';
	echo '<p><a href="' . esc_url( admin_url( 'admin.php?page=qahm-help' ) ) . '" target="_blank">' . esc_html__( 'HELP' ) . '</a></p>';
	echo '</body></html>';
	exit();
}
?>

<!DOCTYPE html>
<html lang="<?php echo esc_attr( get_locale() ); ?>">
	<head>
		<meta charset="utf-8">
		<meta http-equiv="X-UA-Compatible" content="IE=edge">
		<meta name="viewport" content="width=device-width, initial-scale=1.0">

		<title>QA Heatmap View</title>

		<link rel="stylesheet" type="text/css" href="./css/doctor-reset.css?ver=<?php esc_attr_e( $plugin_version ); ?>">
		<link rel="stylesheet" type="text/css" href="./css/common.css?ver=<?php esc_attr_e( $plugin_version ); ?>">
		<link rel="stylesheet" type="text/css" href="./css/heatmap-view.css?ver=<?php esc_attr_e( $plugin_version ); ?>">

		<script src="./js/lib/jquery/jquery-3.6.0.min.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		<script src="./js/lib/sweet-alert-2/sweetalert2.min.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		<script src="./js/alert-message.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		<script src="./js/lib/font-awesome/all.min.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		
		<script>
			var qahm = qahm || {};
			let obj = {
				'ajax_url':'<?php echo $ajax_url; ?>',
				'type':'<?php echo $wp_qa_type; ?>',
				'id':'<?php echo $wp_qa_id; ?>',
				'ver':'<?php echo $version_no; ?>',
				'dev':'<?php echo $device_name; ?>',
				'version_id':'<?php echo $version_id; ?>',
			};
			qahm = Object.assign( qahm, obj );
		</script>
		<script src="./js/common.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		<script src="./js/load-screen.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		<script src="./js/cap-create.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		<script src="./js/lib/heatmap/heatmap.min.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		<script src="./js/heatmap-view.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		<script src="./js/heatmap-bar.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		<script src="./js/heatmap-main.js?ver=<?php esc_attr_e( $plugin_version ); ?>"></script>
		
		<script>
			jQuery(function(){
				jQuery( '#heatmap' ).on( 'load', function() {
					jQuery( '#heatmap' ).contents().find('head').append(
						'<link rel="stylesheet" href="<?php echo esc_url( plugin_dir_url( __FILE__ ) ); ?>css/heatmap-frame.css?ver=<?php echo esc_attr( QAHM_PLUGIN_VERSION ); ?>" type="text/css" />'
					);
				});
			});
		</script>
	</head>
	<body>
		<?php
			$html = <<< EOT
				<div id="qahm-container">
					<div id="qahm-loading-container" class="qahm-fade">
						<div id="qahm-loading"></div>
					</div>
					<div id="qahm-progress-container" class="qahm-fade">
						<div id="qahm-progress-back">
							<div id="qahm-progress-bar"></div>
						</div>
						<div id="qahm-progress-info">
							<p id="qahm-progress-text"></p>
							<p id="qahm-progress-percent"></p>
						</div>

					</div>
					<div id="qahm-bar">
						<div id="qahm-bar-inner">
							<nav id="qahm-nav">
								{$html_bar}
							</nav>
							<nav id="qahm-mobile-nav">
								{$html_bar_mobile}
							</nav>
						</div>
					</div>
				
					<div id="qahm-scroll-tooltip">{$on_this_line_watched}<br><span id="qahm-scroll-data-num"></span> {$on_this_line_watched_ninzu}</div>

				</div>
				<div id="frame">
					<iframe id="heatmap" src="{$heatmap_view_work_url}{$version_id}-cap.php" width="100%" height="100%"></iframe>
				</div>
EOT;
			echo $html;
		?>
	</body>
</html>
