var qahm = qahm || {};

// QAHMのテーブルのtypeをヒートマップ管理画面用に整形
qahm.convTableType = function( type ) {
	switch ( type ) {
		case 'home':		return qahml10n['home'];
		case 'p':			return qahml10n['post'];
		case 'page_id':		return qahml10n['page'];
		case 'cat':			return qahml10n['cat'];
		case 'tag':			return qahml10n['tag'];
		case 'tax':			return qahml10n['tax'];
		default:			return false;
	}
};

// 文字列省略。全角半角対応
qahm.omitStr = function( text, len, truncation ) {
	if (truncation === undefined) {
		truncation = ''; }
	let text_array = text.split( '' );
	let count      = 0;
	let str        = '';
	for (i = 0; i < text_array.length; i++) {
		let n = escape( text_array[i] );
		if (n.length < 4) {
			count++;
		} else {
			count += 2;
		}
		if (count > len) {
			return str + truncation;
		}
		str += text.charAt( i );
	}
	return text;
};

// POST送信にパラメーターを追加
qahm.appendPostData = function( name, value ) {
	let ele = document.createElement( 'input' );
	ele.setAttribute( 'type', 'hidden' );
	ele.setAttribute( 'name', name );
	ele.setAttribute( 'value', value );
	document.dataForm.appendChild( ele );
};

// POST送信にパラメーターを追加（全体）
qahm.postParam = function() {
	let list_num     = qahm.list.length;
	let list_rec_num = 0;
	qahm.appendPostData( 'list_num', list_num );
	for ( let i = 0;  i < list_num;  i++  ) {
		if ( qahm.list[i]['measure'] ) {
			list_rec_num++;
		}
		if ( ! qahm.list[i]['change'] ) {
			continue;
		}
		qahm.appendPostData( 'ids_' + i, qahm.list[i]['id'] );
		qahm.appendPostData( 'type_' + i, qahm.list[i]['type'] );
		qahm.appendPostData( 'rec_flag_' + i, qahm.list[i]['measure'] );
	}
	qahm.appendPostData( 'list_rec_num', list_rec_num );
};

qahm.getDeviceNumHtml = function( ver_info ) {
	let html = '';
	
	let devNotExistsStyle = 'style="opacity: 0.3; margin-right: 8px;"';
	if ( ver_info && ver_info['data_dsk'] > 0 ){
		html += '<a target="_blank" class="qahm-heatmap-link" data-device_name="dsk" data-version_id="' + ver_info['version_id_dsk'] + '">';
		html += '<span class="dashicons dashicons-desktop"></span>';
		html += '<span>(' + ver_info['data_dsk'] + ')</span>';
		html += '</a>';
	} else {
		html += '<span ' + devNotExistsStyle + '>';
		html += '<span class="dashicons dashicons-desktop"></span>';
		html += '<span>(0)</span>';
		html += '</span>';
	}
	
	if ( ver_info && ver_info['data_tab'] > 0 ){
		html += '<a target="_blank" class="qahm-heatmap-link" data-device_name="tab" data-version_id="' + ver_info['version_id_tab'] + '">';
		html += '<span class="dashicons dashicons-tablet"></span>';
		html += '<span>(' + ver_info['data_tab'] + ')</span>';
		html += '</a>';
	'</a>';
	} else {
		html += '<span ' + devNotExistsStyle + '>';
		html += '<span class="dashicons dashicons-tablet"></span>';
		html += '<span>(0)</span>';
		html += '</span>';
	}
	
	if ( ver_info && ver_info['data_smp'] > 0 ){
		html += '<a target="_blank" class="qahm-heatmap-link" data-device_name="smp" data-version_id="' + ver_info['version_id_smp'] + '">';
		html += '<span class="dashicons dashicons-smartphone"></span>';
		html += '<span>(' + ver_info['data_smp'] + ')</span>';
		html += '</a>';
	} else {
		html += '<span ' + devNotExistsStyle + '>';
		html += '<span class="dashicons dashicons-smartphone"></span>';
		html += '<span>(0)</span>';
		html += '</span>';
	}

	return html;
}

// ヒートマップ管理状況のリストを作成
qahm.createList = function() {
	if ( ! document.querySelector( '#qahm-data-list' ) ) {
		return;
	}

	// リスト構築
	let sort_title    = jQuery( '#qahm-data-sort-title' ).val();
	let sort_rec      = jQuery( '#qahm-data-sort-rec' ).val();
	let sort_view     = jQuery( '#qahm-data-sort-view' ).val();

	jQuery.Deferred(
		function(d) {
			qahm.setProgressBar( 10, 'create list' );

			if ( qahm.list ) {
				d.resolve();
			} else {
				jQuery.ajax(
					{
						type: 'POST',
						url: qahm.ajax_url,
						dataType : 'json',
						data: {
							'action' : 'qahm_ajax_create_heatmap_list',
						},
					}
				).done(
					function( list ){
						qahm.list = list;
						
						// freeプランの場合、計測チェック数によってメッセージを表示
						if ( qahm.license_plan === qahm.const_license_plan['free'] ) {
							if ( qahm.is_first_heatmap_setting ) {
								jQuery( '#qahm-measure-comment-container' ).show();
							} else {
								for ( let i = 0;  i < qahm.list.length;  i++  ) {
									if ( qahm.list[i]['measure'] ) {
										jQuery( '#qahm-measure-comment-text' ).html( qahml10n['measure_comment_text'] );
										jQuery( '#qahm-measure-comment-container, #qahm-measure-comment-button' ).show();
										break;
									}
								}
							}
						}

						qahm.setProgressBar( 30, 'sort list' );

						// データ数順にソート
						qahm.list.sort(
							function( a, b ) {
								if ( a['version_info'] === undefined ) {
									return 1;
								} else if ( b['version_info'] === undefined ) {
									return -1;
								}
								return( a['version_info'][a['version_index']]['data_all'] < b['version_info'][b['version_index']]['data_all'] ? 1: -1 );
							}
						);

						d.resolve();
					}
				).fail(
					function( jqXHR, textStatus, errorThrown ){
						qahm.log_ajax_error( jqXHR, textStatus, errorThrown );
						d.reject();
					}
				);
			}
		}
	)
	.then(
		function(){
			let d = new jQuery.Deferred();

			// 検索件数
			if ( sort_view === 'all' ) {
				sort_view = qahm.list.length;
			} else {
				sort_view = parseInt( sort_view );
				// if( sort_view > qahm.list.length ){ sort_view = qahm.list.length; }
			}

			let sort_find = 0;
			let html = [];
			const data_len = qahm.list.length;
			for ( let i = 0; i < data_len; i++ ) {
				if ( sort_title ) {
					// タイトルのチェック
					if ( qahm.list[i]['title'].toLowerCase().indexOf( sort_title.toLowerCase() ) === -1 ) {
						continue;
					}
				}
				if ( sort_rec === 'true' && ! qahm.list[i]['measure'] ) {
					continue;
				} else if ( sort_rec === 'false' && qahm.list[i]['measure'] ) {
					continue;
				}

				let title        = qahm.omitStr( qahm.list[i]['title'], 40, '…' );
				let urlObj       = new URL( qahm.list[i]['url'] );
				let slug         = decodeURIComponent( qahm.list[i]['url'].replace( urlObj.origin, '' ) );
				if ( ! slug ) {
					slug = '/';
				}
				let rec_disabled = '';
				let rec_flag     = '';
				let ver_idx      = qahm.list[i]['version_index'];
				let ver_info     = null;
				if ( ver_idx !== null ) {
					ver_info = qahm.list[i]['version_info'][ver_idx];
				}

				// 全URL計測状態のときは特殊処理
				if ( qahm.is_raw_save_all ) {
					rec_disabled = ' disabled';
					rec_color    = ' style="color: #aaa;"';
					//rec_flag     = ' checked';
				}
				if ( qahm.list[i]['measure'] !== 0 ) {
					rec_flag = ' checked';
				}

				// タイトル
				html[sort_find] = '<tr>';
				html[sort_find] += '<td>';
				html[sort_find] += '<a href="' + qahm.list[i]['url'] + '" target="_blank">' + title + '</a>';
				if ( slug !== undefined ) {
					html[sort_find] += '<br>' + '<span style="font-size:70%;">' + slug + '</span>';
				}
				html[sort_find] += '</td>';

				// データ数
				html[sort_find] += '<td id="qahm-data-num-' + i + '">';
				html[sort_find] += qahm.getDeviceNumHtml( ver_info );
				html[sort_find] += '</td>';
				
				// バージョン：データ期間
				html[sort_find] += '<td>';
				if ( ver_info ) {
					html[sort_find] += '<select class="qahm-version-select" data-list_index="' + i + '">';
					for ( let j = ver_idx ; j > 0 ; j-- ) {
						let info = qahm.list[i]['version_info'][j];
						if ( info === undefined ) {
							break;
						}
						html[sort_find] += '<option value="' + j + '">';
						html[sort_find] += qahml10n['ver'] + j + ' : ' + info['period_from'] + ' - ' + info['period_to'];
						html[sort_find] += '</option>';
					}
					html[sort_find] += '</select>';
				}
				html[sort_find] += '</td>';

				// バージョン切替
				html[sort_find] += '<td>';
				if ( qahm.list[i]['version_info'] ) {
					let ref_data = ' data-list_index="' + i + '"';
					ref_data += ' data-page_id="' + qahm.list[i]['page_id'] + '"';
					ref_data += ' data-wp_qa_type="' + qahm.list[i]['type'] + '"';
					ref_data += ' data-wp_qa_id="' + qahm.list[i]['id'] + '"';
					ref_data += ' data-url="' + qahm.list[i]['url'] + '"';
					ref_data += ' data-title="' + qahm.list[i]['title'] + '"';
					html[sort_find] += '<span id="qahm-refresh-icon-' + i + '" class="qahm-refresh-icon"' + ref_data + ' style="cursor: pointer;"><i class="fas fa-sync"></i></span>';
					
					html[sort_find] += '<span id="qahm-refresh-text-' + qahm.list[i]['page_id'] + '" style="color: #ff5353; margin-left: 6px;">';
					if ( qahm.list[i]['version_refresh'] ) {
						html[sort_find] += 'ver.' + qahm.list[i]['version_refresh'] + qahml10n['switch'];
					}
					html[sort_find] += '</span>';
				}
				html[sort_find] += '</td>';

				// タイプ
				html[sort_find] += '<td>' + qahm.convTableType( qahm.list[i]['type'] ) + '</td>';
				
				// 計測チェック
				html[sort_find] += '<td><input type="checkbox" id="qahm-rec-flag-' + i + '" class="qahm-rec-flag" ' + rec_flag + rec_disabled + '></td>';
				
				html[sort_find] += '</tr>';

				sort_find++;
				if ( sort_find >= sort_view ) {
					break;
				}
			}

			// 作成したhtmlを分割
			// htmlの追加が環境によっては一番処理時間がかかるので、
			// htmlは分割して挿入してプログレスバーで進捗を更新している
			const html_split = 5;
			let html_append  = [];
			const html_len   = html.length;
			const split_line = html_len / html_split;

			for ( let i = 0; i < html_len; i++ ) {
				let idx = Math.floor( i / split_line );
				if ( html_append[idx] ) {
					html_append[idx] += html[i];
				} else {
					html_append[idx] = html[i];
				}
			}

			// リストの追加
			jQuery( '#qahm-data-list > tbody' ).empty();
			let prom = jQuery.Deferred().resolve().promise();
			for (let i = 0; i < html_split; i++) {
				prom = prom.then(
					(function() { return function() {
						let d_list = jQuery.Deferred();
						jQuery( '#qahm-data-list > tbody' ).append( html_append[i] );
						qahm.setProgressBar(
							40 + (i + 1) * (Math.floor( 50 / html_split )),
							'append list',
							function(){
								d_list.resolve();
								if ( i === html_split - 1 ) {
									d.resolve();
								}
							}
						);
						return d_list.promise();
					};})()
				);
			}

			return d.promise();
		}
	)
	.then(
		function() {
			let d = new jQuery.Deferred();

			qahm.setProgressBar( 95, 'load achievements' );

			// 実績判定用 全てのデータ数取得
			qahm.dataNum = 0;
			for ( let listIdx = 0; listIdx < qahm.list.length; listIdx++ ) {
				if ( ! qahm.list[listIdx]['version_info'] ) {
					continue;
				}

				let verCur = parseInt( qahm.list[listIdx]['version_index'] );
				for ( let verIdx = verCur ; verIdx > 0 ; verIdx-- ) {
					let info = qahm.list[listIdx]['version_info'][verIdx];
					if ( info === undefined ) {
						break;
					}
					qahm.dataNum += parseInt( info['data_all'] );
				}

				/*
				let verKeys = Object.keys( qahm.list[listIdx]['version_info'] );
				let verLen = verKeys.length;
				for ( let verIdx = 1; verIdx <= verLen; verIdx++ ) {
					qahm.dataNum += parseInt( qahm.list[listIdx]['version_info'][verIdx]['data_all'] );
				}
				*/
			}

			d.resolve();
			return d.promise();
		}
	)
	.then(
		function() {
			qahm.setProgressBar( 100 );

			// 実績の更新
			if ( qahm.achievements === '' ) {
				qahm.achievements = new Object();
			}
			if ( qahm.achievements['data_num_1'] === undefined ) {
				checkAchievementsDataNum( 1 );
			} else if ( qahm.achievements['data_num_1000'] === undefined ) {
				checkAchievementsDataNum( 1000 );
			} else if ( qahm.achievements['data_num_10000'] === undefined ) {
				checkAchievementsDataNum( 10000 );
			} else if ( qahm.achievements['data_num_100000'] === undefined ) {
				checkAchievementsDataNum( 100000 );
			} else if ( qahm.achievements['data_num_1000000'] === undefined ) {
				checkAchievementsDataNum( 1000000 );
			}

			function checkAchievementsDataNum( targetDataNum ) {
				if ( qahm.dataNum < targetDataNum ) {
					return;
				}

				let title, text;
				if ( targetDataNum === 1 ) {
					title = qahml10n['achievements_data_num_1_title'];
					text = qahml10n['achievements_data_num_1_text'];
				}
				else if( targetDataNum >= 1000 ) {
					title = qahml10n['achievements_data_num_1000_title'];
					text  = qahm.sprintf( qahml10n['achievements_data_num_1000_text'], targetDataNum );
				} else {
					return;
				}

				qahm.achievements['data_num_' + targetDataNum] = true;

				jQuery.ajax(
					{
						type: 'POST',
						url: qahm.ajax_url,
						dataType : 'json',
						data: {
							'action' : 'qahm_ajax_set_achievements',
							'achievements' : JSON.stringify( qahm.achievements ),
						},
					}
				).done(
					function(){
						qahm.startConfetti();
						AlertMessage.alert(
							title,
							text,
							'success',
							function(){
								qahm.stopConfetti();
							}
						);
					}
				).fail(
					function( jqXHR, textStatus, errorThrown ){
						qahm.log_ajax_error( jqXHR, textStatus, errorThrown );
					}
				);
			}
		}
	);
};

qahm.loadScreen.promise()
.then(
	function() {
		// バージョン切り替え
		jQuery( document ).on(
			'change',
			'.qahm-version-select',
			function(){
				let listIdx    = jQuery(this).data('list_index');
				let newVerIdx = jQuery(this).val();
				let oldVerIdx = qahm.list[listIdx]['version_index'];
				qahm.list[listIdx]['version_index'] = newVerIdx;
				let verInfo = qahm.list[listIdx]['version_info'];
				let verIdAry = [
					verInfo[newVerIdx]['version_id_dsk'],
					verInfo[newVerIdx]['version_id_tab'],
					verInfo[newVerIdx]['version_id_smp'],
				];

				let getDataNumDef = new jQuery.Deferred();
				if ( verInfo[newVerIdx]['data_all'] === 0 ) {
					qahm.showLoadIcon();
		
					jQuery.ajax(
						{
							type: 'POST',
							url: qahm.ajax_url,
							dataType : 'json',
							data: {
								'action' : 'qahm_ajax_get_data_num',
								'version_id_ary': JSON.stringify( verIdAry ),
							},
						}
					).done(
						function( dataNumAry ){
							qahm.list[listIdx]['version_info'][newVerIdx]['data_all'] = dataNumAry[0] + dataNumAry[1] + dataNumAry[2];
							qahm.list[listIdx]['version_info'][newVerIdx]['data_dsk'] = dataNumAry[0];
							qahm.list[listIdx]['version_info'][newVerIdx]['data_tab'] = dataNumAry[1];
							qahm.list[listIdx]['version_info'][newVerIdx]['data_smp'] = dataNumAry[2];

							let html = qahm.getDeviceNumHtml( qahm.list[listIdx]['version_info'][newVerIdx] );
							jQuery( '#qahm-data-num-' + listIdx ).html( html );
							getDataNumDef.resolve();
						}
					).fail(
						function( jqXHR, textStatus, errorThrown ){
							qahm.log_ajax_error( jqXHR, textStatus, errorThrown );
							getDataNumDef.reject();
						}
					).always(
						function(){
							qahm.hideLoadIcon();
						}
					);
				} else {
					getDataNumDef.resolve();
				}
				
				getDataNumDef.promise()
				.then(
					function() {
						let html = qahm.getDeviceNumHtml( qahm.list[listIdx]['version_info'][newVerIdx] );
						jQuery( '#qahm-data-num-' + listIdx ).html( html );
					}
				);
			}
		);

		// デバイスリンククリック
		jQuery( document ).on(
			'click',
			'.qahm-heatmap-link',
			function(){
				qahm.createCap( jQuery( this ).data( 'version_id' ), jQuery( this ).data( 'device_name' ), null, null, true );
			}
		);

		// 計測フラグ変更
		jQuery( document ).on(
			'change',
			'.qahm-rec-flag',
			function(){
				let id                   = jQuery( this ).attr( 'id' );
				id                       = id.replace( /[^0-9]/g, '' );
				let this_rec             = qahm.list[id]['measure'];
				qahm.list[id]['measure'] = jQuery( this ).prop( 'checked' ) ? 1 : 0;

				// 無料ユーザーなら1ページまでのチェック制限
				if ( qahm.list[id]['measure'] &&
						qahm.license_plan <= qahm.const_license_plan['friend'] ) {

					let cnt = 0;
					for ( let i = 0, list_len = qahm.list.length;  i < list_len;  i++  ) {
						if ( qahm.list[i]['measure'] ) {
							cnt++;
						}
					}
					
					if ( qahm.license_plan <= qahm.const_license_plan['friend'] && cnt > qahm.heatmap_measure_max ) {
						qahm.list[id]['measure'] = 0;
						jQuery( this ).removeAttr( 'checked' ).prop( 'checked', false );

						let title = qahml10n['limit_measure_title'];
						let text  = qahml10n['limit_measure_text'];
						if ( qahm.const_license_plan['free'] === qahm.license_plan ) {
							text = qahm.sprintf( text, 1 ) + qahml10n['limit_free_text'];
						} else if ( qahm.const_license_plan['friend'] === qahm.license_plan ) {
							text = qahm.sprintf( text, qahm.heatmap_measure_max ) + qahml10n['limit_free_text'];
						}
						AlertMessage.alert( title, text, 'warning' );
					}
				}

				qahm.log( qahm.list[id]['measure'] );
				qahm.list[id]['change'] = true;
			}
		);

		
		// 計測フラグ変更
		jQuery( document ).on(
			'change',
			'#qahm-all-rec-flag',
			function(){
				// フリー、フレンドライセンスなら警告
				if ( qahm.license_plan <= qahm.const_license_plan['friend'] ) {
					jQuery( this ).removeAttr( 'checked' ).prop( 'checked', false );
					
					let title = qahml10n['limit_measure_title'];
					let text  = qahml10n['limit_measure_text'];
					if ( qahm.const_license_plan['free'] === qahm.license_plan ) {
						text = qahm.sprintf( text, 1 ) + qahml10n['limit_free_text'];
					} else if ( qahm.const_license_plan['friend'] === qahm.license_plan ) {
						text = qahm.sprintf( text, qahm.heatmap_measure_max ) + qahml10n['limit_free_text'];
					}

					AlertMessage.alert( title, text, 'warning' );
				} else {
					jQuery( this ).prop( 'disabled', true );
					qahm.showLoadIcon();
	
					// パーソナル以上のライセンスであればフラグ変更ののち再読み込み
					jQuery.ajax(
						{
							type: 'POST',
							url: qahm.ajax_url,
							dataType : 'json',
							data: {
								'action' : 'qahm_ajax_change_is_raw_save_all'
							},
						}
					).done(
						function(){
							window.location.reload();
						}
					).fail(
						function( jqXHR, textStatus, errorThrown ){
							qahm.log_ajax_error( jqXHR, textStatus, errorThrown );
						}
					);
				}
			}
		);

		// バージョン切り替え
		jQuery( document ).on(
			'click',
			'.qahm-refresh-icon',
			function(){

				let list_index = jQuery(this).data('list_index');
				let page_id    = jQuery(this).data('page_id');
				let wp_qa_type = jQuery(this).data('wp_qa_type');
				let wp_qa_id   = jQuery(this).data('wp_qa_id');
				let url        = jQuery(this).data('url');
				let title      = jQuery(this).data('title');

				let start_text = '<div id="swal2-content" class="swal2-html-container" style="display: block;">';
				start_text += qahml10n['version_switch_start_text_1'];
				start_text += '<br><span style="font-size:80%;color:gray">';
				start_text += qahml10n['version_switch_start_text_2'];
				start_text += '</span></div>';
				
				AlertMessage.confirm(
					qahml10n['version_switch'],
					start_text,
					'info',
					function ( _enter ) {
						if ( _enter ) {
							qahm.showLoadIcon();
							jQuery.ajax(
								{
									type: 'POST',
									url: qahm.ajax_url,
									dataType : 'json',
									data: {
										'action'     : 'qahm_ajax_refresh_version',
										'nonce'      : qahm.nonce_refresh,
										'page_id'    : page_id,
										'wp_qa_type' : wp_qa_type,
										'wp_qa_id'   : wp_qa_id,
										'url'        : url,
										'title'      : title
									},
								}
							).done(
								function( data ){
									if ( data['result'] ) {
										AlertMessage.alert(
											qahml10n['version_switch'],
											data['version_switch_success_text'],
											'success'
											);
										
										qahm.list[list_index]['version_refresh'] = data['version'];
										jQuery( '#qahm-refresh-text-' + page_id ).text( 'ver.' + data['version'] + qahml10n['switch'] );

									} else {
										AlertMessage.alert(
											qahml10n['version_switch'],
											qahml10n['version_switch_error_text'],
											'error'
											);
									}
									qahm.hideLoadIcon();
								}
							).fail(
								function( jqXHR, textStatus, errorThrown ){
									qahm.log_ajax_error( jqXHR, textStatus, errorThrown );
									AlertMessage.alert(
										qahml10n['version_switch'],
										qahml10n['version_switch_error_text'],
										'error'
										);
									qahm.hideLoadIcon();
								}
							);
						}
					}
				);
			}
		);

		qahm.createList();
	}
);

