var qahm = qahm || {};

qahm.docHeight        = 0;
qahm.isScrollMapEvent = false;

qahm.initHeatmap = function() {
	qahm.docHeight = Math.max.apply(
		null,
		[qahm.iframeDoc.body.clientHeight,
		qahm.iframeDoc.body.scrollHeight,
		qahm.iframeDoc.documentElement.scrollHeight,
		qahm.iframeDoc.documentElement.clientHeight]
	);
};

// 配列の指定パーセントの位置（インデックス）に対応した値を返す関数
qahm.getArrayRateValue = function( rateAry, rateVal ) {
	if ( rateVal <= 0 ) {
		return Math.min.apply( null, rateAry );
	} else if ( rateVal >= 100 ) {
		return Math.max.apply( null, rateAry );
	}

	// 降順に並んだ配列に位置する要素のインデックス
	idx = Math.floor( ( rateAry.length * rateVal / 100 ) - 0.01 );
	return rateAry[idx];
}

qahm.getSiblingElemetsIndex = function( el, name ) {
	let index = 1;
	let sib   = el;

	while ( ( sib = sib.previousElementSibling ) ) {
		if ( sib.nodeName.toLowerCase() === name ) {
			++index;
		}
	}

	return index;
}

qahm.getSelectorFromElement = function( el ) {
	let names = [];
	if ( ! ( el instanceof Element ) ) {
		return names; }

	while ( el.nodeType === Node.ELEMENT_NODE ) {
		let name = el.nodeName.toLowerCase();
		if ( el.id ) {
			// id はページ内で一意となるため、これ以上の検索は不要
			name += '#' + el.id;
			names.unshift( name );
			break;
		}

		// 同じ階層に同名要素が複数ある場合は識別のためインデックスを付与する
		// 複数要素の先頭 ( index = 1 ) の場合、インデックスは省略可能
		//
		let index = qahm.getSiblingElemetsIndex( el, name );
		if ( 1 < index ) {
			name += ':nth-of-type(' + index + ')';
		}

		names.unshift( name );
		el = el.parentNode;
	}

	return names;
}

// ヒートマップ
qahm.createClickHeatMap = function(){
	if ( qahm.iframeDoc.getElementById( 'qahm-heatmap' ) === null || ! qahm.mergeC ) {
		return;
	}

	// サイトの高さをもとに分割数を求める
	const px_split = 8192;		// IEのcanvas最大値
	const px_last  = Math.floor( qahm.docHeight % px_split );
	let h_split    = Math.floor( qahm.docHeight / px_split ) + 1;

	// qahm-heatmap以下を削除して再構築
	let tagHeatmap = '';
	for (let i = 0; i < h_split; i++) {
		tagHeatmap += '<div id="qahm-heatmap' + i + '"></div>';
	}
	tagHeatmap = jQuery( tagHeatmap );
	jQuery( '#qahm-heatmap', qahm.iframeDoc ).empty().append( tagHeatmap );

	// DOM追加後に処理を進める
	tagHeatmap.ready(
		function() {
			// ヒートマップを展開する座標配列の設定
			// オフセットを用いているため↓の高さ設定前に設定
			let points = [];
			for (let i = 0; i < qahm.mergeC.length; i++) {
				// タグからの相対座標を求める
				let x = qahm.mergeC[i][qahm.DATA_HEATMAP_SELECTOR_X];
				let y = qahm.mergeC[i][qahm.DATA_HEATMAP_SELECTOR_Y];

				let offs = jQuery( qahm.mergeC[i][qahm.DATA_HEATMAP_SELECTOR_NAME], qahm.iframeDoc ).offset();
				if ( offs === undefined ) {
					continue;
				}
				x = Math.floor( x + offs.left );
				y = Math.floor( y + offs.top );

				// どの高さ配列に格納するか求める
				let h_idx = Math.floor( y / px_split );

				let point = {
					x: x,
					y: y,
					value: 1
				};
				// qahm.log( 'heatmap_pos:' + point.x + ' ,' + point.y, true );
				points.push( { 'index': h_idx, 'point': point } );
			}

			for (let i = 0; i < h_split; i++) {
				let h_points = [];
				for (let j = 0; j < points.length; j++) {
					if ( points[j]['index'] !== i ) {
						continue;
					}
					points[j]['point'].y -= px_split * i;
					h_points.push( points[j]['point'] );
				}

				const h_data = {
					max: 2,
					data: h_points
				};

				// #qahm-heatmap以下にDOMを追加
				const h_dom  = i === h_split - 1 ? px_last : px_split;
				const dom_id = 'qahm-heatmap' + i;
				const dom    = jQuery( '<div id="' + dom_id + '" height="' + h_dom + '"></div>' );
				// jQuery('#qahm-heatmap').append( dom );
				// ヒートマップの高さを設定（使用する前提条件）
				jQuery( '#' + dom_id, qahm.iframeDoc ).css( 'top', px_split * i );
				jQuery( '#' + dom_id, qahm.iframeDoc ).css( 'height', h_dom );

				// dom.ready(function() {
				// create instance
				let heatmapInstance = h337.create(
					{
						container: qahm.iframeDoc.querySelector( '#' + dom_id ),
						radius: 40
					}
				);
				heatmapInstance.setData( h_data );

				// setDataが終わったあとにpositionを変更
				// こうしないと駄目
				jQuery( '#' + dom_id, qahm.iframeDoc ).css( 'position', 'absolute' );
				// });
			}

		}
	);
};

// スクロールマップ
qahm.createScrollMap = function(){
	if ( qahm.iframeDoc.getElementById( 'qahm-scroll' ) === null || ! qahm.mergeAS ) {
		return;
	}

	// カラーコード。スクロールマップ用の10色+黒
	let colorCode = ["#9e0142","#d6424b","#f47d4d","#fdbe70","#feeda1","#f0f8a9","#bee5a0","#77c6a7","#438fb4","#5e4fa2","#000000"];
	let opacity   = 0.7;
	let rateAry   = [];

	// 離脱ユーザーのパーセント位置から実際の高さに変換
	for ( let i = 0; i < qahm.mergeAS.length; i++ ) {
		let exitUserNum = qahm.mergeAS[i][qahm.DATA_ATTENTION_SCROLL_EXIT_USER];
		if( 0 === exitUserNum ){
			continue;
		}
		for ( let j = 0; j < exitUserNum; j++ ) {
			rateAry.push( qahm.docHeight * ( qahm.mergeAS[i][qahm.DATA_ATTENTION_SCROLL_PERCENT] / 100 ) );
		}
	}

	let html         = '';
	let scrollMapAry = [];
	let nowDataNum   = rateAry.length;

	// 精読率100%ライン
	scrollMapAry.push( {
		'heightTop'   : 0,
		'heightBottom': qahm.getArrayRateValue( rateAry, 10 ),
		'colorTop'    : colorCode[0],
		'colorBottom' : colorCode[1],
		'line'        : 100
	} );
	
	for ( let i = 1;  i <= 9;  i++ ) {
		let prevScrollMap = scrollMapAry[scrollMapAry.length - 1];

		let heightTop     = prevScrollMap['heightBottom'];
		let heightBottom  = qahm.getArrayRateValue( rateAry, (i+1)*10 );
		if ( heightTop === heightBottom ) {
			continue;
		}

		let colorTop      = prevScrollMap['colorBottom'];
		let colorBottom   = colorCode[i];
		
		scrollMapAry.push( {
			'heightTop'   : heightTop,
			'heightBottom': heightBottom,
			'colorTop'    : colorTop,
			'colorBottom' : colorBottom,
			'line'        : 100 - i*10
		} );
	}

	// 精読率0%ライン
	if ( scrollMapAry[scrollMapAry.length - 1]['heightBottom'] !== qahm.docHeight ) {
		scrollMapAry[scrollMapAry.length - 1]['colorBottom'] = colorCode[10];
		scrollMapAry.push( {
			'heightTop'   : scrollMapAry[scrollMapAry.length - 1]['heightBottom'],
			'heightBottom': qahm.docHeight,
			'colorTop'    : scrollMapAry[scrollMapAry.length - 1]['colorBottom'],
			'colorBottom' : colorCode[10],
			'line'        : 0
		} );
	}

	for ( let i = 0;  i < scrollMapAry.length;  i++ ) {
		let styleHeight     = 'height:' + ( scrollMapAry[i]['heightBottom'] - scrollMapAry[i]['heightTop'] ) + 'px;';
		let styleBackground = 'background: linear-gradient( ' + scrollMapAry[i]['colorTop'] + ', ' + scrollMapAry[i]['colorBottom'] + ' );';
		let styleOpacity    = 'opacity: ' + opacity + ';';
		let styleColor      = 'color:#fff;';
		let styleTextshadow = 'text-shadow: black 1px 1px 1px, black -1px 1px 1px, black 1px -1px 1px, black -1px -1px 1px;';
		let scrollVal       = scrollMapAry[i]['line'] + '%';
		html += '<div style="' + styleHeight + ' ' + styleBackground + ' ' + styleOpacity + ' ' + styleColor + ' ' + styleTextshadow + '">' + scrollVal + '</div>';
	}
	jQuery( '#qahm-scroll', qahm.iframeDoc ).html( html );


	// jQueryの多重登録や配列の再定義を防ぐため、フラグを設けている
	if ( ! qahm.isScrollMapEvent ) {
		qahm.isScrollMapEvent = true;
		qahm.scrollStayNum    = 0;

		// 滞在人数を100分割した地点で求める
		qahm.stayPerAry = [];
		nowDataNum = rateAry.length;
		qahm.stayPerAry[0] = nowDataNum;

		// qahm.mergeAS.length - 1 しているのは最後（画面一番下）を対象外にしたいがため
		for ( let i = 0; i < qahm.mergeAS.length - 1; i++ ) {
			let exitUserNum = qahm.mergeAS[i][qahm.DATA_ATTENTION_SCROLL_EXIT_USER];
			nowDataNum -= exitUserNum;
			qahm.stayPerAry[i] = nowDataNum;
		}

		jQuery( '#qahm-scroll', qahm.iframeDoc ).hover( function(e){
			jQuery( '#qahm-scroll-tooltip' ).fadeIn( 200 );
		},function(){
			jQuery( '#qahm-scroll-tooltip' ).fadeOut( 200 );
		});

		// iframe内のマウスカーソルの位置を記録
		jQuery( '#qahm-scroll', qahm.iframeDoc ).on('mousemove', function(e) {
			let offsY = jQuery( '#qahm-bar' ).height();
			let mouseX = e.clientX + 12;
			let mouseY = e.clientY + offsY;

			console.log( qahm.iframeDoc.documentElement.scrollHeight );
			console.log( );

			// マウスカーソルが下に行きすぎたときの対策
			//const mouseYLimit = window.document.body.clientHeight - 80;
			const mouseYLimit = qahm.iframeDoc.documentElement.clientHeight + offsY - 80;
			if ( mouseY > mouseYLimit  ) {
				mouseY = mouseYLimit;
			}

			// 吹き出しの位置座標を取得したマウスカーソルの座標に変換
			jQuery( '#qahm-scroll-tooltip' ).css({
				'left': mouseX,
				'top' : mouseY
			});

			let stayIdx = Math.floor( e.pageY / qahm.docHeight * 100 );
			if ( qahm.scrollStayNum !== qahm.stayPerAry[stayIdx] ) {
				qahm.scrollStayNum = qahm.stayPerAry[stayIdx];
				jQuery( '#qahm-scroll-data-num' ).text( qahm.scrollStayNum );
			}
		});

		// スクロールマップのチェックボックスにチェックを入れたとき、ツールチップを表示
		jQuery( '.qahm-bar-read' ).on( 'click', function() {
			if ( jQuery( this ).prop( 'checked' ) ) {
				let offsX = jQuery( '#qahm-scroll', qahm.iframeDoc ).width() - 20;
				let offsY = jQuery( '#qahm-bar' ).height() + 20;
				jQuery( '#qahm-scroll-tooltip' ).css({
					'left': offsX,
					'top' : offsY
				});
				
				let scrollTop = jQuery( 'html', qahm.iframeDoc ).scrollTop();
				let stayIdx = Math.floor( ( scrollTop + 20 ) / qahm.docHeight * 100 );
				qahm.scrollStayNum = qahm.stayPerAry[stayIdx];
				jQuery( '#qahm-scroll-data-num' ).text( qahm.scrollStayNum );
				jQuery( '#qahm-scroll-tooltip' ).fadeIn( 200 );
			} else {
				jQuery( '#qahm-scroll-tooltip' ).hide();
			}
		});
	}
};

// アテンションマップ
qahm.createAttentionMap = function(){
	if ( qahm.iframeDoc.getElementById( 'qahm-attention' ) === null || ! qahm.mergeAS ) {
		return;
	}

	// let colorCode = ["#440154","#481b6d","#46327e","#3f4788","#365c8d","#2e6e8e","#277f8e","#21918c","#1fa187","#2db27d","#4ac16d","#73d056","#a0da39","#d0e11c","#fde725"];
	let colorCode      = ["#5e4fa2","#4478b2","#4ba0b1","#72c3a7","#a0d9a3","#ccea9f","#ebf7a6","#fbf8b0","#fee89a","#fdca79","#fba35e","#f3784c","#e1524a","#c42c4a","#9e0142"];
	let color_num       = colorCode.length;
	let limit_time      = 7;
	let color_round_coe = color_num / limit_time;
	let opacity         = 0.7;

	// 分割した範囲に滞在した秒数を計測
	let split_max    = 100;
	let split_num    = 50;								// 50分割
	let split_per    = split_max / split_num / 100;		// 何分割するかはサイトの縦幅によって自動判定予定
	let height_split = qahm.docHeight * split_per;		// 高さを分割
	let height_range = height_split * 0.5;				// 範囲は半分

	let total_avg_time   = 0;
	let total_user_num   = qahm.dataNum;

	// 分割地点ごとの滞在時間を格納する配列（全ユーザーの平均滞在時間）	+1しているのはhtml最下部のライン用
	let height_avg_time  = (new Array( split_num + 1 )).fill( 0 );

	// 分割地点ごとの滞在時間の数
	let height_avg_num   = (new Array( split_num + 1 )).fill( 0 );

	let data_len = qahm.mergeAS.length;
	for (let i = 0; i < data_len; i++ ) {
		if( split_max === qahm.mergeAS[i][qahm.DATA_ATTENTION_SCROLL_PERCENT] ){
			continue;
		}
		let idx = Math.floor( qahm.mergeAS[i][qahm.DATA_ATTENTION_SCROLL_PERCENT] / ( split_max / split_num ) );

		height_avg_time[ idx ] += qahm.mergeAS[i][qahm.DATA_ATTENTION_SCROLL_STAY_TIME];
		total_avg_time += qahm.mergeAS[i][qahm.DATA_ATTENTION_SCROLL_STAY_TIME];
		height_avg_num[ idx ]++;
	}

	// 全ユーザーの平均滞在時間を求める
	total_avg_time /= total_user_num;
	qahm.log( 'ページ平均滞在時間：' + total_avg_time );

	// colorCodeの添え字にするため、分割地点ごとの滞在時間を求めて四捨五入する
	let height_avg_time_len = height_avg_time.length;
	for (let i = 0; i < height_avg_time_len; i++ ) {
		if( height_avg_num[ i ] > 0 ){
			height_avg_time[ i ] /= height_avg_num[ i ];
			height_avg_time[ i ] = Math.round( height_avg_time[i] );
			qahm.log( '分割地点' + i + 'の平均滞在時間：' + height_avg_time[i] + '秒' );
			height_avg_time[ i ] = Math.min( height_avg_time[ i ], limit_time );
		}
	}

	let html     = '';
	let colorTop  = '#000000';
	let colorBottom  = '#000000';
	let heightTop = 0;
	let heightBottom = 0;

	// 例えば5%区切りの場合、2.5%±2.5%の範囲に色付け
	// 実際はグラデーションをかけるのでラインのひき方は0%, 2.5%, 7.5%, 12.5%…といったようにひいていく
	for ( let i = 0;  i <= split_num; i = (i + 1) | 0 ) {
		heightTop = heightBottom;
		heightBottom = i * height_split + height_range;
		if ( heightBottom > split_num * height_split ) {
			heightBottom = split_num * height_split;
		}
		colorTop = colorBottom;

		// colorCodeの配列参照先は四捨五入して求める
		let color_index = Math.round( height_avg_time[i] * color_round_coe );
		if( color_index >=  color_num ){
			color_index = color_num - 1;
		}
		colorBottom = colorCode[color_index];

		// html += '<div style="height:' + (heightBottom-heightTop) + 'px; background: linear-gradient( ' + colorTop + ', ' + colorBottom + ' ); opacity: ' + opacity + '; color: #fff;">座標：' + heightTop + 'px ～ ' + heightBottom + 'px, 色:' + colorTop + ' ～ ' + colorBottom + ', 平均滞在時間:' + height_avg_time[i] + '秒</div>';
		html += '<div style="height:' + (heightBottom - heightTop) + 'px; background: linear-gradient( ' + colorTop + ', ' + colorBottom + ' ); opacity: ' + opacity + '; color: #fff;"></div>';
	}

	jQuery( '#qahm-attention', qahm.iframeDoc ).html( html );
	// });
};


// クリックマップ
qahm.createClickCountMap = function(){
	if ( qahm.iframeDoc.getElementById( 'qahm-click' ) === null || ! qahm.mergeC ) {
		return;
	}
	
	// サイトの幅と高さ
	const w_site = qahm.iframeDoc.body.clientWidth;

	// サイトの高さをもとに分割数を求める
	const px_split = 8192;		// IEのcanvas最大値
	const px_last  = Math.floor( qahm.docHeight % px_split );
	let h_split    = Math.floor( qahm.docHeight / px_split ) + 1;

	// qahm-click以下を削除して再構築
	let qahm_dom = '';
	for (let i = 0; i < h_split; i++) {
		const h_dom = i === h_split - 1 ? px_last : px_split;
		qahm_dom   += '<div id="qahm-click' + i + '" style="top:' + px_split * i + 'px; height:' + h_dom + 'px; position: absolute; line-height: 0px;">' +
				'<canvas width="' + w_site + '" height="' + h_dom + '" style="position: absolute; left: 0px; top: 0px;">' +
				'</canvas></div>';
	}
	qahm_dom = jQuery( qahm_dom );
	jQuery( '#qahm-click', qahm.iframeDoc ).empty().append( qahm_dom );

	// DOM追加後に処理を進める
	qahm_dom.ready(
		function() {

			const CLICK_SEL_NUM   = 0;
			const CLICK_SEL_NAME  = 1;
			const CLICK_SEL_X     = 2;
			const CLICK_SEL_Y     = 3;
			const CLICK_SEL_W     = 4;
			const CLICK_SEL_H     = 5;
			const CLICK_CANVAS_ID = 6;

			// ヒートマップを展開する座標配列の設定
			// オフセットを用いているため↓の高さ設定前に設定
			let points = [];
			let target_tag_ary   = ['a','input','button','textarea'];
			for (let i = 0, d_len = qahm.mergeC.length; i < d_len; i++ ) {
				let find = false;
				let sel_name = qahm.mergeC[i][qahm.DATA_HEATMAP_SELECTOR_NAME];
				let sel_tag_ary = sel_name.split('>');

				for (let j = 0, s_len = sel_tag_ary.length; j < s_len; j++ ) {
					for (let k = 0, t_len = target_tag_ary.length; k < t_len; k++ ) {
						if( sel_tag_ary[j].indexOf( target_tag_ary[k] ) !== 0 ){
							continue;
						}

						if( sel_tag_ary[j].length === target_tag_ary[k].length ||
							sel_tag_ary[j].indexOf( target_tag_ary[k] + '#' ) === 0 ||
							sel_tag_ary[j].indexOf( target_tag_ary[k] + ':' ) === 0 ) {
							find = true;
							break;
						}
					}
					if( find ){
						break;
					}
				}
				if ( ! find ) {
					continue;
				}

				let p_idx = -1;
				let p_len = points.length;
				if ( p_len > 0 ) {
					for (let j = 0; j < p_len; j++ ) {
						if ( qahm.mergeC[i][qahm.DATA_HEATMAP_SELECTOR_NAME] === points[j][CLICK_SEL_NAME] ) {
							p_idx = j;
							break;
						}
					}
				}
				if ( p_idx === -1 ) {
					let sel = jQuery( qahm.mergeC[i][qahm.DATA_HEATMAP_SELECTOR_NAME], qahm.iframeDoc );
					if ( sel.offset() === undefined || sel.offset().top < 0 ) {
						continue;
					}

					const canvas_base_y = Math.floor( sel.offset().top / px_split );
					const canvas_id     = '#qahm-click' + canvas_base_y + ' > canvas';

					qahm.log( qahm.mergeC[i][qahm.DATA_HEATMAP_SELECTOR_NAME] )
					points.push(
						[
						1,
						qahm.mergeC[i][qahm.DATA_HEATMAP_SELECTOR_NAME],
						sel.offset().left,
						sel.offset().top - canvas_base_y * px_split,
						sel.outerWidth(),
						sel.outerHeight(),
						canvas_id,
						]
					);
				} else {
					points[p_idx][CLICK_SEL_NUM]++;
				}
			}

			// qahm.log( points );

			// 描画
			for (let i = 0, p_len = points.length; i < p_len; i++ ) {
				let ctx = qahm.iframeDoc.querySelector( points[i][CLICK_CANVAS_ID] ).getContext( '2d' );
				// 四角形（輪郭）
				ctx.beginPath();
				ctx.lineWidth   = 5;
				ctx.strokeStyle = 'rgba(' + [255, 200, 200, 1.0] + ')';
				ctx.strokeRect(
					points[i][CLICK_SEL_X],
					points[i][CLICK_SEL_Y],
					points[i][CLICK_SEL_W],
					points[i][CLICK_SEL_H]
				);

						ctx.beginPath();
				ctx.fillStyle = 'rgba(' + [0, 0, 0, 0.8] + ')';
				ctx.fillRect(
					points[i][CLICK_SEL_X],
					points[i][CLICK_SEL_Y],
					points[i][CLICK_SEL_W],
					points[i][CLICK_SEL_H]
				);

				// 数字
				ctx.beginPath();
				ctx.font         = '22px bold serif';
				ctx.fillStyle    = 'white';
				ctx.textAlign    = 'center';
				ctx.textBaseline = 'middle';
				ctx.fillText(
					points[i][CLICK_SEL_NUM],
					points[i][CLICK_SEL_X] + points[i][CLICK_SEL_W] / 2,
					points[i][CLICK_SEL_Y] + points[i][CLICK_SEL_H] / 2
				);
			}
		}
	);
};

// ウインドウをリサイズした時の処理
qahm.resizeWindow = function(){
	let timer = false;
	jQuery( window ).resize(
		function() {
			// ヒートマップ用のタグを一時的に削除して再作成
			qahm.showLoadIcon();
			jQuery( '#qahm-heatmap' ).empty();
			jQuery( '#qahm-heatmap' ).css( 'height', '' );
			jQuery( '#qahm-heatmap' ).css( 'position', '' );
			jQuery( '#qahm-scroll' ).empty();
			jQuery( '#qahm-attention' ).empty();
			jQuery( '#qahm-click' ).empty();
			if (timer !== false) {
				clearTimeout( timer );
			}
			timer = setTimeout(
				function() {
					qahm.createClickHeatMap();
					qahm.createScrollMap();
					qahm.createAttentionMap();
					qahm.createClickCountMap();
					qahm.hideLoadIcon();
				},
				300
			);
		}
	);
}